#!/usr/local/bin/perl -w

use bytes;

require 5.003;
use Socket;

# origin: http://home.hiroshima-u.ac.jp/u1079009/files/
# hacked by james lots
# read perldoc perlipc, signal handling and socketing

# ΥץȤϡTCPΥݡȤ listen ޤ
# connect ³ɽǡɽޤ
# ǥեȤϥݡ10007ХåʤǤ

# ˡ: listener.pl [-e] [-v] [-h] [-l logfile] [port]
# ץ:	-e: 饤Ȥ˥Хå

# եλꡣˤȥե¸ʤ
#$logfile = 'listener.log';

# MS-DOS  Windows ƥȥ⡼ɤȥХʥ⡼ɤOSξ1ˤƤ
$windows = 0;

my $usage = <<END;
tcp/ip socket server listen & log script useful for testing

listener.pl [-p port] [-e] [-v] [-h] [-l logfile] [-q cmd] [-Q cmd] [-r cmd txt | -R cmd file] [port]
  -p port    port to bind listener to for incoming connections
  -e         echo bytes received back to sending socket
  -v         verbose some extra connection/debug info
  -h         show received bytes in format addr: hexdump ascii instead of plain text
  -l file    log all bytes received
  -q cmd     QUIT connection (if listener receives cmd it closes connection)
  -Q cmd     QUIT listening (if listener receives cmd it closes connection and exits)
  -w n       wait and process n connections, then quit
  -r cmd txt if listener receives cmd it replies with txt 
             only one instance of -r or -R is permitted
  -R cmd file if listener receives cmd it replies with contents of file
             only one instance of -r or -R is permitted

END

=head1 NAME

listener.pl - tcp/ip socket server listen & log script useful for testing

=head1 USAGE

$usage

=head1 SYNOPSIS

e.g. #0

 #serveri  ./listener.pl -q GET -Q GET -r GET "<html><head><title>T</title></head><body>B</body></html>" -l ../log/test-email2sms-listener.log

 #clienti  wget --proxy=off http://localhost:10007/something

 #clienti  cat ../log/test-email2sms-listener.log
 #client GET /something HTTP/1.0^M

e.g. #1

  #serveri ~/bin/listener.pl -v 7777
  #server  port:7777 ... socket ... setsockopt ... bind ... listen ... accept ... select ... CONNECTED.
  #server  Connected. addr: 192.168.1.1 host: by.dev.ie.alphyra.com port: 41370
  #server  Connection from by.dev.ie.alphyra.com [192.168.1.1] at port 41370
  #server  hello
  #server  accept ... 

  #clienti telnet by 7777
  #client  Trying 192.168.1.1...
  #client  Connected to by.
  #client  Escape character is '^]'.
  #clienti hello
  #clienti ^]
  #client  telnet> 
  #clienti quit
  #client  Connection closed.

e.g. #2

  #serveri ~/bin/listener.pl -v 7777 -q QUIT -Q EXIT
  #server  port:7777 ... socket ... setsockopt ... bind ... listen ... accept ... select ... CONNECTED.
  #server  Connected. addr: 192.168.1.1 host: by port: 41370
  #server  Connection from by [192.168.1.1] at port 41370
  #server  hello
  #server  QUIT
  #server  accept ... 

  #clienti telnet by 7777
  #client  Trying 192.168.1.1...
  #client  Connected to by.
  #client  Escape character is '^]'.
  #clienti hello
  #clienti QUIT
  #client  Connection closed.

e.g. #3

  ./listener.pl -v -l test-o2xmlrpc-listener.log -R "</methodCall>" o2xmlrpc-response.xml -q "</methodCall>" 7778
  perl o2xmlrpc.pl

=cut

my $echo = 0;
my $verbose = 0;
my $hexdump = 0;
my $logfile = 0;
my $connection_quit_cmd = 0;
my $server_quit_cmd = 0;
my $connection_wait_count = 0;
my $receive_cmd = 0;
my $receive_txt = 0;

while ($ARGV[0] && $ARGV[0] =~ /^-/){
    $tmp = shift;
    if ($tmp eq "-e"){
	$echo = 1;
    } elsif ($tmp eq "-v"){
	$verbose = 1;
    } elsif ($tmp eq "-h"){
	$hexdump = 1;
    } elsif ($tmp eq "-l"){
	$logfile = shift;
    } elsif ($tmp eq "-p"){
	$port = shift;
    } elsif ($tmp eq "-q"){
	$connection_quit_cmd = shift;
    } elsif ($tmp eq "-Q"){
	$server_quit_cmd = shift;
    } elsif ($tmp eq "-w"){
	$connection_wait_count = shift;
    } elsif ($tmp eq "-r"){
        #die "-r not implemented";
	$receive_cmd = shift;
	#$receive_txt = sprintf(shift);
	$receive_txt = shift;
	$receive_txt =~ s#\\\\#\\#g;
	$receive_txt =~ s#\\n#\n#g;
	$receive_txt =~ s#\\r#\r#g;
    } elsif ($tmp eq "-R"){
	$receive_cmd = shift;
	$receive_file = shift;
	open(RFILE, "<$receive_file") 
	    or die "Can't open $receive_file for reading.\n$usage";
	$receive_txt = "";
	while (<RFILE>) { 
	    $receive_txt .= $_;
	}
	close(RFILE);

    } else {
	die "unknown cmd line option $tmp\n$usage";
    }
}

if ($verbose) {
    print "-e $echo -v $verbose -h $hexdump -l $logfile -q $connection_quit_cmd -Q $server_quit_cmd -w $connection_wait_count\n";
}

use IO::Handle;
STDOUT->autoflush(1);

if (!defined($port)) {
    $port = shift || 10007;
}
if ($verbose) { print "port:$port ... "; }

if ($verbose) { print "socket ... "; }
socket(CLIENT_WAIT, PF_INET, SOCK_STREAM, getprotobyname('tcp'))	
    or die "socket: $!";
if ($verbose) { print "setsockopt ... "; }
setsockopt(CLIENT_WAIT, SOL_SOCKET, SO_REUSEADDR, 1)			
    or die "setsockopt: $!";
if ($verbose) { print "bind ... "; }
bind(CLIENT_WAIT, sockaddr_in($port, INADDR_ANY))			
    or die "bind: $!";
if ($verbose) { print "listen ... "; }
listen(CLIENT_WAIT, SOMAXCONN)						
    or die "listen: $!";

$server_quit = 0;

while( !$server_quit && 
       (!$connection_wait_count || $connection_wait_count>0) ){
    if ($connection_wait_count) { $connection_wait_count--; }
    
    if ($verbose) { print "accept ... "; }
    $c_sockaddr = accept(CLIENT, CLIENT_WAIT);
    if ($verbose) { print "select ... CONNECTED.\n"; }
    select(CLIENT);  $| = 1;  select(STDOUT);

    ($c_port, $c_iaddr) = sockaddr_in($c_sockaddr);
    $c_addr = inet_ntoa($c_iaddr);
    $c_host = gethostbyaddr($c_iaddr, PF_INET);
    if ( ! $c_host ){ $c_host = $c_addr;}
    if ($verbose) { 
	print "Connected. addr: $c_addr host: $c_host port: $c_port\n";
    }

    if ($logfile){
        if (-e $logfile){
	    my $time = time();
	    rename($logfile, $logfile.$time);
	}
	open(LOG, "> $logfile");
	if ($windows){
	    binmode(LOG);				# Windows binary mode
	}
    }
    
    $| = 1;

    if ($verbose) { 
	print "Connection from $c_host [$c_addr] at port $c_port\n\n"; 
    }
    if ($hexdump) { 
	print "------: 00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F   0123456789ABCDEF\n\n";
    }

    $i = $j = 0;
    $connection_quit = 0;
    $line = "";

    # for trigger checking optomise (a little)
    my $l1=length($connection_quit_cmd);
    my $l2=length($server_quit_cmd);
    my $l3=length($receive_cmd);
    if ($l1==0) { $l1 = 8000; }
    if ($l2==0) { $l2 = 8000; }
    if ($l3==0) { $l3 = 8000; }
    my $mintrigger = 8000;
    if ($l1<=$mintrigger) { $mintrigger = $l1; }
    if ($l2<=$mintrigger) { $mintrigger = $l2; }
    if ($l3<=$mintrigger) { $mintrigger = $l3; }
    #my $mintrigger = min( $l1, $l2, $l3 );

    while ( !$connection_quit && sysread(CLIENT, $buf, 1) ){  # 1 char by char

        #if ($verbose) { print "buf: $buf\n"; } # every 1 char by char
	$line .= $buf;
	$c_value = unpack("C", $buf);

	# TODO: optomise 
	if ($verbose) { print "buflen" . length($buf) . "mintrigger $l1 $l2 $l3" . $mintrigger . "line: $line\n"; }
	if (length($line) >= $mintrigger ) {
	#if ( 1 ){   # if rx ctrl char or cr or ?? then check our triggers TODO: silly > 126, allow utf16, ...
	#if ($c_value < 32 || $c_value > 126 ){   # if rx ctrl char or cr or ?? then check our triggers TODO: silly > 126, allow utf16, ...

	    # hard work trigger checking every char by char but #
            # needed when cliend sends request without cr/lf after last byte #

	    #if ($verbose) { print "buf: $buf\n"; }
	    #if ($verbose) { print "line: $line\n"; }
	    if ($verbose) { print "trigger check: $buf $connection_quit_cmd $server_quit_cmd receive cmd $receive_cmd\n"; }
	    if ($connection_quit_cmd && ($line =~ m/$connection_quit_cmd/)){
		if ($verbose) { print "Command trigger: connection quit\n"; }
		$connection_quit = 1;
	    }

	    if ($server_quit_cmd && ($line =~ m/$server_quit_cmd/)){
		if ($verbose) { print "Command trigger: server quit\n"; }
		$server_quit = 1;
		#$connection_quit = 1; # server quit also triggers connection quit
	    }

	    if ($receive_cmd && ($line =~ m/$receive_cmd/)){
		if ($verbose) { print "Command trigger: writing response $receive_txt\n"; }
		syswrite(CLIENT, $receive_txt);
	    }
	}

	if ($c_value < 32 || $c_value > 126 ){   # if rx ctrl char or cr or ?? then check our triggers TODO: silly > 126, allow utf16, ...
	    $line = "";
	}

	if ($echo){
	    syswrite(CLIENT, $buf, 1);
	}

	if ($logfile){
	    syswrite(LOG, $buf, 1);
	}

	if ($hexdump) {
	    $c_value = unpack("C", $buf);
	    $hex_str .= sprintf " %02X", $c_value;

	    if ($c_value >= 32 && $c_value <= 126){
		$asc_str .= $buf;
	    } else {
		$asc_str .= ".";
	    }

	    printf "\r%05X0:%-51s%-16s", $j, $hex_str, $asc_str;

	    $i++;
	    if ($i == 16){
		$i = 0;
		$j++;
		$hex_str = $asc_str = "";
		print "\n";
	    }

	} else {
       
	    # char by char if ($verbose) { print "print buf: $buf\n"; }
	    print "$buf" if (!defined($logfile));

	}

    }

    # you want time-wait state ?
    # if don't close sockets perl does the right thing. YAY perl!
    # close this connection so connectee will go away
    close(CLIENT);

}

#close(CLIENT_WAIT);

exit;
