#  FakeSms.pm
#    - providing a simple testbed object for send/receive sms 
#      SmsEngine object iface
#
#  Copyright (C) 2004 Doolin Technologies
#
#  This module is free software; you can redistribute it and/or modify it
#  under the same terms as Perl itself.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
#  $Id: FakeSms.pm,v 1.4 2004/10/12 13:20:15 jamesc Exp $
#

package SmsEngine::FakeSms;

$VERSION = '0.00';

use strict;
use bytes;

sub new($%) {
    my $proto = shift;
    my $class = ref $proto || $proto;

    my %args = @_;

    my $self = {};

    $self->{debug} = ($args{debug})?$args{debug}:"";
    $self->{verbose} = ($args{verbose})?$args{verbose}:"";
    $self->{file} = ($args{file})?$args{file}:"";
    $self->{testresponse} = ($args{testresponse})?$args{testresponse}:"";
    $self->{newforcedie} = ($args{newforcedie})?$args{newforcedie}:"";

    if ($self->{file}) {
	open(FILE, ">>".$self->{file} ) 
	    or die "Can't open ".$self->{file}." for writing.";
	print FILE "SmsEngine Fake iface instantiated\n";
	close(FILE);
    }

    if ($self->{newforcedie}) {
        die "new force die  is set";
    }
    
    bless $self, $class;
}

sub sendsms {

    my ( $self, $to, $from, $text ) = @_;
    my %args = @_;

    if ($self->{file}) {
	open(FILE, ">>".$self->{file} ) 
	    or die "Can't open ".$self->{file}." for writing.";
	print FILE "SmsEngine Fake sendsms to $to from $from text $text\n";
	close(FILE);
    }
    return $self->{testresponse};
}

1;


=head1 NAME

FakeSms - providing a simple object for send/receive sms
          using SmsEngine object iface

=head1 SYNOPSIS

  #!/usr/bin/perl -w

  use strict;
  use Doolin::SmsEngine::FakeSms;

  # example - send sms FakeSms simple test
  my $fakesms = FakeSms->new( 
    file => "/tmp/DoolinSmsEngineFakeSmsTest.log",
    testresponse => 0);
  my $to = "353861111111";
  my $from = "FakeSms.pm"; # text from not supported by all smscs
  my $text = "FakeSms.pm example text message";
  my $result = $fakesms->sendsms($to,$from,$text);
  print "sendsms result: $result\n";
  undef $fakesms;

  # example - send sms FakeSms instantiate fails
  my $fakesms = FakeSms->new( newforcedie => 1);
  my $result = $fakesms->sendsms($to,$from,$text);
  print "sendsms result: $result\n";
  undef $fakesms;

=head1 DESCRIPTION

Useful for testing SmsEngine SmsIface container object.

=head1 COPYRIGHT

Copyright (C) 2004 Doolin Technologies

=cut
