#  KannelSms.pm
#    - providing a simple object for send/receive sms using kannel
#
#  Copyright (C) 2004 Doolin Technologies
#
#  This module is free software; you can redistribute it and/or modify it
#  under the same terms as Perl itself.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
#  $Id: KannelSms.pm,v 1.4 2004/10/12 13:20:15 jamesc Exp $
#

package SmsEngine::KannelSms;

$VERSION = '0.00';

use strict;
use bytes;

sub new($%) {
    my $proto = shift;
    my $class = ref $proto || $proto;

    my %args = @_;

    my $self = {};

    $self->{debug} = ($args{debug})?$args{debug}:"";
    $self->{verbose} = ($args{verbose})?$args{verbose}:"";


    if ($args{url}) {
	# croak ( "Invalid server url: $args{homedir}\n")
	#    unless TODO: how do we validate the url ?
	$self->{url} = $args{url};
    } else {
	croak ( "KannelSms new argument url is mandatory" );
    }

    if ( $args{user} ) {
	$self->{user} = $args{user};
    } else {
	croak ( "KannelSms new argument user is mandatory" );
    }

    if ( $args{pass} ) {
	$self->{pass} = $args{pass};
    } else {
	croak ( "KannelSms new argument pass is mandatory" );
    }

    $self->{dlrurl} = ($args{dlrurl})?$args{dlrurl}:"";
    $self->{dlrmask} = ($args{dlrmask})?$args{dlrmask}:"";
    $self->{smsc} = ($args{smsc})?$args{smsc}:"";

    bless $self, $class;
}

sub _getURL {
   my $self = shift;
   my $url = shift;
   my $cmd = "wget --proxy=off \"$url\" -o /tmp/wget-kannelsms-pm.log -O -";
   if ($self->{verbose}) { print "$cmd\n"; }
   return `$cmd`;
   #return `wget "$url" -o /tmp/wget.log -O -`;
}

sub sendsms {

    my ( $self, $to, $from, $text ) = @_;
    my %args = @_;

    my $dlrurl = ($args{dlrurl})?$args{dlrurl}:$self->{dlrurl};
    my $dlrmask = ($args{dlrmask})?$args{dlrmask}:$self->{dlrmask};
    my $smsc = ($args{smsc})?$args{smsc}:$self->{smsc};

    # TODO: test optional override default params:
    #       smsc (routing), dlrurl, dlrmask (delivery reports)

    my $urltosend = $self->{url} . 
        "?username=" . $self->{user} .
        "&password=" . $self->{pass} .
        "&dlrurl="   . $dlrurl .
        "&dlrask="   . $dlrmask .
        "&smsc="     . $smsc .
        "&to="       . $to .
        "&from="     . $from .
        "&text="     . $text;

    my $response = $self->_getURL($urltosend);
    return $response;
}

1;


=head1 NAME

KannelSms - providing a simple object for send/receive sms using kannel

=head1 SYNOPSIS

  #!/usr/bin/perl -w

  use strict;
  use KannelSms;

  # example - send sms KannelSms should hide this and read from config file
  my $smsboxurl = "http://localhost:13013/cgi-bin/sendsms";
  my $smsboxuser = "foo";
  my $smsboxpass = "test";
  my $kannelsms = KannelSms->new( url => $smsboxurl, user => $smsboxuser, pass => $smsboxpass);

  my $to = "353861111111";
  my $from = "KannelSms.pm"; # text from not supported by all smscs
  my $text = "KannelSms.pm example text message";
  my $result = $kannelsms->sendsms($to,$from,$text);
  print "sendsms result: $result\n";

=head1 DESCRIPTION

Provide a simple perl module for easy interface to send/receive sms.
Using kannel sms gateway. http://www.kannel.org
Obviously kannel software should be installed and running before
this module is usable.

=head1 DESIGN

Kannel interfaces to mobile modems/phones connected directly to host
and to smsc centres over tcp/ip, http, ...
 
For using kannel to send sms an ipc interface is provided using http protocol 
listening on port. KannelSms.pm uses wget to talk to that port and send sms.

Kannel is configured to trigger call of cgi script on receipt of sms.

=head1 kannel config

the url kannel uses to send sms is configured at compile time so
see install notes

port and host for sending sms is configured in kannel's smsbox group:

e.g.

  group = smsbox
  bearerbox-host = localhost
  sendsms-port = 13013

username & pass for sending sms is configured as a kannel sendsms-user group
(multiple sendsms-user groups are permitted

e.g. 

  group = sendsms-user
  username = tester
  password = foobar

services are configured in kannel to handle receipt of messages:
 
e.g. 

  # kannel filter messages start with "hello" 
  # auto handle AND sends reply (max-messages = 3)
  group = sms-service
  keyword = hello
  get-url = "http://localhost/topupclub-cgi-bin/receivesmshello.cgi?from=%p&to=%P&cmd=%k&text=%s&smscid=%i&dr1=%d&dr2=%A&user=%n&coding=%c&charset=%C"
  max-messages = 3
  concatenation = true
  catch-all = true

  # Preferred to catch all messages once in kannel with default service
  # catch all other messages and parse in cgi
  group = sms-service
  keyword = default
  # %u %A (s45 not used)
  get-url = "http://localhost/topupclub-cgi-bin/receivesms.cgi?from=%p&to=%P&text=%a&smscid=%i&dr1=%d&user=%n&coding=%c&charset=%C"
  max-messages = 0
  concatenation = true
  #catch-all = true

=head1 COPYRIGHT

Copyright (C) 2004 Doolin Technologies

=cut
