#  O2ProprietarySms.pm
#    - providing a simple object for send/receive sms using 
#      o2 proprietary XML/RPC interface, Doolin::SmsEngine object
#      compatible
#
#  Copyright (C) 2004 Doolin Technologies
#
#  $Id: O2ProprietarySms.pm,v 1.31 2005/04/27 10:07:12 jamesc Exp $
#

$VERSION = '0.00';

package SmsEngine::O2ProprietarySms;

my $SELFREF;

# TODO: make O2ProprietarySms class an SmsEngine
#       I think it would make things more straight forward
#use SmsEngine::SmsEngine;
#@ISA = ("SmsEngine");
#1;

use SmsEngine::SmsFilter;
@ISA = ("SmsEngine::SmsFilter");
1;

use strict;
use bytes;

sub new($%) {
    my $proto = shift;
    my $class = ref $proto || $proto;#

    my %args = @_;
    my $self = {};

    $self->{debug} = ($args{debug})?$args{debug}:0;
    $self->{verbose} = ($args{verbose})?$args{verbose}:0;#
    $self->{replytext} = "";
    $SELFREF = \$self;
    $SmsEngine::O2ProprietarySms::SELFREF = \$self;

    bless $self, $class;
}

# can be called b4 $self is blessed
sub loadaccount{
    my $self = shift;
    my $parent = shift;

    $self->{debug} = $parent->{debug}; 
    $self->{verbose} = $parent->{verbose}; 

    # read in config vars
    for my $c (qw(https_xmlrpc_url https_cert_file 
		  https_pkcs12_file https_pkcs12_password https_xmlrpc_timeout
		  smsinaddress smsinport
		  )) {

        #$self->{$c} = $parent->_val( $self, $self->{account}, "$c" );
        #$self->{$c} = $self->_val( $self, $self->{account}, "$c" );
	$self->{$c} = $parent->{cfg}->val( $parent->{account}, "$c" );
	#we replace whitespace at start and end of vars (in particular port numbers were affected)
        # better to fix by fixing ReadConfig.pm library but will save trouble by making
        # sure is okay here too
	if ($self->{$c}) {
	    $self->{$c} =~ s/^\s*//;
	    $self->{$c} =~ s/\s*$//;
	} else {
	    return { error => 1,
		     errors => [ "O2ProprietarySms loadaccount argument $c is mandatory.\n" ] };
	}
    }

    # set optional arg defaults
    $self->{io_socket_timeout} = 10;
    $self->{socket_read_timeout} = 10;

    # optional args
    for my $c (qw(proxy io_socket_timeout socket_read_timeout)) {
	$self->{$c} = $parent->{cfg}->val( $parent->{account}, "$c" );
	if ($self->{$c}) {
	    $self->{$c} =~ s/^\s*//;
	    $self->{$c} =~ s/\s*$//;
	}
    }

    # SmsFilter
    $self->loadsmsfilter($parent);

    # die ( "Invalid server url: $args{homedir}\n")
    #    unless TODO: how do we validate the url ?

    for my $c (qw(https_cert_file https_pkcs12_file
		  )) {
	return { error => 1,
		 errors => [ "O2ProprietarySms loadaccount Can\'t read file " . $self->{$c} . " $!" ] }
	    unless -r $self->{$c};
    }

    # do not die on error. handle error gently
    use XMLRPC::Lite on_fault => 
	sub {
	    my($soap, $res) = @_; 
	    # we log error, later we notice nothing returned and handle fault logging further
	    #print STDERR "xmlrpcliteon_fault " . Data::Dumper->Dumper($res) . "\n";
	    #print STDERR Data::Dumper->Dumper($soap) . "\n";
	    print STDERR ref $res ? "result fault detail:" . $res->faultdetail 
		: "soap transport status:" . $soap->transport->status, "\n"
	    };
    
    #use XMLRPC::Lite;
    #use XMLRPC::Lite +trace => 'debug';
    #http://www.xav.com/perl/site/lib/SOAP/Lite.html
    #we switch on/off trace in XMLRPC::Lite depending on $self->{debug}
    my $xmlrpctrace;
    if ($self->{debug}) {
	#$xmlrpctrace = "transport dispatch result parameters headers objects method fault freeform trace debug";
	SOAP::Trace->import(trace => 'debug');
    }

    # read perldoc /usr/lib/perl5/site_perl/5.8.0/SOAP/Transport/HTTP.pm
    # =head2 SSL CERTIFICATE AUTHENTICATION

    #To get certificate authentication working you need to specify three
    #environment variables: C<HTTPS_CERT_FILE>, C<HTTPS_KEY_FILE>, and 
    #(optionally) C<HTTPS_CERT_PASS>:

    $ENV{HTTPS_CERT_FILE}       = $self->{https_cert_file};
    $ENV{HTTPS_PKCS12_FILE}     = $self->{https_pkcs12_file};
    $ENV{HTTPS_PKCS12_PASSWORD} = $self->{https_pkcs12_password};

    # DEFAULT SSL VERSION, 
    # read O2 java sms broker faq and perl xmlrpc info
    $ENV{HTTPS_VERSION} = '3';

    # cert/key problems (if any) in detail I hope?
    $ENV{HTTPS_DEBUG} = 0;
    if ($self->{debug}) {
	$ENV{HTTPS_DEBUG} = 1;
    }

    return { error => 0 };

}

# showconfig is also an example of how to access config settings
sub showconfig {

    my $self = shift;

    for my $c (qw(debug verbose
		  https_xmlrpc_url https_cert_file 
		  https_pkcs12_file https_pkcs12_password https_xmlrpc_timeout
		  proxy
		  smsinaddress smsinport smsinfilter smsinfiltersettings
		  smsinfilterprioritylist
		  )) {
	print "O2ProprietarySms arg $c = " . $self->{"$c"} . "\n";
    }

    $self->showfilterconfig();

}


# response from sendmessage, sendsimplemessage or sendbatchmessage
sub printsendresponse {
    my $self = shift;
    my $res = shift;
    my $all = shift;

    print "RESULT $res RESULT\n";
    #SOAP::Data->type(base64 => $string)
    print "id:".$res->{RequestID}." " if (defined($res->{RequestID}));
    print "errcode:".$res->{ErrorCode}." " if (defined($res->{ErrorCode}));
    print "errmsg:".$res->{ErrorMessage}." " if (defined($res->{ErrorMessage}));

    print "failed_MSISDN:".$res->{Failed_MSISDN}." " 
	if (defined($res->{Failed_MSISDN})); # ARRAY(0xaddr)
    my $i=0;
    while (defined($res->{Failed_MSISDN}->[$i]) &&
	   ( defined($all) || $i<10 ) 
           # maximum of 10 verbose output unless we explicitly ask for $all
	   ) {
	print "YO! we have something in the Failed_MSISDN array";

	# doesn't work ? elements:ARRAY(0x86c4d64)
	my $s = scalar($res->{Failed_MSISDN}); # "Not an ARRAY reference at o2xmlrpc.pl line 122."
	print "elements:".$s." ";

	print $i."snrdth element: ".$res->{Failed_MSISDN}->[$i]." "; # that be a hash
	print "errorcode: ".$res->{Failed_MSISDN}->[$i]->{errorcode}." ";
	print "num: ".$res->{Failed_MSISDN}->[$i]->{num}."\n";

        #foreach my $fmsisdn (@$res->{Failed_MSISDN}) { 
        #print "msisdn:".$fmsisdn." ";
        #}

	$i++; # check for a next element!
    }
    print "\n";
}

sub logsendresponse {
    my $self = shift;
    my $logfile = shift;
    my $res = shift;

    if ( ! open(LOGFILE, ">>$logfile") ) {
	# failing to open log is not an executable offence
	print STDERR "ERROR: Couldn't open $logfile $!";
    } else {
	my $logentry;
	$logentry = "\$ts=" . time() . ";";
	$logentry .= "\$reqid=" . $res->{RequestID} . ";" 
	    if (defined($res->{RequestID}));
	$logentry .= "\$errorcode=" . $res->{ErrorCode} . ";";
	$logentry .= "\$errormessage=" . qq($res->{ErrorMessage}) . ";";

	my $fi=0;
	while (defined($res->{Failed_MSISDN}->[$fi])) {
	    $logentry .= "fault[$fi]=";
	    $logentry .= "{ errorcode => ".$res->{Failed_MSISDN}->[$fi]->{errorcode}.",";
	    $logentry .=  "num => ".qq($res->{Failed_MSISDN}->[$fi]->{num})."};";
	    $fi++; # check for a next element!
	}
	$logentry .= "\n";
	print LOGFILE $logentry;
	close(LOGFILE);
    }   
}

# response from querysenddeliverystatus, querybatchrequest or queryaccountrequest
sub printqueryresponse {
    my $self = shift;
    my $res = shift;

    #use Data::Dumper; print Dumper($res) if ($debug);
    if (!defined($res) || defined($res->{ErrorCode}) &&
        $res->{ErrorCode} != 0) {
        print "ErrorCode: " . $res->{ErrorCode} . "\n";
        print "ErrorMessage: " . $res->{ErrorMessage} . "\n";
    } else {

    print "RESULT $res RESULT\n";

    my @reqStatus = ( "?", "Queued", "Processing", "?", "Failed", "Finished" );

    print "TotalNumberOfMessages:".$res->{TOTALNUMBEROFMESSAGES}." ";
    print "MessagesSent:".$res->{MESSAGESSENT}." ";
    print "MessagesSkipped:".$res->{MESSAGESSKIPPED}." ";
    print "StartDate:".$res->{STARTDATE}." ";
    print "Message:".$res->{MESSAGE}." ";
    print "Status:".$res->{STATUS}." ".$reqStatus[$res->{STATUS}]." ";
    print "ErrorCode:".$res->{ErrorCode}." ";
    print "ErrorMessage:".$res->{ErrorMessage}." ";

    print "\n";
    }
}

sub xmlrpccall {
    my $self = shift;
    my $call = shift;
    my $param = shift;

    my $url = $self->{https_xmlrpc_url};
    my $proxy = $self->{proxy};
    my $timeout = $self->{https_xmlrpc_timeout};
    $timeout = 5 if (!$timeout);

    my ($res,$xml,$xmlres);

    if ($proxy) {
	$xml = XMLRPC::Lite
	    -> proxy($url, proxy => ['http', $proxy], timeout => $timeout);
	    #-> proxy('https://216.239.194.94:444/vvtlistener1/SMSOA_xmlrpc_listener', 
                     #timeout => $timeout)
	    #-> proxy('http://localhost:7778')
	    #-> proxy('https://localhost:7778')
    } else {
	$xml = XMLRPC::Lite
	    -> proxy($url, timeout => $timeout);
    }

    # not o2 error code, SmsEngine/O2ProprietarySms error code
    my $connecterrorcode = 8152;
    my $connecterror = "SmsEngine connect Error: can\'t connect url $url proxy $proxy timeout $timeout"; 

    if (!defined($xml) || $@) {
	$res->{RequestID} = "";
	$res->{ErrorCode} = $connecterrorcode;
	$res->{ErrorMessage} = $connecterror.".oOo.".$@;
	return $res;
    } # accessing $xml->fault makes xmlrpc call fault - not what we want
      # same for $xml->transport
      # if we don't catch fault error when we try to do xml call we get die
#soap transport status:500 SSL negotiation failed: 
#Can't call method "fault" without a package or object reference at blib/lib/SmsEngine/O2ProprietarySms.pm line 261.
#make: *** [test_dynamic] Error 9

#elsif (defined($xml->fault) && $xml->fault) { #transport->is_success
#      elsif (0 && $xml->fault) { # fault can be set when there is no fault
#	$res->{RequestID} = "";
#	$res->{ErrorCode} = $connecterrorcode;
#	$res->{ErrorMessage} = join (".oOo.",$connecterror, 
#				     "TRANSPORT ERROR: ", $xml->fault,
#				     $xml->transport->is_success, $xml->transport->status
#				     );  
#        return $res;
#    }

    #!$self->transport->is_success || # transport fault
    #  $@

    #use Data::Dumper;
    #print STDERR "Dump:". ( ref $xml ) . ".oOo." . Data::Dumper->Dumper($xml) . ".oOo.\n";

    $xmlres = $xml->call($call, ( $param ) );

    #print STDERR "Dump:". ( ref $xmlres ) . ".oOo." . Data::Dumper->Dumper($xmlres) . ".oOo.\n";

    if (!defined($xmlres)) {   
	$res->{RequestID} = "";
	$res->{ErrorCode} = $connecterrorcode;
	$res->{ErrorMessage} = join (".oOo.",$connecterror, 
				     "Problem calling XMLRPC::Lite method $call $param");
	return $res;
    } elsif ($@) {
	$res->{RequestID} = "";
	$res->{ErrorCode} = $connecterrorcode;
	$res->{ErrorMessage} = join(".oOo.",$connecterror,$@);
	return $res;
    #} elsif ($can && !UNIVERSAL::isa($xmlres => 'XMLRPC::SOM')) {
    #   print $res;
    } elsif (defined($xmlres) && ! (ref $xmlres) ) {
	$res->{RequestID} = "";
	$res->{ErrorCode} = $connecterrorcode;
	$res->{ErrorMessage} = join (".oOo.",$connecterror, 
				     "XMLRPC FAULT: ");
        #, @{$xmlres->fault}{'faultCode','faultString'});
	return $res;
    }
  
    #join ("\n",$xmlres->paramsall);
    $res = $xmlres->result();

    return $res;
}

sub sendsimplemessage {
    my $self = shift;
    my $h = shift;

    if (!$h->{text} || !$h->{to}) {
	die "sendsimplemessage: text and to params are mandatory";
    }

    my $param = { ( "Type" => "NOR" ),
		  ( "Message" =>
		    [ { "text" => qq($h->{text}) } ], ),
		  (    "MSISDN" =>
		       [ { "num" => SOAP::Data->type(string => qq($h->{to})) } ], ),
		  };

    # adding in optional parameters
    #print $param->{Type};
    #print $param->{Message}->[0]->{text};

    # Type can be 'NOR', 'FLA', 'RTS', 'CLO', 'CON', 'LOG', 'RIN', 'PCS'

    for my $c (qw(Type AccountCategory DynamicOriginator
		  )) {
	if ($h->{$c}) {
	    $param->{$c} = $h->{$c};
	}
    }

    my $res = $self->xmlrpccall( 'sendsimplemessage', ( $param ) );

    return $res;
}

sub sendmessage {
    my $self = shift;
    my $h = shift;

    if (!$h->{text} || !$h->{to}) {
	die "sendmessage: text and to params are mandatory";
    }

    # additionally $h->{to} must be an array with >0 elements
    my $size = scalar(@{$h->{to}});
    if (!$size) {
	die "sendmessage: $h->{to} must be an array with >0 elements";
    }

    #die "sendmessage debug: $h->{to} must be an array with >0 elements".$size;

    my $param = { ( "Type" => "NOR" ),
		  ( "RequestSize" => $size ),
		  };
		  #( "Message" =>
		  #  [ { "text" => qq($h->{text}) } ], ),
		  #( "MSISDN" =>
		  #  [ { "num" => SOAP::Data->type(string => qq($h->{to})) } ], ),

    my $i=0;
    while (defined($h->{to}->[$i])) {
	#$param->{MSISDN}->[$i]->{num} = $h->{to}->[$i];
	$param->{MSISDN}->[$i]->{num} = SOAP::Data->type(string => qq($h->{to}->[$i]));
	$i++;
    }

    if (!defined($h->{Type}) || $h->{Type} eq "NOR") {

	# we _could_ also support type "CON" which is split into 100 char parts BUT
	#  some types of phones may not support that
 
	# o2 proprietary maxlen for their MPN sms is 100 bytes! this is silly
	# sms maxlen actually 140 bytes or 160 7-bit characters. 
	#  143 bytes really in my head and now uncertainty is introduced 
	#  147 is also clamouring for attention
	# multi part messages, message parts >100 bytes return error
	my $l = length $h->{text};
	my $SMSMAXLEN = 143;  # we can send simgle messages max length is ????
	my $SMSMULTMAXLEN = 100;

	if ($l <= $SMSMAXLEN) { # not allowed specify index with single part message
	    $param->{Message}->[0]->{text} = qq($h->{text}); 
	} else { # multi-part message
	    $i=0; my $p=0; my $seg;
	    while ($p < $l) {
		$param->{Message}->[$i]->{index} = $i+1; 
		$seg = substr($h->{text},$p,$SMSMULTMAXLEN);
		$param->{Message}->[$i]->{text} = 
		    SOAP::Data->type(string => qq($seg));
		$i++; $p+=$SMSMULTMAXLEN;
	    }
	    $param->{Type} = "MPN"; # multi-part type message
	    $param->{NumOfParts} = $i; # num of parts
	    
	}
    } else {
	$param->{Message}->[0]->{text} = qq($h->{text}); 
    }

    # adding in optional parameters
    # Type can be 'NOR', 'FLA', 'RTS', 'CLO', 'CON', 'LOG', 'RIN', 'PCS'
    for my $c (qw(Type MTCode
		  AdvertFlag ReceiptFlag TimeToLive Route
		  ValidateOnly
		  AccountCategory
		  Originator DynamicOriginator
		  Index
		  )) {
                  # O2 SMS Push API says not used MTBillPrice MTBillCurrency
	if ($h->{$c}) {
	    $param->{$c} = $h->{$c};
	}
    }

    my $res = $self->xmlrpccall( 'sendmessage', ( $param ) );

    return $res;
}

sub sendbatchmessage {
    my $self = shift;
    my $h = shift;

    if (!$h->{text} || !$h->{to}) {
	die "sendbatchmessage: text and to params are mandatory";
    }

    # additionally $h->{to} must be an array with >0 elements
    my $size = scalar(@{$h->{to}});
    if (!$size) {
	die "sendbatchmessage: $h->{to} must be an array with >0 elements";
    }

    #die "sendmessage debug: $h->{to} must be an array with >0 elements".$size;

    my $param = { ( "Type" => "NOR" ),
		  ( "RequestSize" => $size ),
		  ( "Message" =>
		    [ { "text" => qq($h->{text}) } ], ),
		  #(    "MSISDN" =>
		  #     [ { "num" => SOAP::Data->type(string => qq($h->{to})) } ], ),
		  };

    my $i=0;
    while (defined($h->{to}->[$i])) {
	#$param->{MSISDN}->[$i]->{num} = $h->{to}->[$i];
	$param->{MSISDN}->[$i]->{num} = SOAP::Data->type(string => qq($h->{to}->[$i]));
	$i++;
    }

    # adding in optional parameters
    #print $param->{Type};
    #print $param->{Message}->[0]->{text};

    # Type can be 'NOR', 'FLA', 'RTS', 'CLO', 'CON', 'LOG', 'RIN', 'PCS'

    for my $c (qw(Type MTCode
		  AdvertFlag ReceiptFlag TimeToLive Route
		  ValidateOnly
		  AccountCategory
		  Originator DynamicOriginator
		  NumOfParts Index
		  )) {
                  # O2 SMS Push API says not used MTBillPrice MTBillCurrency
	if ($h->{$c}) {
	    $param->{$c} = $h->{$c};
	}
    }

    if (!$param->{StartTime}) {
	# do it now. StartTime mandatory parameter
	$param->{StartTime} = time();
    }

    my $res = $self->xmlrpccall( 'sendbatchmessage', ( $param ) );

    return $res;
}

sub _sendrequestIDmessage {
    my $self = shift;
    my $method = shift;
    my $h = shift;

    if (!$h->{RequestID}) {
	die "$method: RequestID is mandatory";
    }

    my $param = { ( "RequestID" => qq($h->{RequestID}) ), };

    my $res = $self->xmlrpccall( $method, ( $param ) );

    return $res;
}

sub querysenddeliverystatus {
    my $self = shift;
    my $h = shift;
    my $res = $self->_sendrequestIDmessage( 'querysenddeliverystatus', $h );
    return $res;
}

sub querybatchrequest {
    my $self = shift;
    my $h = shift;
    my $res = $self->_sendrequestIDmessage( 'querybatchrequest', $h );
    return $res;
}

sub retrieveskippedrecords {
    my $self = shift;
    my $h = shift;
    my $res = $self->_sendrequestIDmessage( 'retrieveskippedrecords', $h );
    return $res;
}

sub startbatchrequest {
    my $self = shift;
    my $h = shift;
    my $res = $self->_sendrequestIDmessage( 'startbatchrequest', $h );
    return $res;
}

sub stopbatchrequest {
    my $self = shift;
    my $h = shift;
    my $res = $self->_sendrequestIDmessage( 'stopbatchrequest', $h );
    return $res;
}

sub removebatchrequest {
    my $self = shift;
    my $h = shift;
    my $res = $self->_sendrequestIDmessage( 'removebatchrequest', $h );
    return $res;
}

sub updatebatchrequest {
    my $self = shift;
    my $h = shift;

    if (!$h->{RequestID} || !$h->{StartTime}) {
	die "updatebatchrequest: RequestID and StartTime are mandatory";
    }
    my $param = { ( "RequestID" => qq($h->{RequestID}) ),
		  ( "StartTime" => qq($h->{StartTime}) ), };

    my $res = $self->xmlrpccall( 'updatebatchrequest', ( $param ) );

    return $res;
}

sub queryaccountrequest {
    my $self = shift;
    # queryaccountrequest needs <params><param><value><struct></struct></value></param></params>
    # perl+SMLRPC lib used doesn't really do that ,,, if struct empty it does  
    # <?xml version="1.0" encoding="UTF-8"?><methodCall><methodName>queryaccountrequest</methodName><params><param><value><struct/></value></param></params></methodCall>
    my $param = {(())};
    #$param->{void} = 0;
    #undef $param->{void};
    #delete $param->{void};
    my $res = $self->xmlrpccall( 'queryaccountrequest', ( $param ) );
    return $res;
}

sub stopallbatchrequests {
    my $self = shift;
    my $param = { ( ), };
    $param->{thing} = 1;
    #undef $param->{thing};
    my $res = $self->xmlrpccall( 'stopallbatchrequests', ( $param ) );
    return $res;
}


sub sendsms {

    my ( $self, $to, $from, $text ) = @_;
    my %args = @_;

    my $res = $self->sendmessage( { text => qq($text),
				    to => [ $to ] } );
    $self->printsendresponse($res) if ($self->{verbose});

    return $res;
}


my $o2errorcodes = {
    0    => "Success",
    2    => "Quota Exceeded",
    4    => "Internal network error. Please try again later",
    5    => "Invalid Message datastructure",
    6    => "Invalid MSISDN datastructure",
    7    => "Invalid NumOfParts",
    8    => "Invalid RequestSize",
    9    => "Error! This account is not currently active",
    10   => "This API is unavailable",
    11   => "Invalid message type",
    12   => "Personalisation not allowed for multipart message types",
    13   => "Index does not exist for preindexed content",
    15   => "At least one mandatory parameter is missing",
    16   => "Unrecognised parameter passed to API call",
    18   => "Argument of wrong type passed",
    19   => "Personalisation not allowed for SEND API",
    21   => "You are not authorised to use MT billing",
    22   => "You are not authorised to use receipting",
    31   => "TimeToLive outside valid range",
    41   => "Account ID is an empty string",
    43   => "Unrecognised method call",
    49   => "Invalid Account ID",
    50   => "Invalid Request ID",
    51   => "Request ID is an empty string",
    52   => "Request is not a valid SEND request",
    53   => "Request is not a valid BATCH request",
    59   => "Datetime not in correct format",
    61   => "Invalid Message type",
    62   => "Account attempting to request data for a request belonging to another account",
    63   => "No default route configured for this Account",
    64   => "Route specified is not permitted for this Account",
    65   => "Message type specified is not permitted for this Account",
    66   => "Message type specified for the Account is not permitted on this Route",
    67   => "Originator specified is not permitted for this Account",
    68   => "Originator specified for the Account is not permitted on this Route",
    69   => "MTCode specified is not permitted for this Account",
    70   => "Receipting for this Account is not supported on this Route",
    71   => "Start date out of acceptable range",
    72   => "Max MSISDN fail limit exceeded",
    73   => "Dynamic Originator is not permitted for this Account",
    74   => "Dynamic Originator is too long",
    80   => "Message file processing error",
    99   => "You are not authorised to call this API",
    100  => "Internal error",
    101  => "Action not supported, submitted request is in QUEUED state",
    102  => "Action not supported, submitted request is already in PROCESSING state",
    103  => "Action not supported, submitted request is in SUSPENDED state",
    104  => "Action not supported, submitted request is in FAILED state",
    105  => "Action not supported, submitted request is already FINISHED",
    106  => "Action not supported, submitted request is already REMOVED",
    201  => "SimpleSend and SimpleMTSend".
	" only support one entry in both the Message and MSISDN arrays",
    202  => "Invalid format specified for num in MSISDN structure",
    203  => "Number is not MT Billable",
    204  => "Message length is more than 160 characters",
    205  => "MSISDN contains invalid personalisation field",
    206  => "Unable to send MTBilled messages at this time, please try later",

    'GX-8' => "Unable to send the message due to illegal message content",

    'MT-1' => "MSISDN is blacklisted and cannot be billed",
    'MT-2' => "The phone number does not fall under any known operator",
    'MT-3' => "The Number is not a billable number for the operator",
    'MT-4' => "MTCode is not valid for the operator",
    'MT-5' => "Number is not billable due to the operator it belongs to",
    'MT-8' => "The number has exceeded its quota for that day",
};

sub maperrorcode {
    my $self = shift;
    my $errorcode = shift;
    return $o2errorcodes->{$errorcode};
}

sub setreplytext {
    my $self = shift;
    $self->{replytext} = shift;
}

sub getreplytext {
    my $self = shift;
    return $self->{replytext};
}



sub encode_chars_for_tx {

# Character (space) ! " # $ % & ` ( ) * + , . / 0 ­ 9 : ; < = > ? @ A - Z ¡ £ 
# Unicode (hex) 20 - 2F 30 ­ 39 3A - 40 41 ­ 5A A1 A3 
# XML Encoding (space) ! " # &#036; % &amp; ` ( ) * + , . / 0 - 9 : ; &lt; = &gt; ? @ A - Z &#161; &#163; 

# Character ¤ ¥ § ¿ Ä Å Æ Ç É Ñ Ö Ø Ü à ä å æ è é ì ñ ò ö ø ù ü 
# Unicode (hex) A4 A5 A7 BF C4 C5 C6 C7 C8 D1 D6 D8 DC E0 E4 E5 E6 E8 E9 EC F1 F2 F6 F8 F9 FC 
# XML Encoding &#164; &#165; &#167; &#191; &#196; &#197; &#198; &#199; &#200; &#209; &#214; &#216; &#220; &#224; &#228; &#229; &#230; &#232; &#233; &#236; &#241; &#242; &#246; &#248; &#249; &#252;

    tr/[\x00-\x1F]//; # rm ctrl chars  (this doesn't work?)
    tr/[\x00..\x1F]//; # rm ctrl chars  (this doesn't work?)
    s/[[:cntrl:]]/CTL/gs; # try this instead?

    # Character $ & < > ¡ £ 
    # XML Encoding &#036; &amp; &lt; &gt; &#161; &#163; 
    
    # &amp; MUST be done first.
    s/&/&amp;/g;
    s/\$/&#036;/g;
    s/</&lt;/g;
    s/>/&gt;/g;
    s/¡/&#161;/g;
    s/£/&#163;/g;

    # TODO: euro ?

}

########################################

my ($MSG_ERR, $MSG_DBG, $MSG_INFO, $MSG_CRIT, $MSG_EMERG) 
    = ('warning', 'debug', 'info', 'crit', 'emerg');

sub debug {
    my $self = shift;
    return $self->{debug};
}

use XMLRPC::Transport::HTTP on_fault => 
    sub {my($soap, $res) = @_; 
	 SmsEngine::O2ProprietarySms::logevent(0, $MSG_EMERG, "XMLRPC::Transport::HTTP fault");
	 die ref $res ? $res->faultdetail : $soap->transport->status, "\n"};

my $debug = SmsEngine::O2ProprietarySms::debug();

if ($debug) {
    SmsEngine::O2ProprietarySms::logevent(0, $MSG_DBG, "debug $debug is on, XMLRPC trace is on");

    if ($debug & 0x40) {
	# TODO not as expected at times already used => 
	# this is an horrendous amount of debug
	use XMLRPC::Transport::HTTP +trace => 'debug';
        SmsEngine::O2ProprietarySms::logevent(0, $MSG_DBG, "debug XMLRPC trace is on");
    }

    if ($debug & 0x80) {
	# both following do the same (socket level data trace/debug)
	# this is an horrendous amount of debug
	$HTTP::Daemon::DEBUG=1;
	$XMLRPC::Transport::HTTP::Daemon::DEBUG=1;
        SmsEngine::O2ProprietarySms::logevent(0, $MSG_DBG, "debug socket trace is on");
    }

} else {
    use XMLRPC::Transport::HTTP;
}

sub stopinservice {
    my $self = shift;
    undef $self->{daemon}; 
}

sub startinservice {
    my $self = shift;
    my $o2port = shift;
    $self->{poe_kernel} = shift;
    $self->{forwardmessageref} = shift; 
    $self->{logeventref} = shift;

    my $socket_read_timeout = $self->{socket_read_timeout};
    $socket_read_timeout = 10 if (!defined($socket_read_timeout));
    my $io_socket_timeout = $self->{io_socket_timeout};
    $io_socket_timeout = 10 if (!defined($io_socket_timeout));

    # call like &$forwardmessageref(....)

    if (defined($self->{daemon})) {
	warn "in service daemon already started. NOT starting another.";
    } else {

	# see in perldoc below, Design handling of xmlrpc in socket
	$self->{daemon} = XMLRPC::Transport::HTTP::Daemon
	    #  -> new (Timeout => 10, io_socket_timeout => 10, socket_read_timeout => 10, 
	    -> new (io_socket_timeout => $io_socket_timeout, 
		    socket_read_timeout => $socket_read_timeout,
		    LocalPort => $o2port, 
		    Reuse => 1,      # Can't create daemon: Invalid argument at bin/o2xmlrpcin.daemon line 401
		    #ReusePort => 1,  # Your vendor has not defined Socket macro SO_REUSEPORT, used at /usr/lib/perl5/5.8.0/IO/Socket/INET.pm line 158
		    #ReuseAddr => 2
		    )

	    #-> setsockopt('SOL_SOCKET', 'SO_REUSEADDR', 1)

	    -> dispatch_to('XMLRPCHandler')
	    ;
    }

    #use Socket;
    #use IO::Handle;
    #$daemon->setsockopt('SOL_SOCKET', 'SO_REUSEADDR', 1);

    #use Data::Dumper;
    #print STDERR "timeout? " . $daemon . "name" . ref $daemon ."\n";
    #print STDERR "timeout? " . Data::Dumper->Dumper($daemon) ."\n";
    #print STDERR "timeout? " . Data::Dumper->Dumper($daemon->{_daemon}) ."\n";
 
    $self->logevent($MSG_DBG, "Active server (o2prop xmlrpc flavour) at " . $self->{daemon}->url);

    # TODO: this handler doesn't work beautifully with POE
    # POE likes things to do a bit and then return fast
    # but this server does only want to do this one thing
    # but behaviour not beautiful as signals not caught:
    #  - kill -9 must be used, Ctrl-C not caught, ...
    $self->{daemon}->handle;

    # not reached!!!
    return $self->{daemon};
}

# callback callers
sub logevent {
    my $self = shift;
    #$self = $SmsEngine::O2ProprietarySms::SELFREF if (!$self);
    #use Data::Dumper; print Dumper($self);
    #use Data::Dumper; print Dumper(@_);
    if (defined($self->{logeventref})) {
	return &{$self->{logeventref}}(@_);
    } else {
	print STDERR "WARNING: logevent not set and $1 $2 $3 $4\n";
    }
}

sub forwardmessage {
    my $self = shift;
    #$self = $SmsEngine::O2ProprietarySms::SELFREF if (!$self);
    return &{$self->{forwardmessageref}}(@_);
}

sub poe_kernel_var {
    my $self = shift;
    #$self = $SmsEngine::O2ProprietarySms::SELFREF if (!$self);
    return $self->{poe_kernel};    
}


# package like wot o2 use and method getContent
package XMLRPCHandler;

# we should possibly reply immediately with blank msg as can not guarantee
#  response in reasonable time and blocking here would hold things up not nicely
# BUT ... we don't >;)
# we hold connection open, call tucd to forward on sms, get response back ... and send it
sub getContent { 
    shift if UNIVERSAL::isa($_[0] => __PACKAGE__);	   
    my $h = shift;

    # get over package 
    my $selfref = $SmsEngine::O2ProprietarySms::SELFREF;
    #use Data::Dumper; print Dumper($selfref);

    $$selfref->logevent
	($MSG_DBG,  "Yo! XMLRPCHandler.getContent $h hash" .
                        "SHORTCODE:".$h->{SHORTCODE}." ".
     	                "NETWORK:".$h->{NETWORK}." ".
	                "MSISDN:".$h->{MSISDN}." ".
	                "msg:".$h->{msg});

    # TODO: less crude shutdown 
    #   this at least though allows us to avoid using kill -9
    # perl o2serversim-xmlrpc-sendsms.pl "go away poe thing" "no message"
    if ($h->{MSISDN} =~ "go away poe thing") {
	$$selfref->logevent($MSG_DBG,  "go away poe thing!!!");
	$$selfref->poe_kernel_var()->stop();
        die "exiting. goodbye";
        #$main::die "die main";
	#$main::daemon->exit;

    } else {

        # TODO: rm hack for reference test env sub keyword alphyra with TUC
	#$h->{msg} =~ s/alphyra/TUC/;

	## TODO apply filters
	my ($filterresult,$filtername,$failmessage,$address,$port) = 
	    $$selfref->applyfilters($h->{SHORTCODE}, $h->{msg});
        ### if defined($address) && defined($port) not done yet
	## $address,$port not implemented, not requested as feature
	if (defined($failmessage)) {
            ### daemon code if (defined($failmessage) then send failmessage back
           $$selfref->logevent($MSG_DBG,  "incoming filter trap: ".$filtername);
           $$selfref->setreplytext($failmessage);

        } else {
            ### otherwise pass it on as usual
	    $$selfref->logevent($MSG_DBG,  "forward message ".$h->{msg});
	    # Run the poe client to forward text to tucd
	    $$selfref->forwardmessage($h->{MSISDN},$h->{msg});
	    $$selfref->poe_kernel_var()->run();

        }

    }

    my $res;

    # This message is returned to 02 sms server
    #  if text is blank no message is sent to originating mobile
    #  if text is set that text is sent to originating mobile
    #  if we do not return this mobile gets an error message
    $res->{Content} = ( {
	( Type => "NOR" ),
	#( MTCode => "" ),
	( "Message" =>
	  [ { "text" => $$selfref->getreplytext() } ], ),
	} );

    # clear replytext for next message
    $$selfref->setreplytext("");

    return $res;
}


1;


=head1 NAME

O2ProprietarySms - providing a simple object for send/receive sms using 
      o2 proprietary XML/RPC interface, Doolin::SmsEngine object compatible

=head1 SYNOPSIS

  #!/usr/bin/perl -w

  use strict;

  use SmsEngine::SmsEngine;

  # new SmsEngine container (config)
  # load config will die if can't find config file
  # from config o2proprietary_iface causes instantiation of O2ProprietarySms object
  my $smsengine = SmsEngine->new( file => "smsengine.topupclub.conf",
                              verbose => 0, 
                              debug => 0, 
			      iface => "o2proprietary_iface", 
			      account => "topupclub_o2_account" );

  # access params
  my $verbose = $smsengine->getparam("verbose");

  my $res = $smsengine->{ObjectSms}->sendmessage( { text => "Hello Ciao",
				     to => 353869999999 } );

  $smsengine->{ObjectSms}->printsendresponse($res);

  my $qres = $smsengine->{ObjectSms}->querysenddeliverystatus( 
      { RequestID => $res->{RequestID} } );
  $smsengine->{ObjectSms}->printqueryresponse($qres);

=head1 DESCRIPTION

 Provide a simple perl module for easy interface to send/receive sms.
 Using o2proprietary sms gateway. http://www.o2proprietary.org
 Obviously o2proprietary software should be installed and running before
 this module is usable.

=head1 DESIGN

 O2proprietary interfaces to mobile modems/phones connected directly to host
 and to smsc centres over tcp/ip, http, ...
 
 For using o2proprietary to send sms an ipc interface is provided using http protocol 
 listening on port. O2ProprietarySms.pm uses wget to talk to that port and send sms.

 O2proprietary is configured to trigger call of cgi script on receipt of sms.

 The object hierarchy changed and went a bit wrong.
 We did work one way it KannelSms and SmsEngine
 But then O2ProprietartSms came and ....
 Well some things may not make sense now and should be refactored out in time.
 O2ProprietarySms should be an SmsEngine, not contained by it.

=head1 o2proprietary config example

 [o2proprietary_iface]
 object = iface
 ifacetype = o2proprietary
 inout = smsengine_tucd

 [topupclub_o2_account]
 object = account
 ifacetype = o2proprietary
 https_soap_url = https://216.239.194.94:444/vvtlistener1/SMSOA_soap_listener
 https_xmlrpc_url = https://216.239.194.94:444/vvtlistener1/SMSOA_xmlrpc_listener
 https_xmlrpc_timeout = 5
 https_cert_file = /usr/local/smsengine/etc/cacert-reference.pem
 https_pkcs12_file = /usr/local/smsengine/etc/HybyteRef.p12
 https_pkcs12_password = h1zt34l6
 proxy = http://proxy:8080

 # e.g. for testing no proxy and:
 #https_xmlrpc_url = http://localhost:7778
 #https_xmlrpc_url = https://localhost:7778

 # server that receives xmlrpc from o2 and sends xml to tucd
 # localhost, port 8987 from tucd suggested
 # server/port settings will be mandated by configuration of 
 #   connection from o2 and firewall/routing of traffic from o2 
 smsinaddress = localhost
 smsinport = 8987

 # filter messages to restrict services on free shortcode
 smsinfilter = free_filter, non_free_filter, default_filter
 # messages from non_free or unknown shortcode (not matching any filter)
 # are unrestricted 
 # so we can actually do without non_free filter and default_filter
 # though if filters are misconfigured default behaviour being to make
 # available all services may not be desired

 [free_filter]
 object=filter
 # only allowed to do two actions on free shortcode, otherwise alphyra pays
 # shortcode/keywords can be exact strings or could be regular expressions
 shortcode = 353863833342
 smsinkeywords = REGISTER, VALIDATE
 failmessage = "Only REGISTER and VALIDATE services are available on this number. Send TUC text commands to 51101."
 # TODO action
 # TODO address/port

 [non_free_filter]
 object=filter
 shortcode = 353863832032
 smsinkeywords = .*

 [default_filter]
 object=filter
 shortcode = .*
 smsinkeywords = .*

 # this section must match tucd configuration in topup_club.ini
 #  [topup_club]
 #   ip = 0.0.0.0
 #   port = 8151
 #  [SMSservers]
 #   servers = localhost:8152


 [smsengine_tucd]
 object = inout
 # server that receives xml from tucd and sends xmlrpc to o2
 # sms out localhost, port 8152 from tucd suggested
 outaddress = localhost
 outport = 8152
 # sms in localhost, port 8151 from tucd suggested
 inaddress = localhost
 inport = 8151

=head1 charset

 perl

 o2

 Character $ & < > ¡ £ 
 XML Encoding &#036; &amp; &lt; &gt; &#161; &#163; 

 Character (space) ! " # $ % & ` ( ) * + , . / 0 ­ 9 : ; < = > ? @ A - Z ¡ £ 
 Unicode (hex) 20 - 2F 30 ­ 39 3A - 40 41 ­ 5A A1 A3 
 XML Encoding (space) ! " # &#036; % &amp; ` ( ) * + , . / 0 - 9 : ; &lt; = &gt; ? @ A - Z &#161; &#163; 

 Character ¤ ¥ § ¿ Ä Å Æ Ç É Ñ Ö Ø Ü à ä å æ è é ì ñ ò ö ø ù ü 
 Unicode (hex) A4 A5 A7 BF C4 C5 C6 C7 C8 D1 D6 D8 DC E0 E4 E5 E6 E8 E9 EC F1 F2 F6 F8 F9 FC 
 XML Encoding &#164; &#165; &#167; &#191; &#196; &#197; &#198; &#199; &#200; &#209; &#214; &#216; &#220; &#224; &#228; &#229; &#230; &#232; &#233; &#236; &#241; &#242; &#246; &#248; &#249; &#252;

 TODO: euro

 test/smsouttosmsengine.pl -d -v -x "<SMS><batchrequest><mobile>1234</mobile><msg>\`1234567890-=qwertyuiop[]asdfghjkl;'#\zxcvbnm,./!&quot;&pound;$%^&amp;*()_+{}:~&lt;&gt;?|</msg></batchrequest></SMS>"

 Error parsing XML request:  undefined entity at line 1, column 100, byte 100 
  at /usr/lib/perl5/site_perl/5.6.1/i386-linux/XML/Parser.pm line 187  
 request: <SMS><batchrequest><mobile>1234</mobile><msg>`1234567890-=qwertyuiop[]asdfghjkl;'#\zxcvbnm,./!&quot;&pound;$%^&amp;*()_+{}:~&lt;&gt;?|</msg></batchrequest></SMS>  
 Failure parsing xml :( no object referenceroot=,type=  

 [jamesc@bakunin smsengine]$ test/smsouttosmsengine.pl -d -v -x "<SMS><batchrequest><mobile>1234</mobile><msg>\`1234567890-=qwertyuiop[]asdfghjkl;'#\zxcvbnm,./!&quot;&pound;</msg></batchrequest></SMS>"

 [jamesc@bakunin smsengine]$ test/smsouttosmsengine.pl -d -v -x "<SMS><batchrequest><mobile>1234</mobile><msg>\`1234567890-=qwertyuiop[]asdfghjkl;'#\zxcvbnm,./!&quot;</msg></batchrequest></SMS>"

 DOESN'T like &pound;

 shouldn't like the $ but is okay

 test/smsouttosmsengine.pl -d -v -x "<SMS><batchrequest><mobile>1234</mobile><msg>\`1234567890-=qwertyuiop[]asdfghjkl;'#\zxcvbnm,./!&quot;$%^&amp;*()_+{}:~&lt;&gt;?|</msg></batchrequest></SMS>"

 res is -<SMS><response><errorcode>0</errorcode><msg>Success</msg><id>65168029</id><batch>1</batch></response></SMS>

TODO: do o2 proprietary char encoding in send function
<jamesc> it doesn't like the &pound; it wants &#163; instead of &pound;
<jamesc>  Character $ & < > ¡ £ 
<jamesc>  XML Encoding &#036; &amp; &lt; &gt; &#161; &#163; 

 test/smsouttosmsengine.pl -d -v -x "<SMS><batchrequest><mobile>353861953134</mobile><msg>\`1234567890-=qwertyuiop[]asdfghjkl;'#\zxcvbnm,./!&quot;$%^&amp;*()_+{}:~&lt;&gt;?|&#163;&#162;&#161;&#036;</msg></batchrequest></SMS>"
 test/smsouttosmsengine.pl -d -v -x "<SMS><batchrequest><mobile>353861953134</mobile><msg>\`1234567890-=qwertyuiop[]asdfghjkl;'#\zxcvbnm,./!\"@&quot;%^&amp;*()_+{}:~&lt;&gt;?|&#163;&#162;&#161;&#036;</msg></batchrequest></SMS>"

 => Frontier::RPC2::Base64=SCALAR(0x905b920) received on mobile

 Frontier is an old perl module for doing xmlrpc stuff

 these cause problems: &#163;&#162;&#161;&#036;

most success to date:

 test/smsouttosmsengine.pl -d -v -x "<SMS><batchrequest><mobile>353861953134</mobile><msg>\`1234567890-=qwertyuiop[]asdfghjkl;'#\zxcvbnm,./!\"@&quot;%^&amp;*()_+{}:~&lt;&gt;?|</msg></batchrequest></SMS>"

 Only quibble is that | is changed to @

 oops oops oops
 Nov  4 17:56:19 bakunin /USR.LOCAL/smsengine/bin[22943]: /USR.LOCAL/smsengine/bin: DEBUG: 1099590979 [term 127.0.0.1 52] SMS sms_batchrequest mobile0:353861953134msg: £ `1234567890-=qwertyuiop[]asdfghjkl;'#\zxcvbnm,./!"@"%^&*()_+{}:~<>?|  
 so MY scripts are translating &#163; to pound sign! ?
 or else XML Parser lib is.


TODO: fix above &#163; translation

=head1 COPYRIGHT

 Copyright (C) 2004 Doolin Technologies

=cut
