#  SmsFilter.pm
#    - providing a simple object for sms filtering across Doolin::SmsEngine objects
#
#  Copyright (C) 2004 Doolin Technologies
#
#  $Id: SmsFilter.pm,v 1.1 2004/11/26 17:42:07 jamesc Exp $
#

$VERSION = '0.00';

package SmsEngine::SmsFilter;

use strict;
use bytes;

sub new($%) {
    my $proto = shift;
    my $class = ref $proto || $proto;#

    my %args = @_;
    my $self = {};

    bless $self, $class;
}

sub loadsmsfilter {
    my $self = shift;
    my $parent = shift;
    # optional args
    for my $c (qw(smsinfilter)) {
	$self->{$c} = $parent->{cfg}->val( $parent->{account}, "$c" );
	if ($self->{$c}) {
	    $self->{$c} =~ s/^\s*//;
	    $self->{$c} =~ s/\s*$//;
	}
    }

    if (defined($self->{smsinfilter})){
	# load in filter settings
	$self->{smsinfilterprioritylist} = ();
        my @filters = split(/,/, $self->{smsinfilter});
	my @prilist;
        foreach my $filtername(@filters) {
	    $filtername =~ s/\s*//g;
	    my $thisfilter = \$self->{smsinfiltersettings}->{"$filtername"};

	    for my $fc (qw(object shortcode smsinkeywords failmessage
			   outaddress outport priority swallow
			   )) { # DONE: failmessage, priority TODO: action, address/port 
		${$thisfilter}->{$fc} = 
		    $parent->{cfg}->val("$filtername", "$fc" );
                if (${$thisfilter}->{$fc}) {
		    ${$thisfilter}->{$fc} =~ s/^\s*//;
		    ${$thisfilter}->{$fc} =~ s/\s*$//;
		}
	    }

            # keep track of priorities set so we can loop through 
            # in priority order later
	    my $pri = ${$thisfilter}->{priority};
	    if (defined $pri) {
		push @prilist, $pri;
	    }

	    my @keywords = 
		split(/,/, 
		      ${$thisfilter}->{smsinkeywords});
	    my $i=0;
	    while (defined($keywords[$i])) {
		$keywords[$i] =~ s/\s*//g;
		$i++;
	    }		   
	    # put keywords regexp for easy grepping later
	    ${$thisfilter}->{smsinfilterkeywordregexp} =
		"(" . join("|",@keywords) . ")"; 

	}

        # remove duplicate priorities
	# read perldoc -q array 
        # How can I remove duplicate elements from a list or array?
        # this also sorts the priorities
	my %saw;
	@{$self->{smsinfilterprioritylist}} = grep(!$saw{$_}++, @prilist);
	#use Data::Dumper;
	#print STDERR "pri list" . 
	#    Data::Dumper->Dumper($self->{smsinfilterprioritylist}) . "\n";
	    

    }
}

sub showfilterconfig {
    my $self = shift;

    if (defined($self->{smsinfiltersettings})) {

	#use Data::Dumper;
	#print STDERR "Dump self->{smsinfiltersettings}:". 
	#    Data::Dumper->Dumper($self->{smsinfiltersettings}) . "\n";

       foreach my $filterset(keys(%{$self->{smsinfiltersettings}})) {
	   #print STDERR "Dump filterset " .
	   #    Data::Dumper->Dumper($filterset) . "\n";
	   my $thisfilter = \$self->{smsinfiltersettings}->{$filterset};

	   print "XiamSms arg filter $filterset .oO0Oo.".
	       ${$thisfilter}."\n";

	   foreach my $filterthing(keys(%{${$thisfilter}})) {
	       print "  $filterthing = " . 
		   ${$thisfilter}->{$filterthing}."\n"
		   if (defined(${$thisfilter}->{$filterthing}));
	   }
	}
   }
}

sub applyfilters {
    my $self = shift;
    my $shortcode = shift;
    my $message = shift;
    #my $network = shift;

    my $smsinfiltersettings = $self->{smsinfiltersettings};

    #DONE: priority, from 1 up
    foreach my $priority (@{$self->{smsinfilterprioritylist}}, ( undef )) {

	foreach my $filtername(keys(%{$smsinfiltersettings})) {
	    my $thisfilter = \$smsinfiltersettings->{$filtername};
	    
	    # if priorities not defined default behaviour is to check filter
	    if ((!defined($priority) && !defined(${$thisfilter}->{priority})) || 
		defined($priority) && defined(${$thisfilter}->{priority}) &&
		${$thisfilter}->{priority} == $priority) {

		print STDERR "filter check pri: $priority name: $filtername\n" 
		    if ($self->{debug} || $self->{verbose});

		print STDERR $shortcode . "=~" . ${$thisfilter}->{shortcode} . 
		    ".oO0Oo.".
		    "grep " . ${$thisfilter}->{smsinfilterkeywordregexp} . " " .
		    $message . "\n" if ($self->{verbose});

		print STDERR "if " . ($shortcode =~ ${$thisfilter}->{shortcode}) . "&&" .
		    ($message =~ /${$thisfilter}->{smsinfilterkeywordregexp}/) .
		    "\n" if ($self->{verbose});

		# If shortcode matches and keywords don't match 
                # then reject with fail message 
		if ($shortcode =~ ${$thisfilter}->{shortcode}) {
		    if ($message =~ /${$thisfilter}->{smsinfilterkeywordregexp}/) {
		    # grep in scalar context returns number of matches
		    #grep(/${$thisfilter}->{smsinfilterkeywordregexp}/, $message) == 0
		    #$message =~ ${$thisfilter}->{smsinkeywords}
			print STDERR "filter $filtername match and ACCEPT\n" 
			    if ($self->{verbose});
			return (1,
				$filtername,
				undef,
				${$thisfilter}->{outaddress},
				${$thisfilter}->{outport});
		    } else {
			print STDERR "filter $filtername match and REJECT\n" 
			    if (defined ${$thisfilter}->{failmessage} && $self->{verbose});
			# by returning we are swallowing (catching) message and
                        # not applying any more filters
			return (1,
				$filtername,
				${$thisfilter}->{failmessage},
				${$thisfilter}->{outaddress},
				${$thisfilter}->{outport});
		    }
		}
	    }
	}
    }
    return (0, undef, undef, undef);
}
