#  XiamSms.pm
#    - providing a simple object for send/receive sms using 
#      o2 and xiam proprietary XML http post interface, Doolin::SmsEngine object
#      compatable
#
#  Copyright (C) 2004 Doolin Technologies
#
#  $Id: XiamSms.pm,v 1.11 2005/04/27 10:07:12 jamesc Exp $
#

$VERSION = '0.00';

package SmsEngine::XiamSms;

# TODO: make XiamSms class an SmsEngine
#       I think it would make things more straight forward
#use SmsEngine::SmsEngine;
#@ISA = ("SmsEngine");
#1;

use SmsEngine::SmsFilter;
@ISA = ("SmsEngine::SmsFilter");
1;

use strict;
use bytes;
use DBI;
#use Digest::MD5  qw(md5 md5_hex md5_base64);
#use LWP::UserAgent;

sub new($%) {
    my $proto = shift;
    my $class = ref $proto || $proto;#

    my %args = @_;
    my $self = {};

    $self->{debug} = ($args{debug})?$args{debug}:0;
    $self->{verbose} = ($args{verbose})?$args{verbose}:0;#
    $self->{replytext} = "";

    bless $self, $class;
}

# can be called b4 $self is blessed
sub loadaccount{
    my $self = shift;
    my $parent = shift;

    $self->{parent} = $parent; # naughty
    $self->{cfg} = $parent->{cfg}; # naughty naughty
    $self->{debug} = $parent->{debug}; 
    $self->{verbose} = $parent->{verbose}; 

    # read in mandatory config vars
    for my $c (qw(url providerid from
		  smsinaddress smsinport database
		  )) {

	$self->{$c} = $parent->{cfg}->val( $parent->{account}, "$c" );

	#we replace whitespace at start and end of vars (in particular port numbers were affected)
        # better to fix by fixing ReadConfig.pm library but will save trouble by making
        # sure is okay here too
	if ($self->{$c}) {
	    $self->{$c} =~ s/^\s*//;
	    $self->{$c} =~ s/\s*$//;
	} else {
	    return { error => 1,
		     errors => [ "XiamSms loadaccount argument $c is mandatory.\n" ] };
	}
    }

    # set optional arg defaults
    $self->{timeout} = 160;

    for my $c (qw(timeout)) {
	$self->{$c} = $parent->{cfg}->val( $parent->{account}, "$c" );
	if ($self->{$c}) {
	    $self->{$c} =~ s/^\s*//;
	    $self->{$c} =~ s/\s*$//;
	}
    }

    # get a machine id thing
    use Socket;
    use Sys::Hostname;
    my $host = hostname();
    print "host $host\n";
    my $addr = inet_ntoa(scalar gethostbyname($host || 'localhost'));
    $addr =~ s/\.//g;
    my $mhid = substr($addr,2,2) . substr($addr,-2);
    # multiple instances possible on machine so add random    

    my $r = int(rand(100));

    # initialise 
    $self->{msgsrvid} = $mhid.$r;
    $self->{msgseq} = 0;

    # SmsFilter
    $self->loadsmsfilter($parent);

    return { error => 0 };

}

# showconfig is also an example of how to access config settings
sub showconfig {

    my $self = shift;

    for my $c (qw(debug verbose
		  url providerid from
		  smsinaddress smsinport 
		  database dbtype dbserver dbname dbuser dbpass
		  smsinfilter smsinfiltersettings
		  smsinfilterprioritylist
		  )) {
	print "XiamSms arg $c = " . $self->{"$c"} . "\n" 
	    if (defined($self->{"$c"}));
    }

    $self->showfilterconfig();

}


# response from sendmessage, sendsimplemessage or sendbatchmessage
sub printsendresponse {
    my $self = shift;
    my $res = shift;
    my $all = shift;

#<?xml version="1.0" encoding="UTF-8"?>
#  <!DOCTYPE xiamSMS SYSTEM "xiamSMSMessage.dtd">
#  <xiamSMS>
#    <deliverResponse id="miaow"><result status="OK">+353119898989</result></deliverResponse>
#  </xiamSMS>

#<?xml version="1.0" encoding="UTF-8"?>
#<!DOCTYPE xiamSMS SYSTEM "xiamSMSMessage.dtd">
#<xiamSMS status="OK" statusText="XML contained 1 xir message">
#  <submitResponse id="5tZmOhbAgwiK/dWsIpVSzA">
#    <result status="OK" statusText="">+to</result>
#  </submitResponse>
#</xiamSMS>

# CHOICE is wonderful: 
# 1. whonk out params by regexps
# 2. use XML::SimpleObject

    print "RESULT $res RESULT\n";
    print "id:".$res->{RequestID}." " if (defined($res->{RequestID}));
    print "errcode:".$res->{ErrorCode}." " if (defined($res->{ErrorCode}));
    print "errmsg:".$res->{ErrorMessage}." " if (defined($res->{ErrorMessage}));

    my $id = m/<deliverResponse\s+id=\"([^\"+])\"/;
    my $status = m/<result\s+status=\"([^\"+])\"/;
    my ($errcode, $errmsg) = m/^([0-9])+\s(.+)$/;

    #SOAP::Data->type(base64 => $string)
    print "id:".$id." " if (defined($id));
    print "status:".$status." " if (defined($status));
    print "errcode:".$errcode." " if (defined($errcode));
    print "errmsg:".$errmsg." " if (defined($errmsg));

    print "\n";
}

sub logsendresponse {
    my $self = shift;
    my $logfile = shift;
    my $res = shift;

    if ( ! open(LOGFILE, ">>$logfile") ) {
	# failing to open log is not an executable offence
	print STDERR "ERROR: Couldn't open $logfile $!";
    } else {
	my $logentry;
	$logentry = "\$ts=" . time() . ";";
	$logentry .= "\$reqid=" . $res->{RequestID} . ";" 
	    if (defined($res->{RequestID}));
	$logentry .= "\$errorcode=" . $res->{ErrorCode} . ";";
	$logentry .= "\$errormessage=" . qq($res->{ErrorMessage}) . ";";

	my $fi=0;
	while (defined($res->{Failed_MSISDN}->[$fi])) {
	    $logentry .= "fault[$fi]=";
	    $logentry .= "{ errorcode => ".$res->{Failed_MSISDN}->[$fi]->{errorcode}.",";
	    $logentry .=  "num => ".qq($res->{Failed_MSISDN}->[$fi]->{num})."};";
	    $fi++; # check for a next element!
	}
	$logentry .= "\n";
	print LOGFILE $logentry;
	close(LOGFILE);
    }   
}

# response from querysenddeliverystatus, querybatchrequest or queryaccountrequest
sub printqueryresponse {
    my $self = shift;
    my $res = shift;

    print "RESULT $res RESULT\n";

    print "TotalNumberOfMessages:".$res->{TOTALNUMBEROFMESSAGES}." ";
    print "MessagesSent:".$res->{MESSAGESSENT}." ";
    print "MessagesSkipped:".$res->{MESSAGESSKIPPED}." ";
    print "StartDate:".$res->{STARTDATE}." ";
    print "Message:".$res->{MESSAGE}." ";
    print "Status:".$res->{STATUS}." ";
    print "Status (smsengine):".$res->{smsengine_status}." ";
    print "Status info:".$res->{smsengine_status_info}." ";

    print "ErrorCode:".$res->{ErrorCode}." ";
    print "ErrorMessage:".$res->{ErrorMessage}." ";

    print "\n";

}

sub _make_xiam_xml {
    my $self = shift;
    my $msg = shift;
    my $xiamSMSmsg = <<EOT
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE xiamSMS SYSTEM "xiamSMSMessage.dtd">
<xiamSMS>$msg</xiamSMS>
EOT
;
    return $xiamSMSmsg;
}

sub _make_xiam_xml_err {
    my $self = shift;
    my $msg = shift;
    my $resp = <<EOT
<error>
 <result status="ERROR">$msg</result>
</error>
EOT
;
    return _make_xiam_xml($resp);    
}


sub _make_msg_id {
    my $self = shift;

    # to make life simpler we constrain msg id to something that will 
    # fit into a mysql int(11) unsigned (2^10ish)
    # so that we do not have to support a mapping from longer ids to constrained ids


    # id o2 mms was always numeric
    # tests & compat break unless we just use a number
    # timestamp based?
    # multiple per sec same id => timestamp + random ?  
    #   or track a sequence so by session+machine+sequence


    # use MD5 as the id
    #my $data = $ts;
    #use Digest::MD5  qw(md5 md5_hex md5_base64);
    #my $digest = md5_base64($data);
    #my $digest = "doolin".$ts;
    #$id = $digest;

    # id based partially on machine ip, process seq no, timestamp
    my $ts = time();
    my $id = $ts & 0xfffff; 
    # 0xffff 2^16 = 65536 /60 = 1092mins /60=18hrs
    # 0xfffff 2^20 = 1048575 12 days
    # 0xffffff 2^24 = 16777215 194 days (not unique on second)
    $id = $self->{msgsrvid} . $self->{msgseq}++ . $id;
    $self->{msgseq} %= 1000;

    # constrain id size
    $id = substr($id,-9);
    
    print "id $id\n" if ($self->{debug});
    return $id;
}

# server
sub receivesms {
    my $self = shift;
}

sub xiamsmsxmlreceive {
    my $self = shift;
}

# use user agent object LWP for making HTTP POST
use LWP::UserAgent;

sub _xiamsmsxmlsend {
    my $self = shift;
    my $xiamSMS = shift;
    my $messageid = shift || $self->_make_msg_id();    # for testing
    my $xiamProviderID = shift || $self->{providerid}; # for testing
    my $url = shift || $self->{url};                   # for testing

    my $ua = LWP::UserAgent->new;
    $ua->agent("DoolinSmsEngine/1.1 (penguin powered)");

# sms out to xiam
# IPs: 193.95.160.218, 193.95.160.219, 193.95.160.223
# Port: 8080
# URL: /smsxml/* nope /smsxml/collector
# 
# 1. All are production. 223 is a virtual IP that resolves to either 218 or
# 219. For the moment, please go straight to 218. We will advise in future to
# move to 223.
# 3. As these are live systems, we just have to pick a shortcode and set up
# your keywords on that code. Shay: I'll use 53125, unless you have any
# objections. keywords TUC, TUCTEST, TUCDEV
# 7. An ID identifies an account, which has one URL for MO traffic, so if you
# want MOs to go to 3 different URLs, depending on keywords, I will have to
# set up 3 accounts for you. You can then choose to send from all 3 or could
# do all MT through one account only.
# Your IDs are:
 
# TUC: 128
# TUCDEV: 129 (routed to TUCTEST URL for the moment)
# TUCTEST: 130
 
    my $fullxiamSMS = $self->_make_xiam_xml($xiamSMS);

    print $fullxiamSMS if ($self->{debug});
    print "URL: ", $self->{url}, "\n" if ($self->{debug});
    print "X-XIAM-Provider-ID: ", $xiamProviderID, "\n" if ($self->{debug});

    my $req = HTTP::Request->new(POST => $url);

    $req->content_type('text/xml');
    $req->content($fullxiamSMS);

    $req->header("X-XIAM-Provider-ID", $xiamProviderID);

    # Pass request to xiam and get a response back
    my $timeout = 180 if (!defined($self->{timeout}));
    $ua->timeout($timeout);
    my $res = $ua->request($req);

    #500 Serverside error locating specified provider entry, it cannot be found.
    # this means our provider id used (100) not valid

    # Thu Nov 25 13:52:15 GMT 2004
    #  POST /smsxml HTTP/1.1
    # 219 500 Can't connect to 193.95.160.219:8080 (connect: Connection refused)
    # 218 302 Moved Temporarily
    # 223 302 Moved Temporarily
  
    #  smsxml/*
    # 404 error

    use Data::Dumper; print Dumper($res) if ($self->{debug});

    # Check the outcome of the response
    if ($res->is_success) {
	print $res->content, "\n";
    } else {
	print $res->status_line, "\n";
    }

    $res->{RequestID} = $messageid;

    # TODO map results to smsengine API
    if ($res->is_success) {
	$res->{ErrorCode} = 0;
	$res->{ErrorMessage} = "Success";
    } else {
	$res->{ErrorCode} = 1;
	$res->{ErrorMessage} = $res->status_line;
    }


    $res->{messageid} = $messageid;
    ($res->{status},
     $res->{errorcode},
     $res->{errormessage},
     $res->{startdate},
     $res->{totalnumberofmessages},
     $res->{messagessent},
     $res->{messagesskipped}) = (2,0,"",0,0,0,0);
    my $xiamifaceid = 0; # ngeh
    my $ldr = $self->logdeliverreportindb($res,$xiamifaceid);
    if ($ldr) {
	warn "log message send failed?: $ldr";
    }

    return $res;
}

sub sendbatchmessage {
    my $self = shift;
    my $h = shift;

    if (!$h->{text} || !$h->{to}) {
	die "sendbatchmessage: text and to params are mandatory";
    }

    my $messageid;
    if ($h->{RequestID}) {
       $messageid = $h->{RequestID};
    } else {
	$messageid = $self->_make_msg_id();
    }

    my $to = $h->{to};
    my $text = $h->{text};
    my $from = $h->{from} || $self->{from} || "XiamSms.pm";

    my $xiamSMS = "<submitRequest id=\"$messageid\"><from>$from</from>\n";

    foreach my $j (@$to) { 
	$xiamSMS .= "      <to>".$j."</to>\n";
    }

    $xiamSMS .= "<content type=\"text\">$text</content>" .
	"<requestDeliveryReport value=\"yes\" />" .
	"</submitRequest>";

    #die "xiamsms sendbatchmessage not done yet";

    my $res = $self->_xiamsmsxmlsend($xiamSMS,$messageid);

    $self->printsendresponse($res) if ($self->{verbose});

    return $res;
}

sub querysenddeliverystatus {
    my $self = shift;
    my $h = shift;

    my ($error, $msgstatus) = $self->querymessagestatus($h->{RequestID});
    #die "xiamsms querysenddeliverystatus not done yet";

    #use Data::Dumper; print STDERR "xiam querysenddeliverystatus stat " . Dumper($msgstatus) if ($msgstatus);
    #use Data::Dumper; print STDERR "xiam querysenddeliverystatus err " . $error if ($error);

    #print STDERR "xiam querysenddeliverystatus stat->[0] " . Dumper($msgstatus->[0]) if ($msgstatus);
    #print STDERR "xiam querysenddeliverystatus stat->[1] " . Dumper($msgstatus->[1]) if ($msgstatus);
    #print STDERR "xiam querysenddeliverystatus stat->[0]->[0] " . Dumper($msgstatus->[0]->[0]) if ($msgstatus);
    #print STDERR "xiam querysenddeliverystatus stat->[0]->[1] " . Dumper($msgstatus->[0]->[1]) if ($msgstatus);
    #print STDERR "xiam querysenddeliverystatus stat->[0]->[2] " . Dumper($msgstatus->[0]->[2]) if ($msgstatus);
    #print STDERR "xiam querysenddeliverystatus @{$msgstatus->[0]}[0..2] " . Dumper(@{$msgstatus->[0]}[0..2]) if ($msgstatus);

    my $res;
    
    if ($error) {
	$res->{ErrorCode} = 1;
	$res->{ErrorMessage} = $error;

    } else {

	($res->{smsengine_status},
	 $res->{smsengine_status_info},
	 $res->{smsengine_messageid},
	 $res->{smsengine_ts},
	 $res->{smsengine_ifaceid}) = @{$msgstatus->[0]}[0..4];

	($res->{STATUS},
	 $res->{ErrorCode},
	 $res->{ErrorMessage},
	 $res->{STARTDATE},
	 $res->{TOTALNUMBEROFMESSAGES},
	 $res->{MESSAGESSENT},
	 $res->{MESSAGESSKIPPED}) = @{$msgstatus->[0]}[5..11]; #$msgstatus->[0]->[3..9];

	$res->{MESSAGE} = "unknown";
    }

    return $res;
}

sub querybatchrequest {
    my $self = shift;
    my $h = shift;
    return $self->querysenddeliverystatus($h);
    #die "xiamsms querybatchrequest not done yet";
    #return $res;
}


sub sendsms {

    my ( $self, $to, $from, $text, $messageid ) = @_;

    $from = $self->{from} if (!defined $from);

    # $to can be xml array of to numbers already or just one number
    if ($to !~ qr!^\<to\>.*\<\/to\>$!) {
	$to = "<to>$to</to>";
	print STDERR "adding <to> tags $to\n" if ($self->{debug});
    }

    $messageid = $self->_make_msg_id() if (!defined $messageid);

    my $xiamSMS = <<EOT
    <submitRequest id="$messageid">
      <from>$from</from>
      $to
      <content type="text">$text</content>
      <requestDeliveryReport value="yes" />
    </submitRequest>
EOT
;

    my $res = $self->_xiamsmsxmlsend($xiamSMS,$messageid);

    $self->printsendresponse($res) if ($self->{verbose});

    return $res;
}

sub sendmessage {
    my $self = shift;
    my $h = shift;
    #$h->{from} can be undef
    #$h->{messageid} can be undef

    if (!$h->{text} || !$h->{to}) {
	die "sendmessage: text and to params are mandatory";
    }

    # additionally $h->{to} must be an array with >0 elements
    my $size = scalar(@{$h->{to}});
    if (!$size) {
	die "sendmessage: $h->{to} must be an array with >0 elements";
    }

    my $tonums = "";
    my $i=0;
    while (defined($h->{to}->[$i])) {
	#$param->{MSISDN}->[$i]->{num} = $h->{to}->[$i];
	$tonums .= "<to>".$h->{to}->[$i]."</to>";
	$i++;
    }

    my $res = $self->sendsms( $tonums, $h->{from}, $h->{text}, $h->{messageid} );

    return $res;
}

sub encode_chars_for_tx {

# Character (space) ! " # $ % & ` ( ) * + , . / 0 ­ 9 : ; < = > ? @ A - Z ¡ £ 
# Unicode (hex) 20 - 2F 30 ­ 39 3A - 40 41 ­ 5A A1 A3 
# XML Encoding (space) ! " # &#036; % &amp; ` ( ) * + , . / 0 - 9 : ; &lt; = &gt; ? @ A - Z &#161; &#163; 

# Character ¤ ¥ § ¿ Ä Å Æ Ç É Ñ Ö Ø Ü à ä å æ è é ì ñ ò ö ø ù ü 
# Unicode (hex) A4 A5 A7 BF C4 C5 C6 C7 C8 D1 D6 D8 DC E0 E4 E5 E6 E8 E9 EC F1 F2 F6 F8 F9 FC 
# XML Encoding &#164; &#165; &#167; &#191; &#196; &#197; &#198; &#199; &#200; &#209; &#214; &#216; &#220; &#224; &#228; &#229; &#230; &#232; &#233; &#236; &#241; &#242; &#246; &#248; &#249; &#252;

    tr/[\x00-\x1F]//; # rm ctrl chars  (this doesn't work?)
    tr/[\x00..\x1F]//; # rm ctrl chars  (this doesn't work?)
    s/[[:cntrl:]]/CTL/gs; # try this instead?

    # Character $ & < > ¡ £ 
    # XML Encoding &#036; &amp; &lt; &gt; &#161; &#163; 
    
    # &amp; MUST be done first.
    s/&/&amp;/g;
    s/\$/&#036;/g;
    s/</&lt;/g;
    s/>/&gt;/g;
    s/¡/&#161;/g;
    s/£/&#163;/g;

    # TODO: euro ?

}


# dbase in XiamSms perl module

# need to initdb before querymessagestatus or logdeliverreportindb

# should connect and disconnect immediately on initdb call to verify connection & configuration
# get sensible errors if dbase does a funny and we try to connect
# don't get sensible errors if dbase does a funny and we are using already connected handle

sub initdb {
    my $self = shift;
    my $dbh;

    if (defined $self->{database}) {
	# load database config

	# read in config vars
	for my $c (qw(dbtype dbserver dbname dbuser dbpass)) {
	    $self->{$c} = $self->{cfg}->val( $self->{database}, "$c" );
	    #we replace whitespace at start and end
	    if (defined($self->{$c})) {
		$self->{$c} =~ s/^\s*//;
		$self->{$c} =~ s/\s*$//;
	    } else {
		return "XiamSms initdb config file arg $c is mandatory.\n";
	    }
	}

    } else {
	return "no database defined. config error?";
    }

    if (!defined $self->{dbh}) {

	#print STDERR "DBI:$self->{dbtype}:$self->{dbname}:$self->{dbserver}\n";
	#print STDERR "DBI:$self->{dbuser},$self->{dbpass}\n";

	$self->{dbh} = 
	    DBI->connect("DBI:$self->{dbtype}:$self->{dbname}:$self->{dbserver};mysql_ssl=0;",
			 $self->{dbuser},$self->{dbpass}) or
			 return "Could not connect to the smsengine database: $!".
			 $self->{dbname};
    }

    return 0; # success
}

sub closedb {
    my $self = shift;
    $self->{dbh}->disconnect();
    delete $self->{dbh}; # or undef?
    return 0;
}

# store message status
sub logdeliverreportindb {
    my $self = shift;
    my $dr = shift;
    my $ifaceid = shift;
    my $ts = time();

    my $xiamifaceid = 0; # ngeh

    my $dbres = $self->initdb();
    if ($dbres) {
	return "Fail to init db: $dbres";
    }

    my $SQL= "REPLACE INTO msg_status ".
	"(messageid, timestamp, ifaceid, ifacestatus, ".
	" errorcode, errormessage, startdate, totalnumberofmessages, messagessent,  messagesskipped) ". 
	"VALUES (?,?,?,?,?,?,?,?,?,?)";
    print STDERR $SQL if ($self->{debug} && $self->{debug} =~ /sql/);   

    my $sth = $self->{dbh}->prepare($SQL);
    return "error preparing sql $SQL" if (!defined($sth));

    $sth->execute($dr->{messageid},
		  $ts,
		  # ifaceid, ifacestatus,
		  # errorcode, errormessage, startdate, totalnumberofmessages, messagessent,  messagesskipped 
		  $xiamifaceid,
		  $dr->{status},
		  $dr->{errorcode},
		  $dr->{errormessage},
		  $dr->{startdate},
		  $dr->{totalnumberofmessages},
		  $dr->{messagessent},
		  $dr->{messagesskipped});
    return "ERROR: updating message blah db: ".
	    $sth->errstr if ($sth->err);

#    while ($r = pop(@$userstat)) {
#	$tucsth->execute(@$r);
#	mycroak("ERROR: updating user 3v status into tuc db: ".
#		$tucsth->errstr) if ($tucsth->err);
#	print "tuc UPDATE user SET allow_3V = $r->[0] WHERE card_no = $r->[1];\n";
#    }

#    print "tuc ".$tucsth->rows." updated\n" if ($debug);

#    #warn("no matching thing?") if ($sth->rows <= 0);
#    #my $rows = $sth->fetchall_arrayref();

    $sth->finish();
    $self->closedb();

    # 0 = success
    return 0;
}

sub querymessagestatus {
    my $self = shift;
    my $id = shift;

    return ( "ERROR: no id" ) if (!defined $id);

    my $xiamifaceid = 0; # ngeh

    my $dbres = $self->initdb();
    if ($dbres) {
	return ( "Fail to init db: $dbres" );
    }

    my $SQL= "SELECT msd.status, msd.status_info, ms.* ".
	"FROM msg_status ms, msg_status_desc msd ".
	"WHERE ms.messageid = \"$id\" ".
	"AND msd.ifacestatus = ms.ifacestatus ".
	"AND msd.ifaceid = ms.ifaceid;";
    print STDERR $SQL if ($self->{debug} && $self->{debug} =~ /sql/);   

    my $sth = $self->{dbh}->prepare($SQL);
    return ( "error preparing sql $SQL" ) if (!defined($sth));

    $sth->execute;
    return ( "ERROR: querying db: ".
	    $sth->errstr ) if ($sth->err);

    warn("no matching thing $id $SQL?") if ($sth->rows <= 0);
    return ( "no matching thing $id $SQL?" ) if ($sth->rows <= 0);
    warn("$sth->rows matching things (only want one) $id $SQL?") if ($sth->rows > 1);
    my $rows = $sth->fetchall_arrayref();

    $sth->finish();
    $self->closedb();

    # 0 = success
    return ( 0, $rows );
}



my $xiamerrorcodes = {
    0    => "Success",
};

sub maperrorcode {
    my $self = shift;
    my $errorcode = shift;
    return $xiamerrorcodes->{$errorcode};
}

sub setreplytext {
    my $self = shift;
    $self->{replytext} = shift;
}

sub getreplytext {
    my $self = shift;
    return $self->{replytext};
}


########################################

use HTTP::Daemon;
use HTTP::Status;

my ($MSG_ERR, $MSG_DBG, $MSG_INFO, $MSG_CRIT, $MSG_EMERG) 
    = ('warning', 'debug', 'info', 'crit', 'emerg');

sub debug {
    my $self = shift;
    return $self->{debug};
}

#my $debug = SmsEngine::O2ProprietarySms::debug();

sub stopinservice {
    my $self = shift;
    undef $self->{daemon}; 
}

sub startinservice {
    my $self = shift;
    my $o2port = shift;

    # callback interface
    $self->{poe_kernel} = shift;
    $self->{forwardmessageref} = shift;
    $self->{logeventref} = shift;

    # call like &$forwardmessageref(....)

    if (defined($self->{daemon})) {
	warn "in service daemon already started. NOT starting another.";

    } else {

	$self->{daemon} = HTTP::Daemon
	    -> new (LocalPort => $o2port, 
		    Reuse => 1)
	    ;
    }

    # modules should not die. this on fail to start daemon so caller should die 
    return "fail to create HTTP::Daemon: $!"
	if (!defined($self->{daemon}) || !$self->{daemon});
 
    $self->logevent($MSG_DBG, "Active server (xiam http xml/sms flavour) at " . $self->{daemon}->url);

    # TODO: this handler doesn't work beautifully with POE
    # POE likes things to do a bit and then return fast
    # but this server does only want to do this one thing
    # but behaviour not beautiful as signals not caught:
    #  - kill -9 must be used, Ctrl-C not caught, ...
    $self->handler();

    # not reached!!!
    return $self->{daemon};
}

# callback callers
sub logevent {
    my $self = shift;
    #$self = $SmsEngine::O2ProprietarySms::SELFREF if (!$self);
    #use Data::Dumper; print Dumper($self);
    #use Data::Dumper; print Dumper(@_);
    if (defined($self->{logeventref})) {
	return &{$self->{logeventref}}(@_);
    } else {
	print STDERR "WARNING: logevent not set and $1 $2 $3 $4\n";
    }
}

sub forwardmessage {
    my $self = shift;
    #$self = $SmsEngine::O2ProprietarySms::SELFREF if (!$self);
    return &{$self->{forwardmessageref}}(@_);
}

sub poe_kernel_var {
    my $self = shift;
    #$self = $SmsEngine::O2ProprietarySms::SELFREF if (!$self);
    return $self->{poe_kernel};    
}


sub logsmsin {
    my $self = shift;
    my $msg = shift;
    # use Carp;
    #open(LOG, ">>xiamin.log") || croak "can't open log: $!";
    #print STDERR $msg if (defined($debug));
    #print LOG $msg."\n";
    #close LOG;
    $self->logevent($MSG_DBG, $msg);
}

sub psmserr {
    my $self = shift;
    my $msg = "Error: " . shift;
    print STDERR $msg . "\n";
    $self->logsmsin($msg);
}

sub handler { 
    my $self = shift;

    while (my $c = $self->{daemon}->accept) {

	my $t = time();
	$self->logsmsin ("ACCEPT:" . $t . "\n");
	while (my $r = $c->get_request) {
	    my $t = time();
	    $self->logsmsin ("REQUEST:" . $t . "\n");
	    use Data::Dumper; print Dumper($r);
	    use Data::Dumper; $self->logsmsin(Dumper($r));

	    if ($r->method eq 'GET' and $r->url->path eq "/xyzzy") {
		$self->logsmsin ("SEND FILE FUNNYPATH");
		$c->send_file_response("o2xiamreceive.pl");
		# $c->send_response("TODO: simulate o2 xiam sms out server");	    

	    } elsif ($r->method eq 'POST' and $r->url->path eq "/smsxml" 
		     or $r->url->path eq "/sms/xiam/in") {
		my $response = $self->parse_xiam_xml($r->content);
		my $t = time();
		$self->logsmsin ("SEND RESPONSE $t $response");

		$c->send( $response);
		$c->close;#$c->force_last_request;

                # success resp HTTP/1.1 <content> to perl (sends HTTP/1.1 in front)	    
		# xiam doesn't like
		#$c->send_response($response); 

		#} elsif ($r->method eq 'POST') {
		#    $c->send_response($errorresponse);	    
	    } else {
		$self->logsmsin ("SEND ERROR FORBIDDEN");
		$c->send_error(RC_FORBIDDEN);
	    }
	}
	# do not close immediately?
	#$t = time();
	#$self->logsmsin ("SLEEP:" . $t . "\n");
	#sleep(2);
	#my $r = $c->get_request;
	#use Data::Dumper; print Dumper($r);
	#use Data::Dumper; $self->logsmsin(Dumper($r));
	#$c->send_response("\n\n\n");	    
	#$t = time();
	#$self->logsmsin ("SLEEP:" . $t . "\n");
	#sleep(2);

	$t = time();
	$self->logsmsin ("CLOSE:" . $t . "\n");
	$c->close;
	undef($c);
    }
}


#use XML::Parser;
use XML::SimpleObject;

sub childval {
    my $self = shift;
    my $ref = shift;
    return $ref->value if (defined($ref));
}

sub childattrval {
    my $self = shift;
    my $type = shift;
    my $ref = shift; # $ref can be undef so must be after last definately defined thing
    #print Dumper($ref);
    return $ref->attribute($type) if (defined($ref) && $ref->attributes);
}

sub walkchildren {
    my $self = shift;
    my $ref = shift;
    my $path = shift || "";
    my $i = shift || 0;

    #use Data::Dumper; print STDERR "Dump:".Dumper($ref) if ($i == 0);
    #print "i $i\n";
    #use Data::Dumper; print STDERR "Dump:".Dumper($ref);
    if (defined($ref)) {
	# sttributes
	if ($ref->attributes) {
	    my %attrs = $ref->attributes;
	    while ( my ($key, $value) = each %attrs) {
		print "path: $path attr: $key:".$value."\n";
	    }
	}
	# children
	foreach my $child ($ref->children) {
	    print "path: $path/child: ", $child->name, ": ", $child->value, "\n" if ($child->value);
	    walkchildren($child,$path."/".$child->name,++$i);
	}
    }
}

#my ($root, $type, $object, $msgid, $ref) = parse_xiam_xml($input);
sub parse_xiam_xml {
    my $self = shift;
    my ($request) = @_;
    chomp($request);
    my $ref;

    # ngeh
    $request =~ s/<\?xml\s[^>]+>//;
    $request =~ s/<\!DOCTYPE\s[^>]+>//;
    #$request =~ "<xml>".$request."</xml>";

    my $parser = new XML::Parser(ErrorContext => 2, Style => "Tree");

    # ensure server doesn't die
    # silly silly XML::Parser calls die

    eval {
	$ref = new XML::SimpleObject($parser->parse($request));
    };

    # TODO check for error
    # handle exception errors
    if ($@) {
        $self->psmserr("parse_xml Error parsing XML request ".
		"( $request ) ( $@ ) invalid format.");

	my $xiamresp = $self->_make_xiam_xml("Error parsing XML request");
        return $xiamresp;
    }

    walkchildren($ref);

    #print Dumper($request);

    # validation
    if (!defined($ref->child("xiamSMS"))) {
        $self->psmserr("parse_xml Error in XML request ".
		"- invalid or no root element. ".
		" Valid Root elements are: xiamSMS");
        #logevent($MSG_DBG, XMLout($ref, KeepRoot => 1));
	my $xiamresp = $self->_make_xiam_xml("Error parsing XML request: no xiamSMS root element");
        return $xiamresp;
    }

    #use Data::Dumper; print STDERR "Dump:".Dumper($ref);

    # SMS
    if (defined($ref->child("xiamSMS")->child("deliverRequest"))) {

	my $dr = $ref->child("xiamSMS")->child("deliverRequest");
	my ($id, $to, $from, $xmid) = ("", "", "", "");

	#print Dumper($dr->attributes);;
	$id = $dr->attribute("id");
	#$from = $dr->child("from")->value if (defined($dr->child("from")));
	$from = $self->childval($dr->child("from"));
	$to = $self->childval($dr->child("to"));
	$xmid = $self->childattrval("value", $dr->child("xirMessageID"));
	print "from: $from, to: $to, id: $id, XMID: $xmid\n";

	my $rog = $self->childattrval("value", $dr->child("receivedOnGroup"));
	my $udh = $self->childattrval("value", $dr->child("udh"));
	my $ct = $self->childattrval("value", $dr->child("carrierType"));
	my $sop = $self->childattrval("value", $dr->child("sendOnPath"));

	my $content = $self->childval($dr->child("content"));

# real example
#<?xml version="1.0" encoding="UTF-8"?>
#<!DOCTYPE xiamSMS SYSTEM "xiamSMSMessage.dtd">
#<xiamSMS>
#<deliverRequest id="1907500">
#<receivedOnGroup value="SMPP-O2"/>
#<to npi="Unknown">53125</to>
#<udh value=""/>
#<carrierType value="SMPP"/>
#<sendOnPath value="103"/>
#<from>353861953134</from>
#<content type="text">TUCTEST</content>
#<xirMessageID value="1907500"/>
#</deliverRequest>
#</xiamSMS>

	my $h;
	$h->{SHORTCODE} = $to;
	$h->{MSISDN} = $from;
	$h->{NETWORK} = $rog;
	$h->{msg} = $content;

	# # # # # 
	my $res = $self->handleContent( $h );
	#my $res = $self->handleContent( { 
	#    SHORTCODE => $from,
	#    MSISDN => $to,
	#    NETWORK => $rog,
	#    msg => $content,
	#    } );

	my $resp;

	if ($res ne "") {
	    $resp = "<deliverResponse id=\"$id\">" .
		"<result status=\"ERROR\">$from</result>" . 
		"<content type=\"text\">$res</content>".
		"</deliverResponse>";
	} else {
	    $resp = "<deliverResponse id=\"$id\">" .
		"<result status=\"OK\">$from</result>" . 
		"</deliverResponse>";
	}

	if (0){
	    my $resp = "<deliverResponse id=\"$id\">" .
		# $from in result, not $to! (says Andrew xiam) looks like it's the $to in doc in example
		"<result status=\"OK\">$from</result>" . 
		###"<result status=\"OK\" value=\"OK\">$from</result>" .
		#"<from>$from</from>" .
		#"<to>$to</to>" .
		#"<xirMessageID value=\"$xmid\"/>" .
		#"<receivedOnGroup value=\"$rog\"/>".
		###"<to npi=\"Unknown\">53125</to>".
		#"<udh value=\"$udh\"/>".
		#"<carrierType value=\"$ct\"/>".
		#"<sendOnPath value=\"$sop\"/>".
		#"<content type=\"text\">TUCTEST</content>".
		"</deliverResponse>";
        }

	# TODO filter response probably have to do immediate response
	# but doesn't work as xiam specified
	if (0) {
	    my $immresptext = "james test immediate response";
	    $resp .= "\n<submitRequest id=\"immresp$id>\>".
		"<from>$to</from>".
		"<to>$from</to>".
		"<content type=\"text\">$immresptext</content>" .
		"<requestDeliveryReport value=\"yes\" />" .
		"</submitRequest>";
	}

	my $xiamresp = $self->_make_xiam_xml($resp);

	$self->logevent($MSG_DBG, "parse_xml send SMS response $xiamresp");

	return $xiamresp;

    } elsif (defined($ref->child("xiamSMS")->child("reportRequest"))) {

	my $rr = $ref->child("xiamSMS")->child("reportRequest");
	my ($id, $to, $from, $xmid, $status, $statustext) = ("", "", "", "", "", "");

	# not sure if report will have id
	$id = $rr->attribute("id");

	$from = $self->childval($rr->child("from"));
	$to = $self->childval($rr->child("to"));
	$xmid = $self->childattrval("value", $rr->child("xirMessageID"));
	print "XMID: $xmid\n";

	$status = $self->childval($rr->child("smscStatus"));
	my $specificstatus = $self->childval($rr->child("smscSpecificStatus"));
	$statustext = $self->childval($rr->child("smscStatusText"));

        # sub handleDeliverReport { 
	my $res;
	$res->{RequestID} = $id;
	$res->{messageid} = $id;
	$res->{ErrorCode} = $specificstatus;

	# TODO map results to smsengine API
	if ($res->{ErrorCode} == 0) {
	    $res->{ErrorMessage} = "Success";
	} else {
	    $res->{ErrorMessage} = "Error xiam xir unknown";
	}

	$res->{errorcode} = $res->{ErrorCode};
	$res->{errormessage} = $res->{ErrorMessage};

        #  status int NOT NULL,   # 0 = success, 1 = fail, 2 = not delivered or anything else = not yet

	if ($specificstatus == 0 ) {
	    $res->{status} = 0;
	} elsif ($specificstatus == 1 ||
		 $specificstatus == 2 ||
		 $specificstatus == 6 ) {
	    $res->{status} = 3;
	} elsif ($specificstatus == 5 ) {
	    $res->{status} = 1;
	} elsif ($specificstatus == 3 ||
		 $specificstatus == 4 ||
		 $specificstatus == 7 ) {
	    $res->{status} = 2;
	} else {
	    $res->{status} = 3;
	}

	$res->{startdate} = time();
	
	$res->{totalnumberofmessages} = 1;

	$res->{messagessent} = 0;
	$res->{messagesskipped} = 0;

	my $xiamifaceid = 0; # ngeh
	my $ldr = $self->logdeliverreportindb($res,$xiamifaceid);
	if ($ldr) {
	    warn "log of deliver report failed?: $ldr";
	}

	my $resp = "<reportResponse id=\"$id\">" .
	    "<result status=\"OK\">$to</result>" .
	    "</reportResponse>";

	my $xiamresp = $self->_make_xiam_xml($resp);

	$self->logevent($MSG_DBG, "parse_xml send SMS report response $xiamresp");

	return $xiamresp;

	# TODO
    } elsif (defined($ref->child("xiamSMS")->child("submitResponse"))) {
	# TODO
	use Data::Dumper;
	$self->psmserr("TODO: submitResponse ".
			"Request:".Dumper($request) .
			"Dump:".Dumper($ref));

	my $xiamresp = $self->_make_xiam_xml_err("submitResponse denied");
        return $xiamresp;

    } elsif (defined($ref->child("xiamSMS")->child("submitRequest"))) {
	# TODO only for testing/simulate
	use Data::Dumper;
	$self->psmserr("TODO: submitRequest ".
			"Request:".Dumper($request) .
			"Dump:".Dumper($ref));
	my $xiamresp = $self->_make_xiam_xml_err("submitRequest denied");
        return $xiamresp;

    } else {
	use Data::Dumper;
	$self->psmserr("parse_xml Error in XML unknown request".
			"Request:".Dumper($request) .
			"Dump:".Dumper($ref));
	my $xiamresp = $self->_make_xiam_xml_err("unknown request denied");
        return $xiamresp;
    }

    # impossible
    my $xiamresp = $self->_make_xiam_xml_err("refuse to accept reality");
    return $xiamresp;
}


sub handleContent { 
    my $self = shift;
    my $h = shift;

    $self->logevent
	($MSG_DBG,  "Yo! XMLRPCHandler.getContent $h hash" .
                        "SHORTCODE:".$h->{SHORTCODE}." ".
     	                "NETWORK:".$h->{NETWORK}." ".
	                "MSISDN:".$h->{MSISDN}." ".
	                "msg:".$h->{msg});

    # TODO: less crude shutdown 
    #   this at least though allows us to avoid using kill -9
    # perl o2serversim-xmlrpc-sendsms.pl "go away poe thing" "no message"
    if ($h->{MSISDN} =~ "go away poe thing") {
	$self->logevent($MSG_DBG,  "go away poe thing!!!");
	$self->poe_kernel_var()->stop();
        die "exiting. goodbye";
        #$main::die "die main";
	#$main::daemon->exit;

    } else {

        # TODO: rm hack for reference test env sub keyword alphyra with TUC
	#$h->{msg} =~ s/alphyra/TUC/;

	## TODO apply filters
	my ($filterresult,$filtername,$failmessage,$address,$port) = 
	    $self->applyfilters($h->{SHORTCODE}, $h->{msg});
        ### if defined($address) && defined($port) not done yet
	## $address,$port not implemented, not requested as feature
	if (defined($failmessage)) {
            ### daemon code if (defined($failmessage) then send failmessage back
           $self->logevent($MSG_DBG,  "incoming filter trap: ".$filtername);
           $self->setreplytext($failmessage);

        } else {
            ### otherwise pass it on as usual
	    $self->logevent($MSG_DBG,  "forward message ".$h->{msg});
	    # Run the poe client to forward text to tucd
	    $self->forwardmessage($h->{MSISDN},$h->{msg});
	    $self->poe_kernel_var()->run();

        }

    }

    my $res = $self->getreplytext();
    # This message is returned to 02 sms server
    #  if text is blank no message is sent to originating mobile
    #  if text is set that text is sent to originating mobile
    #  if we do not return this mobile gets an error message

    # clear replytext for next message
    $self->setreplytext("");

    return $res;
}


1;


=head1 NAME

XiamSms - providing a simple object for send/receive sms using 
      o2 proprietary XML/RPC interface, Doolin::SmsEngine object compatible

=head1 SYNOPSIS

  #!/usr/bin/perl -w

  use strict;

  use SmsEngine::SmsEngine;

  # new SmsEngine container (config)
  # load config will die if can't find config file
  # from config o2proprietary_iface causes instantiation of XiamSms object
  my $smsengine = SmsEngine->new( file => "smsengine.topupclub.xiam.conf",
                              verbose => 0, 
                              debug => 0, 
			      iface => "o2proprietary_iface", 
			      account => "topupclub_o2_account" );

  # access params
  my $verbose = $smsengine->getparam("verbose");

  my $res = $smsengine->{ObjectSms}->sendmessage( { text => "Hello Ciao",
				     to => 353869999999 } );

  $smsengine->{ObjectSms}->printsendresponse($res);

  my $qres = $smsengine->{ObjectSms}->querysenddeliverystatus( 
      { RequestID => $res->{RequestID} } );
  $smsengine->{ObjectSms}->printqueryresponse($qres);

=head1 DESCRIPTION

 Provide a simple perl module for easy interface to send/receive sms.

=head1 DESIGN

=head1 xiam xml/sms config example

=head1 charset

 perl

=head1 COPYRIGHT

 Copyright (C) 2004 Doolin Technologies

=cut
