#!/usr/bin/perl -w

use strict;
use bytes;
use lib qw(t/lib);
use Test::More tests => 
    1 +          # use_ok
    4 + 2 + 20 + # error case + setup + enqueue and read back
    8 +          #   try read while locked and unlocked
    9 +          #   remove 2 things ok 1 thing not ok
    4 +          # fully empty check
    17 +         # readnext basic behaviour 1 thing in q
    13           # readnext basic behaviour 2 things in q
    ;

BEGIN { use_ok('FileQueue') }

# test trying to create q with missing/bad params
{
    # no dir
    my $badq = FileQueue->new();
    ::is( $badq->{error}, 1, 'create q with no dir should fail' );
    ::is( $badq->{errors}->[0], 
	  "FileQueue::new error: dir arg is mandatory",
	  'create q with bad dir fails' );

    # bad dir
    $badq = FileQueue->new( dir => "/bad-nonexistant-rootdir".
			    "perlFileQueueTest/".
			    "cannot-create-multiple-subdirs-we-dont-pass-P" );
    ::is( $badq->{error}, 1, 'create q with bad dir should fail' );
    ::like( $badq->{errors}->[0], 
	  qr!.*error.*check permissions.*access or create queue directory.*/bad-nonexistant-rootdir.*!,
	  #"perlFileQueueTest/cannot-create-multiple-subdirs-we-dont-pass-P",
	  'create q with bad dir fails' );
}

# nuke old test data
if ( -d "/tmp/perlFileQueueTest") {
    my $rmtestdircom = "rm -rf /tmp/perlFileQueueTest";
    system ($rmtestdircom) == 0 || die "cannot $rmtestdircom $!";
}

# create Q object for general use, validate it is okay
my $q = FileQueue->new( dir => "/tmp/perlFileQueueTest",
			#debug => 1 
			);
print $q->{errors}->[0] if ($q->{error});
::ok( defined($q) && !defined($q->{error}), 'create q success' );
::fail "can't do without a Q for general use" if (!defined($q));

# NOTE locks only applicable with different Q instances (I think)
# other queue instance to test locks
my $oq = FileQueue->new( dir => "/tmp/perlFileQueueTest",
			 #debug => 1 
			 );
print $oq->{errors}->[0] if ($oq->{error});
::ok( defined($oq) && !defined($oq->{error}), 'create q success' );
::fail "can't do new instance of queue" if (!defined($oq));

# handy routine .... put it into FileQueue.pm ? TODO
sub findinQ {
    my $q = shift;
    my $key = shift;
    my $value = shift;

    # We SHOULD so reset in
    # doesn't do reset so we can rtest where we do not do reset 
    $q->reset();
    # should always reset to start of q otherwise might skip
    #  reset also refresh from disk? 
    # test rm something, can findinQ before reset ?
 
    my $i = 0;
    my $readout = $q->readnext();
    $i++;
    #print STDERR "item $i q id is " . $readout->{id} . "\n";
    #print STDERR $readout->{ref}->{$key} ."=?=". $value . "\n";
    while( defined($readout) && $readout->{ref}->{$key} ne $value) {
        $q->unlock(); # unlock last thing (didn't match)
        $readout = $q->readnext();
	$i++;
	#print STDERR "item $i q id is " . $readout->{id} . "\n";
        #print STDERR $readout->{ref}->{$key} ."=?=". $value . "\n";
    }

    #$q->unlock(); # unlock last thing
    # DON't unlock last thing yet if want to do something with it

    return $readout;
}


# test enqueue
$q or die $!;
{

    my $ri = int(rand(100));
    my %obj0 = ( 'to' => "353867777777".$ri, 
		'text' => "hello foo".$ri,
		'time' => 0,
		'id' => 0 );

    my $id = 0;

    $obj0{time} = time()."-0";
    $obj0{id} = $id++;

    use Data::Dumper;
    print Dumper(%obj0);

    # call enqueue
    my $res = $q->enqueue(\%obj0);

    print STDERR $q->{errors}->[0] if ($q->{error});
    ::is( $res->{error}, 0, 'enqueue success' );
    $q->unlock(); # unlock last thing

    # by eye tested lock is removed okay :)
    # TODO: automatically test this?


    # at least two more things into queue   
    my %obj1; 
    $obj1{to} = "cork";
    $obj1{text} = "second insert by filequeue.t";
    $obj1{time} = time()."-1";
    $obj1{id} = $id++;
    $res = $q->enqueue(\%obj1);
    ::is( $res->{error}, 0, 'enqueue success' );
    $q->unlock(); # unlock last thing

    my %obj2; 
    $obj2{to} = "cobh";
    $obj2{text} = "third insert by filequeue.t";
    $obj2{time} = time()."-2";
    $obj2{id} = $id++;
    $res = $q->enqueue(\%obj2);
    ::is( $res->{error}, 0, 'enqueue success' );
    $q->unlock(); # unlock last thing

    # id not returned as enqueue we throw away lock as this is how we wish to use queues
    #print STDERR "item q id is " . $res->{id} . "\n";


    # careful, goes through Q and resets to start kindof
    #$q->reset();
    #my $count = $q->show_queue();
    #is( $count > 0, 1, "queue count is $count" );
    #print STDERR "queue count is $count\n";
    # TODO: using $qid bad, is unlocked, don't return from enqueue
    #my $readoutusingqid = $q->readnext( $res->{id} );
    #is( defined($readoutusingqid), 1, 'we can read thing out using queue id' );

    ### search queue for a specific thing we know we just put there

    # findinQ does reset itself
    # first without q reset we fail to find something
    my $readout0 = findinQ( $q, "time", $obj0{time} );
    #ok( !defined($readout0), 'without q reset we fail to find something');
    ::ok( defined($readout0), 
	  'without q reset we do not fail to find something as findinQ does a reset itself');

    # TODO: resetting or refreshing(whong) queue 
    # ... may not be needed, may be needed, may be good, may be bad
    # half-done YES is needed (for testing anyway), YES is good and works
    $q->reset();
    #$q->whong();
    ::is( 1, 1, 'we can reset q' );

    $readout0 = findinQ( $q, "time", $obj0{time} );

    ::ok( defined($readout0), 'we can read specific thing out' );
    #print STDERR Dumper($readout);
    ::is( $readout0->{ref}->{to}, $obj0{to}, 'we can read thing out field to' );
    ::is( $readout0->{ref}->{text}, $obj0{text}, 'we can read thing out field text' );
    ::is( $readout0->{ref}->{time}, $obj0{time}, 'we can read thing out field time' );
    ::is( $readout0->{ref}->{id}, $obj0{id}, 'we can read thing out field id' );


    $q->unlock(); # unlock last thing (thing found)


    $q->reset();
    my $readout1 = findinQ( $q, "time", $obj1{time} );
    $q->unlock(); # unlock last thing (thing found)

    ::ok( defined($readout1), 'we can read second specific thing out' );
    #print STDERR Dumper($readout);
    ::is( $readout1->{ref}->{to}, $obj1{to}, 'we can read thing out' );
    ::is( $readout1->{ref}->{text}, $obj1{text}, 'we can read thing out' );
    ::is( $readout1->{ref}->{time}, $obj1{time}, 'we can read thing out' );
    ::is( $readout1->{ref}->{id}, $obj1{id}, 'we can read thing out' );

    $q->reset();
    my $readout2 = findinQ( $q, "time", $obj2{time} );
    $q->unlock(); # unlock last thing (thing found)

    ::ok( defined($readout2), 'we can read third specific thing out' );
    #print STDERR Dumper($readout);
    ::is( $readout2->{ref}->{to}, $obj2{to}, 'we can read thing out' );
    ::is( $readout2->{ref}->{text}, $obj2{text}, 'we can read thing out' );
    ::is( $readout2->{ref}->{time}, $obj2{time}, 'we can read thing out' );
    ::is( $readout2->{ref}->{id}, $obj2{id}, 'we can read thing out' );

    {
    ###### now ... find obj1 and leave it locked

	$q->reset();
	my $readoutkeeplockobj1 = findinQ( $q, "time", $obj1{time} );
	::ok( defined($readoutkeeplockobj1), 'we read obj1 out again' );
	::is( $readoutkeeplockobj1->{ref}->{time}, $obj1{time}, 'we can read thing out' );

        ### now if we use same q to do other things we lose control over lock

	### try find it again other queue
        ### ... it is locked ... so we should not be able to find it

	my $readoutwhenlock = findinQ( $oq, "time", $obj1{time} );
	::ok( !defined($readoutwhenlock), 'we can not read obj1 out other q when obj1 is locked' );

	# verify we can read something else out of other queue
	my $readoutother = findinQ( $oq, "time", $obj0{time} );
	::ok( defined($readoutother), 'we can read obj0 out when obj1 is locked' );
	::is($readoutother->{id}, $readout0->{id}, "queueid same item is same different reads");
	$oq->unlock($readoutother->{id}); # unlock obj0 lock

	### unlock it

	$q->unlock($readoutkeeplockobj1->{id}); # unlock obj1 lock

	### try find it again ... it is unlocked ... so we should be able to find it
	### in other q

	$readoutother = findinQ( $oq, "time", $obj1{time} );
	::ok( defined($readoutother), 'we can read obj1 out when obj1 is unlocked' );
	::is( $readoutother->{ref}->{time}, $obj1{time}, 'we can read thing out' );
	::is( $readoutother->{id}, $readout1->{id}, "queueid same item is same different reads");
	#$oq->unlock($readoutother->{id}); # unlock obj1 lock  LEAVE IT
	#print STDERR Dumper($readoutother);

        #### TODO the id doesn't match in case where visit skips over locked things??

        #### now obj1 is still locked to $oq
    }

    #::is
    #print STDERR "\nreadoutotherid " . $readoutotherid;
    print STDERR "\nreadout0id     " . $readout0->{id};
    print STDERR "\nreadout1id     " . $readout1->{id};
    print STDERR "\nreadout2id     " . $readout2->{id};
    print STDERR "\n\n";
    {
	###### can we rm this from Q
	
	$oq->reset();
	my $readout = findinQ( $oq, "time", $obj1{time} );
	::ok( defined($readout), 
	      'check we CAN read obj1 out (different Q) before remove it' );
	$oq->reset();

	my $dqr = $q->dequeue($readout1->{id});
	::is( $dqr, '', "dequeue result is ''");
	$q->reset();
	my $readoutfail = findinQ( $q, "time", $obj1{time} );
	::ok( !defined($readoutfail), 'we can not read obj1 out (OF SAME Q) after we remove it' );
	$readoutfail = findinQ( $oq, "time", $obj1{time} );
	::ok( !defined($readoutfail), 'we can not read obj1 out (different Q) after we remove it' );
	# rm already rm'd thing
	$dqr = $q->dequeue($readout1->{id});
	::is( $dqr, undef, "dequeue FAIL result is undef");

	# can we remove earlier read item ( not locked ) TODO: is this dangerous?
	$dqr = $q->dequeue($readout0->{id});
	::is( $dqr, undef, "dequeue result is 0");
	$q->reset();
	#$q->whong();
	$readoutfail = findinQ( $q, "time", $obj0{time} );
	::ok( !defined($readoutfail), 'we can not read obj0 out after we remove it and reset' );
    
	# can we remove earlier read item ( not locked ) TODO: is this dangerous?
	$dqr = $q->dequeue($readout2->{id});
	::is( $dqr, undef, "dequeue result is 0");
	$q->reset();
	#$q->whong();
	$readoutfail = findinQ( $q, "time", $obj2{time} );
	::ok( !defined($readoutfail), 'we can not read obj2 out after we remove it and reset' );
    }
    
    #::is after reset q still fully empty
    {
	$q->reset();
	my $readout = $q->readnext();
 	#print STDERR "moo" . Dumper($readout);
	::ok( !defined($readout), 'nothing in q' );
	$readout = $q->readnext();
	::ok( !defined($readout), 'nothing else in q' );
	$readout = $q->readnext();
	::ok( !defined($readout), 'STILL nothing else in q (check for wrap)' );
	$readout = $q->readnext();
	::ok( !defined($readout), 'STILL nothing else in q (check for wrap)' );
    }


    #### readnext basic behaviour
    # add one thing to Q
    $res = $q->enqueue(\%obj1);
    ::is( $res->{error}, 0, 'enqueue success' );
    $q->unlock(); # unlock last thing
    {
	my $readout = $q->readnext();
	::ok( !defined($readout), 'nothing else in q' );
        # TODO: test is it always true that after inserting something next read is at end of Q ?
	$readout = $q->readnext();
	#print STDERR "moo-not-rm" . Dumper($readout);
	::ok( defined($readout), 'we can read obj1 out' );
	::is( $readout->{ref}->{time}, $obj1{time}, 'it is obj1 definately' );
	$readout = $q->readnext();
	::ok( !defined($readout), 'nothing else in q, bottom of q' );
	$readout = $q->readnext();
	::ok( !defined($readout), 'nothing else in q, (WRAP, one locked thing in q)' );
	$readout = $q->readnext();
	::ok( !defined($readout), 'nothing else in q, bottom of q' );



	#print STDERR "does the whong cause close() on unopened filehandle?\n";
	#$q->whong();
	#print STDERR "does the whong cause close() on unopened filehandle?\n";
	$q->reset();
	$readout = $q->readnext();
	#print STDERR "moo-not-rm" . Dumper($readout);
	::ok( defined($readout), 'a reset puts at top of q (readnext call)' );
	::is( $readout->{ref}->{time}, $obj1{time}, 'it is obj1 definately' );
	$readout = $q->readnext();
	::ok( !defined($readout), 'nothing else in q' );
	$readout = $q->readnext();
	::ok( !defined($readout), 'nothing else in q, bottom of q NO WRAP!' );
	$readout = $q->readnext();
	::ok( !defined($readout), 'nothing else in q, bottom of q NO WRAP!' );

        # is there just nothing cos we've locked it ?
        $q->unlock;
	$readout = $q->readnext();
	#print STDERR "moo-not-rm" . Dumper($readout);
	::ok( defined($readout), 'a reset puts at top of q (readnext call)' );
	::is( $readout->{ref}->{time}, $obj1{time}, 'it is obj1 definately' );
        $q->unlock;
	$readout = $q->readnext();
	::ok( !defined($readout), 'nothing else in q' );
	$readout = $q->readnext();
	::ok( defined($readout), 'yes WRAP an unlock allows readnext to wrap' );
	::is( $readout->{ref}->{time}, $obj1{time}, 'it is obj1 definately' );
        $q->unlock;
    }

    #### readnext basic behaviour
    # add one other thing to Q
    $res = $q->enqueue(\%obj0);
    ::is( $res->{error}, 0, 'enqueue success' );
    $q->unlock(); # unlock last thing
    {
	my $readout = $q->readnext();
	::ok( !defined($readout), 'nothing else in q' );
        # TODO: test is it always true that after inserting something next read is
        #   just after insert and insert is always at bottom Q ?
        # er ... is not true, order changes and test fails sometimes
	$readout = $q->readnext();
	::ok( defined($readout), 'we can read obj0 out' );
	::is( $readout->{ref}->{time}, $obj1{time}, 'it is obj1 definately' );
	$q->unlock;
	$readout = $q->readnext();
	::ok( defined($readout), 'we can read obj1 out' );
	::is( $readout->{ref}->{time}, $obj0{time}, 'it is obj1 definately' );
	$q->unlock;
	$readout = $q->readnext();
	::ok( !defined($readout), 'nothing else in q' );
	$readout = $q->readnext();
	::ok( defined($readout), 'wrap' );

	$q->reset();
	$readout = $q->readnext();
	::ok( defined($readout), 'a reset puts at top of q (readnext call)' );
	::is( $readout->{ref}->{time}, $obj1{time}, 'it is obj1 definately' );
	$q->unlock;
	$readout = $q->readnext();
	::ok( defined($readout), 'we can read obj1 out' );
	::is( $readout->{ref}->{time}, $obj0{time}, 'it is obj0 definately' );
	$q->unlock;
	$readout = $q->readnext();
	::ok( !defined($readout), 'nothing else in q' );
    }


    ### remove the thing
    # TODO: rm it!
    # so we can only dequeue things when they arfe unlocked ?
    # weird
    # TESTS TODO (but these are weird error case tests so do them later james.
    #  1. dequeue in q when locked. what happens?
    #  2. dequeue in q when not locked. what happens?
    #  3. dequeue in q when locked in other q. what happens?
    #  4. dequeue in q when not locked in other q. what happens?

    # james. calm down like.  these are funny error tests
    #  WE DO NOT NEED THEM! okay.
    #  oh. alright.

    
}

# Final wrap up
