#!/usr/bin/perl -Tw

# the existance of t directory doesn't override test.pl (unless tests in there fail)
#  also tests under t are assumed to use t/lib ? and Test::More
# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl test.pl'

use strict;
use bytes;

use lib qw(t/lib);
use Test::More tests => 
    1 +          # use_ok
    3 +          # load config
    1 +          # kannel sendsms
    8 +          # o2 proprietary
    9 +          # o2 proprietary filter config
    12 +         # o2 proprietary filter application
    5 +          # o2 prop sendsms
    3 + 6 + 2 + 3 + 8 + 8 + 16 + 9 + 8 + 8 +         
                 # o2 prop config parse tests
#   3 +          # o2 prop network down
    4 +          # o2 prop querysenddeliverystatus
    1 +          # o2 prop querybatchrequest bad
#   2 +          # o2 prop queryaccount
    14 +         # o2 prop send batch, control batch
    4;           # o2 prop pcs picture 

BEGIN { use_ok('SmsEngine::SmsEngine') }

my $grepFileError = "";

sub grepFile
{
    my $file = shift; my $r = 1; my $str; my $res;
    while ($r && ($str = shift)) {
        open(F,"$file");
	$res=0;
	while (!$res && readdir(DIR)) {
	    $res = m/$str/;
	}
	close(F);
        #$res = `grep -c "$str" $file`;
        if ($res eq "") { 
            $r = 0; 
            $grepFileError = "could not find $str in $file";
        }
    }
    return $r;
}

my $ti = 1;
my $tn = "";
sub registerTest
{
    $ti++;
    $tn = shift;
}

my $goodresult;

{
registerTest("Does SmsEngine load config?");

my $sendsms = SmsEngine->new( file => "example.conf" );
# if we're not dead config was found
# not true any more, we actually return errors
::ok( defined($sendsms), "SmsEngine load config" );
::is( $sendsms->{error}, undef, "SmsEngine load config no error" );

registerTest("Does SmsEngine load fakesms config?");
$sendsms->loadiface("fake_iface");
$sendsms->loadaccount("fake_account");
::is( 1,1, "Does SmsEngine load fakesms config?");

registerTest("Does SmsEngine send sms using fakesms config?");
my $to = "353861111111";
my $from = "SmsEngine.pm"; # text from not supported by all smscs/handsets
my $text = "SmsEngine.pm example text message";
my $result = $sendsms->sendsms($to,$from,$text);
undef $sendsms;

my $r1 = grepFile("/tmp/fakeSmsEngineIface.log",
		  "SmsEngine Fake sendsms to $to from SmsEngine.pm text SmsEngine.pm example text message");

::is($r1,1,$tn);
}

{
registerTest("Can SmsEngine speak kannel?");

#TODO: start fake kannel testbed
#TODO: account deprecated
my $sendsms = SmsEngine->new( file => "example.conf", 
			      ifacegroup => "smskannel_iface", 
			      account => "clickatell_kannel_account" );

my $to = "35386badnum";
my $from = "SmsEngine.pm"; # text from not supported by all smscs/handsets
my $text = "SmsEngine.pm example text message";

# hey! I'm being charged for this! And kannel url hang thing ?
#my $result = $sendsms->sendsms($to,$from,$text);
#print "sendsms result: $result\n";
#TODO: validate kannel testbed received data
::is(1,1,$tn);
}

#start o2proprietary testbed
my $sendsms = SmsEngine->new( file => "example.conf", 
			      debug => 0,
			      verbose => 0 );
			      #iface => "o2proprietary_iface", 
			      #account => "topupclub_o2_account" );

registerTest("Can SmsEngine speak o2proprietary? test global config alarmemail");
::like($sendsms->getparam("alarmemail"), qr/jamesc/,$tn);
#$sendsms->showconfig();

registerTest("Can SmsEngine speak o2proprietary? test o2 proprietary config");
$sendsms->{ObjectSms}->showconfig();
#print "getparam(https_xmlrpc_url) = " . 
#    $sendsms->getparam("https_xmlrpc_url") . "\n";

::like( $sendsms->getparam("https_xmlrpc_url"), qr!https://!,  $tn);
::like( $sendsms->getparam("https_cert_file"), qr!^/.*\.pem!,  $tn );
::like( $sendsms->getparam("https_pkcs12_file"), qr!^/.*\.p12!,  $tn );
::ok( $sendsms->getparam("https_pkcs12_password") ne "", $tn );
::like( $sendsms->getparam("proxy"), qr!http://proxy:8080!,  $tn );
::ok( $sendsms->getparam("smsinaddress") eq "localhost", $tn );
::ok( $sendsms->getparam("smsinport") > 0 , $tn );

registerTest("Can SmsEngine speak o2proprietary? test filter config");
my $smsinfiltersettings = $sendsms->getparam("smsinfiltersettings");
::like( $sendsms->getparam("smsinfilter"), qr!free_filter!,  $tn);
::like( $sendsms->getparam("smsinfilter"), qr!non_free_filter!,  $tn );
::like( $sendsms->getparam("smsinfilter"), qr!default_filter!,  $tn );
::ok( defined($smsinfiltersettings), $tn );
::is( $smsinfiltersettings->{free_filter}->{object}, "filter", $tn );
::is( $smsinfiltersettings->{free_filter}->{shortcode}, "353863833342", $tn );
::like( $smsinfiltersettings->{free_filter}->{smsinkeywords}, qr/REGISTER/, $tn );
::ok( $smsinfiltersettings->{free_filter}->{priority}, $tn );
::ok( $smsinfiltersettings->{free_filter}->{swallow}, $tn );

if (0) { foreach my $filterset(keys(%$smsinfiltersettings)) {
    #print STDERR "Dump filterset " .
    #    Data::Dumper->Dumper($filterset) . "\n";
    print "O2ProprietarySms arg filter $filterset .oO0Oo.".
	$smsinfiltersettings->{$filterset}."\n";
    foreach my $filterthing(keys(%{$smsinfiltersettings->{$filterset}})) {
	print "  $filterthing = ".$smsinfiltersettings->{$filterset}->{$filterthing}."\n";
	
    }
}
}

{
registerTest("o2proprietary filter application");

{
# TODO: need filter priority/order  
#   and  if filter catches something stop there or continue
my ($shortcode, $message) = ("no shortcode match so message allowed by default filter", "no match" );
my ($result,$filtername,$failmessage,$address,$port) = 
    $sendsms->{ObjectSms}->applyfilters($shortcode,$message);
::is( $result, 1, $tn);
::is( $filtername, "default_filter", $tn );
::ok( !defined($failmessage), $tn );
}

{
my ($shortcode, $message) = ("353863833342", "shortcode match, no msg match so rejected" );
my ($result,$filtername,$failmessage,$address,$port) = 
    $sendsms->{ObjectSms}->applyfilters($shortcode,$message);
::is( $result, 1, $tn);
::is( $filtername, "free_filter", $tn );
::ok( defined($failmessage), $tn );
}

{
my ($shortcode, $message) = ("353863832032", "non free shortcode match" );
my ($result,$filtername,$failmessage,$address,$port) = 
    $sendsms->{ObjectSms}->applyfilters($shortcode,$message);
::is( $result, 1, $tn);
::is( $filtername, "non_free_filter", $tn );
::ok( !defined($failmessage), $tn );
}

{
my ($shortcode, $message) = ("353863833342", "TUC REGISTER shortcode and REGISTER match" );
my ($result,$filtername,$failmessage,$address,$port) = 
    $sendsms->{ObjectSms}->applyfilters($shortcode,$message);
::is( $result, 1, $tn);
::is( $filtername, "free_filter", $tn );
::ok( !defined($failmessage), $tn );
}
}

### daemon code if (defined($failmessage) then send failmessage back
### if defined($address) && defined($port) not done yet
### otherwise pass it on as usual

{
registerTest("Can SmsEngine speak o2proprietary? call sendsms");

my $to = "35386badnum";
my $from = "SmsEngine.pm"; # text from not supported by all smscs/handsets
my $text = "SmsEngine.pm example text message";
my $result = $sendsms->sendsms($to,$from,$text);
print "sendsms result: $result " .
    $result->{ErrorCode} . "," .
    $result->{RequestID} . "," .
    $result->{ErrorMessage} . "," .
    $result->{Failed_MSISDN}->[0]->{errorcode} . "," .
    $result->{Failed_MSISDN}->[0]->{num} . "," .
    "\n";

#validate o2proprietary testbed received data
::is( $result->{ErrorCode}, 0, $tn);
::ok( $result->{RequestID} > 0, $tn );
::is( $result->{ErrorMessage}, "Success", $tn );
::is( $result->{Failed_MSISDN}->[0]->{errorcode}, 202, $tn );
::is( $result->{Failed_MSISDN}->[0]->{num}, "35386badnum", $tn );

$goodresult = $result;
}

#if (1) { 
#    warn "remove me before checkin";
#    warn "temporary exit to speed up testing while implementing filtering";
#    exit(0); 
#} 


{
# DONE: do not die (SmsEngine) on error, return an error and undef instead,
#       lower layers (SmsEngine itself should print to STDERR the errors?)
# DONE: all scripts which instantiate SmsEngine now must check
#       if error defined (and maybe die)
# DONE: bad config files should not produce cryptic errors
# TODO: test space after port numbers in config file  (code was added to fix that)
# DONE: test line not matching group start "[groupname]" or group var "varname = value"

### whole slew of config parse tests

registerTest("SmsEngine report no config error, do not die");
my $noconfsms = SmsEngine->new( file => "notexistconf.conf" );
::is( $noconfsms->{error}, 1, $tn);
::like( $noconfsms->{errors}->[0], qr/cannot find config file/, $tn );
::like( $noconfsms->{errors}->[0], qr/notexistconf.conf/, $tn );


my $badconfsms;
registerTest("SmsEngine report config syntax error");
$badconfsms = SmsEngine->new( file => "badconf.conf", 
			      debug => 0,
			      verbose => 0 );

if (0 && $badconfsms->{error}) {
    #foreach my $err(@Config::IniFiles::errors) {
    #	print "IniFile error $err\n";
    #}
    use Data::Dumper;
    print Data::Dumper->Dumper($badconfsms);
    foreach my $err(@{$badconfsms->{errors}}) {
	print "$err\n";
    }
}

::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!cannot load config file!, $tn );
::like( $badconfsms->{errors}->[0], qr!etc/badconf.conf!, $tn );
::like( $badconfsms->{errors}->[1], qr!Line 130 in file !, $tn );
::like( $badconfsms->{errors}->[1], qr!etc/badconf.conf is mal-formed:!, $tn );
::like( $badconfsms->{errors}->[1], qr!bad line!, $tn );

registerTest("SmsEngine report config error no ifacegroup");
$badconfsms = SmsEngine->new( file => "badconf-noifacegroup.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::is( $badconfsms->{errors}->[0] 
          =~ "config group \\[commonsms\\] var ifacegroup is mandatory", 1, $tn );

registerTest("SmsEngine report config error not an iface");
$badconfsms = SmsEngine->new( file => "badconf-notaniface.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadiface config error with ifacegroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!object not_iface not an interface?!, $tn );

registerTest("SmsEngine report config error missing iface param");
$badconfsms = SmsEngine->new( file => "badconf-ifacemissingaccount.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadiface config error with ifacegroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!object myo2iface!, $tn );
::like( $badconfsms->{errors}->[1], qr!argument accountgroup is mandatory!, $tn );
$badconfsms = SmsEngine->new( file => "badconf-ifacemissinginout.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadiface config error with ifacegroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!object myo2iface!, $tn );
::like( $badconfsms->{errors}->[1], qr!argument inoutgroup is mandatory!, $tn );

registerTest("SmsEngine report config error missing iface or account group");
$badconfsms = SmsEngine->new( file => "badconf-missingaccount.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadaccount config error with accountgroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!not an account?!, $tn );
::like( $badconfsms->{errors}->[1], qr!notheretopupclub_o2_account_live!, $tn );
$badconfsms = SmsEngine->new( file => "badconf-missinginout.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadiface config error with ifacegroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!inout object notheresmsengine_tucd!, $tn );
::like( $badconfsms->{errors}->[1], qr!argument outaddress is mandatory!, $tn );
#    print Data::Dumper->Dumper($badconfsms);


registerTest("SmsEngine report config error missing iface params");
$badconfsms = SmsEngine->new( file => "badconf-ifacemissing-outaddress.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadiface config error with ifacegroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!inout object smsengine_tucd!, $tn );
::like( $badconfsms->{errors}->[1], qr!argument outaddress is mandatory!, $tn );
$badconfsms = SmsEngine->new( file => "badconf-ifacemissing-outport.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadiface config error with ifacegroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!inout object smsengine_tucd!, $tn );
::like( $badconfsms->{errors}->[1], qr!argument outport is mandatory!, $tn );
$badconfsms = SmsEngine->new( file => "badconf-ifacemissing-inaddress.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadiface config error with ifacegroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!inout object smsengine_tucd!, $tn );
::like( $badconfsms->{errors}->[1], qr!argument inaddress is mandatory!, $tn );
$badconfsms = SmsEngine->new( file => "badconf-ifacemissing-inport.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadiface config error with ifacegroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!inout object smsengine_tucd!, $tn );
::like( $badconfsms->{errors}->[1], qr!argument inport is mandatory!, $tn );

$badconfsms = SmsEngine->new( file => "badconf-ifacemissing-multiple.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadiface config error with ifacegroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!object myo2iface!, $tn );
::like( $badconfsms->{errors}->[1], qr!argument accountgroup is mandatory!, $tn );
#::like( $badconfsms->{errors}->[2], qr!object myo2iface!, $tn );
#::like( $badconfsms->{errors}->[2], qr!argument inoutgroup is mandatory!, $tn );
::like( $badconfsms->{errors}->[2], qr!inout object smsengine_tucd!, $tn );
::like( $badconfsms->{errors}->[2], qr!argument outaddress is mandatory!, $tn );
::like( $badconfsms->{errors}->[3], qr!argument outport is mandatory!, $tn );
::like( $badconfsms->{errors}->[4], qr!argument inaddress is mandatory!, $tn );
::like( $badconfsms->{errors}->[5], qr!argument inport is mandatory!, $tn );

registerTest("SmsEngine report config error missing o2 account params");
$badconfsms = SmsEngine->new( file => "badconf-o2-missing-smsinport.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadaccount config error with accountgroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!O2ProprietarySms loadaccount!, $tn );
::like( $badconfsms->{errors}->[1], qr!argument smsinport is mandatory!, $tn );
$badconfsms = SmsEngine->new( file => "badconf-o2-missing-smsinaddress.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadaccount config error with accountgroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!O2ProprietarySms loadaccount!, $tn );
::like( $badconfsms->{errors}->[1], qr!argument smsinaddress is mandatory!, $tn );

registerTest("SmsEngine report config error missing o2 account file");
$badconfsms = SmsEngine->new( file => "badconf-o2-missing-certfile.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadaccount config error with accountgroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!O2ProprietarySms loadaccount!, $tn );
::like( $badconfsms->{errors}->[1], qr!Can\'t read file!, $tn );
$badconfsms = SmsEngine->new( file => "badconf-o2-missing-keyfile.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadaccount config error with accountgroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!O2ProprietarySms loadaccount!, $tn );
::like( $badconfsms->{errors}->[1], qr!Can\'t read file!, $tn );


#    print Data::Dumper->Dumper($badconfsms);
# TODO: (low priority) full test fake and kannel account loading

# bad libs they die die die :(
#exit(19);

# DONE in daemons, not in libs: emergency message if can't forward to tucd 
#  specifically mentioning port and config file
#   <enrico> james, seems to be working now, in port was set to 8151, changed
#        it back to 28222 and it worked. inport of group [smsengine_tucd]
}

if (0) { # test annoyingly long timeout
registerTest("o2proprietary network down");
#(in config server exists but nothing on port)
#big nasty 5min? 500sec? =~ 8min? timeout  
my $brokensms = SmsEngine->new( file => "brokenexample.conf", 
			      debug => 0,
			      verbose => 0 );
			      #iface => "o2proprietary_iface", 
			      #account => "topupclub_o2_account" );

my $to = "35386badnum";
my $from = "SmsEngine.pm"; # text from not supported by all smscs/handsets
my $text = "SmsEngine.pm example text message";
my $brokenresult = $brokensms->sendsms($to,$from,$text);
print "sendsms result: $brokenresult " .
    $brokenresult->{RequestID} . "," .
    $brokenresult->{ErrorCode} . "," .
    $brokenresult->{ErrorMessage} . "," .
    "\n";

::is( $brokenresult->{ErrorCode}, 8152, $tn);
::like( $brokenresult->{ErrorMessage}, qr!SmsEngine connect Error!,, $tn );
::is( $brokenresult->{ErrorMessage}, "Success", $tn );
}

die "subsequent tests need sendsms test to pass and retrieve good RequestID" 
    if (!defined($goodresult->{RequestID}) || $goodresult->{RequestID} <= 0);

registerTest("SmsEngine o2proprietary querysenddeliverystatus");
my $qsds = $sendsms->{ObjectSms}->querysenddeliverystatus( 
              { RequestID => $goodresult->{RequestID} } );

# might be in any state, might not be finished
print "querysenddeliverystatus result " .
    $qsds->{TOTALNUMBEROFMESSAGES} ."<=".
    $qsds->{MESSAGESSENT} ."+". $qsds->{MESSAGESSKIPPED} .
    " status=" . $qsds->{STATUS} .
    " errorcode=" . $qsds->{ErrorCode} . "\n";
::is( $qsds->{TOTALNUMBEROFMESSAGES}, 1, $tn);
::ok( $qsds->{MESSAGESSENT} + $qsds->{MESSAGESSKIPPED} <= 1, $tn );
::ok( $qsds->{STATUS} <= 5, $tn );
::is( $qsds->{ErrorCode}, 0, $tn );


# possible that there would be batch id in system with that number ... but not that likely
# we expect ERR API053: Request is not a valid BATCH request
registerTest("SmsEngine o2proprietary querybatchrequest invalid id");
my $qbr = $sendsms->{ObjectSms}->querybatchrequest( { RequestID => $goodresult->{RequestID} } );
print "querybatchrequest result " .
    #" status=" . $qbr->{STATUS} .
    " errorcode=" . $qbr->{ErrorCode} . "\n";
::is($qbr->{ErrorCode}, 53, $tn);


if (0) {
registerTest("SmsEngine o2proprietary queryaccountrequest");
# API doc has an error
my $qar = $sendsms->{ObjectSms}->queryaccountrequest();
# also currently problem perl libs sending request empty structure not compatible way o2 do it o2 require empty structure of exactly form <struct></struct> perl/XML::Parser SOAP::Lite somewhere does <struct/>
# no wait!  https://sms-open.o2.co.uk:444/apilistener/SMSOA_xmlrpc_listener doesn't understant <struct/>
# but https://216.239.194.94:444/vvtlistener1/SMSOA_xmlrpc_listener does
print "queryaccountrequest result: " .
    $qar->{ErrorCode} . "," .
    $qar->{RequestID} . "," .
    $qar->{REQUEST_ID} . "," .
    $qar->{STATUS} . "," .
    $qar->{PERCENTAGECOMPLETE} . "," .
    $qar->{STARTDATE} . "," .
    $qar->{MESSAGESSKIPPED} . "," .
    $qar->{AccountRequestData}->[0]->{ErrorMessage} .
    "\n";

::is( $qar->{ErrorCode}, 0, $tn);
::is( $qar->{AccountRequestData}->[0]->{ErrorMessage} == "Success", 1, $tn );
}


# possible that there would be batch id in system with that number ... but not that likely
# we expect ERR API053: Request is not a valid BATCH request
my $in10mins = time() + 600;
my $in5mins = time() + 300;

registerTest("SmsEngine o2proprietary sendbatchmessage");
my $badbatchnum = "35386badbatchnum";
my $btoparams = { text => "perl module test SmsEngine o2proprietary sendbatchmessage", 
		  to => [ $badbatchnum ],
		  StartTime => $in10mins };
my $bresult = $sendsms->{ObjectSms}->sendbatchmessage( $btoparams );
print "sendbatchmessage result " .
    " id=" . $bresult->{RequestID} .
    " errorcode=" . $bresult->{ErrorCode} . "\n";
::is( $bresult->{ErrorCode}, 0, $tn);
::is( $bresult->{RequestID} > 0, 1, $tn );

{
registerTest("SmsEngine o2proprietary querybatchrequest valid id");
my $qbr = $sendsms->{ObjectSms}->querybatchrequest( { RequestID => $bresult->{RequestID} } );
print "querybatchrequest result " .
    " status=" . $qbr->{STATUS} .
    " errorcode=" . $qbr->{ErrorCode} . "\n";
::is( $qbr->{ErrorCode}, 0, $tn);
::is( $qbr->{TOTALNUMBEROFMESSAGES}, 1, $tn );
::is( $qbr->{MESSAGESSENT} == 0 && $qbr->{MESSAGESSKIPPED}, 1, $tn );
::is( $qbr->{STATUS}, 5, $tn );
}

registerTest("SmsEngine o2proprietary retrieveskippedrecords");
my $res = $sendsms->{ObjectSms}->retrieveskippedrecords( { RequestID => $bresult->{RequestID} } );
print "retrieveskippedrecords result " .
    " errorcode=" . $res->{ErrorCode} .
    " MESSAGE_ID=" . $res->{MessagesSkipped}->[0]->{MESSAGE_ID} .
    " ERROR_CODE=" . $res->{MessagesSkipped}->[0]->{ERROR_CODE} .
    " MSISDN=" . $res->{MessagesSkipped}->[0]->{MSISDN} .
    " TIMESKIPPED=" . $res->{MessagesSkipped}->[0]->{TIMESKIPPED} .
    " MESSAGE=" . $res->{MessagesSkipped}->[0]->{MESSAGE} . "\n";

::is( $res->{ErrorCode}, 0, $tn);
::is( $res->{MessagesSkipped}->[0]->{ERROR_CODE}, "202", $tn );
::is( $res->{MessagesSkipped}->[0]->{MSISDN}, $badbatchnum, $tn );

{
# expect ERR API105: Action not supported, submitted request is already FINISHED
registerTest("SmsEngine o2proprietary updatebatchrequest");
my $res = $sendsms->{ObjectSms}->updatebatchrequest( { RequestID => $bresult->{RequestID},
						       StartTime => $in5mins } );
print "updatebatchrequest result " .
    " errorcode=" . $res->{ErrorCode} . "\n";
::is( $res->{ErrorCode}, 105, $tn );
}

{
# expect ERR API105: Action not supported, submitted request is already FINISHED
registerTest("SmsEngine o2proprietary startbatchrequest");
my $res = $sendsms->{ObjectSms}->startbatchrequest( { RequestID => $bresult->{RequestID} } );
print "startbatchrequest result " .
    " errorcode=" . $res->{ErrorCode} . "\n";
::is( $res->{ErrorCode}, 105, $tn );
}

{
# expect ERR API105: Action not supported, submitted request is already FINISHED
registerTest("SmsEngine o2proprietary stopbatchrequest");
my $res = $sendsms->{ObjectSms}->stopbatchrequest( { RequestID => $bresult->{RequestID} } );
print "stopbatchrequest result " .
    " errorcode=" . $res->{ErrorCode} . "\n";
::is( $res->{ErrorCode}, 105, $tn );
}

{
registerTest("SmsEngine o2proprietary removebatchrequest");
my $res = $sendsms->{ObjectSms}->removebatchrequest( { RequestID => $bresult->{RequestID} } );
print "removebatchrequest result " .
    " errorcode=" . $res->{ErrorCode} . "\n";
::is( $res->{ErrorCode}, 0, $tn );
}

# purposely not testing stopallbatchrequests as that could cause problems on live systems




registerTest("Can SmsEngine send PCS picture message?");

my $pcsmsg = "3000000002010000481c01".
    "00000000080000000000000000140000000000000000e300000000c60000004118c00000ee000000360d800000ee73dec8490fbf9e3ceedb9c681c0736c360fefb1868dc0736cf78fec318713e0fb6db3cd6db18313f0db6db0cd6731830fb18f6cf7800000023ffc0000000000000e39de0000000000000019040000000".
    "00000003faf00000000000000ffffc0000000000001f77000000000000000063740000000000003f68bc000000000660f71f3e0000000007e0c1f7fb0290001fe381eb63ff8440001027f1ffe39fc2a0000549c80df79b6770001d6aa9cf3df8e770001feea9ecbe771000001fe8880abeb2177000dfe88fffffffff77ff";

my $to;
$to->[0]="badnumfortest";

my $toparams = { text => qq($pcsmsg), 
		 to => $to,
		 Type => "PCS"};

my $result = $sendsms->{ObjectSms}->sendmessage( $toparams );
#$smsengine->{ObjectSms}->printsendresponse($result) if ($verbose);

{
my $to = "35386badnum";
my $from = "SmsEngine.pm"; # text from not supported by all smscs/handsets
my $text = "SmsEngine.pm example text message";
my $result = $sendsms->sendsms($to,$from,$text);
print "sendsms result: $result " .
    $result->{ErrorCode} . "," .
    $result->{RequestID} . "," .
    $result->{ErrorMessage} . "," .
    $result->{Failed_MSISDN}->[0]->{errorcode} . "," .
    $result->{Failed_MSISDN}->[0]->{num} . "," .
    "\n";

#validate o2proprietary testbed received data
::is( $result->{ErrorCode}, 0, $tn);
::ok( $result->{RequestID} > 0, $tn );
::is( $result->{ErrorMessage}, "Success", $tn );
::is( $result->{Failed_MSISDN}->[0]->{errorcode}, 202, $tn );
}


#example of test validation
#my $r1 = grepFile($listenerlogfile,
#	     "GET /cgi-bin/sendsms?",
#	     "username=testuser",
#	     "password=password",
#             "to=353861111111",
#             "from=SmsEngine.pm",
#             "text=.*example\%20text\%20message");

