#!/usr/bin/perl -w
#!/usr/bin/perl -Tw

use bytes;
use strict;
use lib qw(t/lib);
use Test::More tests => 1 + 2 + 3 + 3 + 2 + 1;

BEGIN { use_ok('SmsEngine::SmsOutToSmsEngineDaemon') }

my $debug = 0;

my $testpath = $0;
$testpath =~ s/\/[^\/]*$/\//; #filename whack
$testpath =~ s/\/lib\/.*$/\/test\//; #other (lib/Doolin/... whack
my $testlistenutil = $testpath . "listener.pl";
die "need executable $testlistenutil" if (! -x $testlistenutil);
my $listenerlogfile = "/tmp/test-smsout-listener.log";

sub testlistener {
    my $host = shift;
    my $port = shift;

    my $resp = "testlistener respond";
    my $cmd = "$testlistenutil -p $port -l $listenerlogfile".
	" -r \"</SMS>\" \"$resp\"".
	" -q \"</SMS>\" -Q \"</SMS>\" &";
    print STDERR "\ncommand is $cmd\n" if ($debug);
    #my $res = `$cmd`; blocks till cmd finished even with &
    my $res = system "$cmd";
    print STDERR "\nres is $res\n" if ($debug);
    return $res;
}



# missing config
{
    my $smsoutbad = SmsEngine::SmsOutToSmsEngineDaemon->new( #debug => 1,
						  file => "doesnotexist.conf");
    ::ok( defined($smsoutbad), "SmsOutToSmsEngineDaemon load config" );
    ::is( $smsoutbad->{error}, 1, "SmsOutToSmsEngineDaemon load config error" );
}

my $smsout = SmsEngine::SmsOutToSmsEngineDaemon->new( debug => 0,
						      file => "smsfilequeue.conf");

# check config loaded okay
{
    ::ok( defined($smsout), "SmsOutToSmsEngineDaemon load config" );
    ::is( $smsout->{error}, undef, "SmsOutToSmsEngineDaemon load config no error" );
    ::is( $smsout->getparam("outpolltime"), 7, "outpolltime param is 10" );
}

#delete $smsout;
$smsout = SmsEngine::SmsOutToSmsEngineDaemon->new( debug => 0,
						      file => "example.conf");

# check config loaded okay
{
    ::ok( defined($smsout), "SmsOutToSmsEngineDaemon load config" );
    ::is( $smsout->{error}, undef, "SmsOutToSmsEngineDaemon load config no error" );
    ::is( $smsout->getparam("outpolltime"), 10, "outpolltime param is 10" );
}

# can we call send
{
    my $invalidmobilenumber = "hi mom";
    my $message = "<SMS><request><mobile>$invalidmobilenumber</mobile>".
	"<msg>TUC smsengine test-o2xmlrpcout.daemon invalid mobile number</msg></request></SMS>";

    my $res = $smsout->send($message);

    my $tn = "connection refused";
    # daemon may or may not be running
    if ($res =~ "^<SMS>") {
	::like( $res, qr!^<SMS><response><errorcode>0</errorcode><msg>Success</msg>!, $tn);
	::like( $res, qr!<mobileerror><mobile>$invalidmobilenumber</mobile>!, $tn);
    } else {
	::like( $res, qr/^error/, $tn);
	::is( $res, 
	      "error Socket could not be created. Reason Connection refused\n",
	      "SmsOutToSmsEngineDaemon create socket error nothing on other end" );
    }

}

# TODO: fire up a listener/responder
# example.conf localhost:8152
my $tlr = testlistener("localhost", 8152);
print STDERR "tlr $tlr\n";
sleep(2); # sleep and allow socket to become active

defined($smsout) && !defined($smsout->{error}) or die $!;

# sendsms success
{
    my $invalidmobilenumber = "hi mom";
    my $message = "<SMS><request><mobile>$invalidmobilenumber</mobile>".
	"<msg>TUC smsengine test-o2xmlrpcout.daemon invalid mobile number</msg></request></SMS>";

    my $res = $smsout->send($message);

    my $tn = "check smsout response";
    #::like( $res, "^error", $tn);
    
    if ($res =~ "^<SMS>") {
	::like( $res, qr!^<SMS><response><errorcode>0</errorcode><msg>Success</msg>!, $tn);
	#::like( $res, "<mobileerror><mobile>$invalidmobilenumber</mobile>", $tn);
	#::like( $res, "<faultcode>202</faultcode>", $tn);
	#::like( $res, "</mobileerror></response></SMS>", $tn);
    } else {
	::is( $res, "testlistener respond", $tn);
    }

    print STDERR "\nres is -$res-\n";
}

# Final wrap up
