#!/usr/bin/perl -w # system calls, readdir, unlink have problems with TAINT
#!/usr/bin/perl -Tw # system calls have problems with TAINT

use strict;
use bytes;
use lib qw(t/lib);
use Test::More tests =>
    1 +          # use_ok
    21 +         # SmsFileQueue create fail
    11           # readnext basic behaviour 2 things in q
    ;

BEGIN { use_ok('SmsEngine::SmsFileQueue') }

my $ti = 1;
my $tn = "";
sub registerTest {
    $ti++;
    $tn = shift;
}


### instantiate config error tests
registerTest("SmsFileQueue missing mandatory params");
my $missparamsms = SmsEngine::SmsFileQueue->new();
::is( $missparamsms->{error}, 1, $tn);
::is( $missparamsms->{errors}->[0],
      'SmsFileQueue::new error: Config file arg and queue name are mandatory', 
      $tn );

$missparamsms = SmsEngine::SmsFileQueue->new(file => "notexistconf.conf");
::is( $missparamsms->{error}, 1, $tn);
::is( $missparamsms->{errors}->[0],
      'SmsFileQueue::new error: Config file arg and queue name are mandatory', 
      $tn );

$missparamsms = SmsEngine::SmsFileQueue->new(queuename => "smsqueue.t");
::is( $missparamsms->{error}, 1, $tn);
::is( $missparamsms->{errors}->[0],
      'SmsFileQueue::new error: Config file arg and queue name are mandatory', 
      $tn );

registerTest("SmsFileQueue cannot find config file");
my $noconfsms = SmsEngine::SmsFileQueue->new( file => "notexistconf.conf",
					      queuename => "smsqueue.t");
::is( $noconfsms->{error}, 1, $tn);
::is( $noconfsms->{errors}->[0],
      'SmsFileQueue::new error: cannot find config file '.
      'No such file or directory notexistconf.conf',
      $tn );
#::is( $noconfsms->{errors}->[0] =~ "cannot find config file", 1, $tn );
#::is( $noconfsms->{errors}->[0] =~ "notexistconf.conf", 1, $tn );

my $badconfsms;
registerTest("SmsFileQueue report config syntax error");
$badconfsms = SmsEngine::SmsFileQueue->new( file => "badconf.conf", 
					    queuename => "smsqueue.t",
					    debug => 0,
					    verbose => 0 );

if (0 && $badconfsms->{error}) {
    use Data::Dumper;
    print Data::Dumper->Dumper($badconfsms);
    foreach my $err(@{$badconfsms->{errors}}) {
	print STDERR "$err\n";
    }
}

::is( $badconfsms->{error}, 1, $tn);
::is( $badconfsms->{errors}->[0] =~ "cannot load config file", 1, $tn );
::is( $badconfsms->{errors}->[0] =~ "etc/badconf.conf", 1, $tn );
::is( $badconfsms->{errors}->[1] =~ "Line 130 in file ", 1, $tn );
::is( $badconfsms->{errors}->[1] =~ "etc/badconf.conf is mal-formed:", 1, $tn );
::is( $badconfsms->{errors}->[1] =~ "bad line", 1, $tn );

registerTest("SmsFileQueue filequeue group does not exist");
$badconfsms = SmsEngine::SmsFileQueue->new( debug => 0,
					    file => "smsfilequeue.conf",
					    queuename => "smsfilequeuenogroup");
::is( $badconfsms->{error}, 1, $tn);
::is( $badconfsms->{errors}->[0], 
      'SmsFileQueue::new error: '.
      'config group [smsfilequeuenogroup] var dir'.
      ' is mandatory for this script.',
      $tn );
::is( $badconfsms->{errors}->[1] =~ "^file: .*smsfilequeue.conf", 1, $tn );

::fail "rmdir test setup" if (-d "/tmp/smsenginefilequeuetest/sillynonexistantdirfortest");

registerTest("SmsFileQueue filequeue dir does not exist");
$badconfsms = SmsEngine::SmsFileQueue->new( debug => 0,
					    file => "smsfilequeue.conf",
					    queuename => "smsfilequeuesillynonexistantdirfortest");
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], 
      qr!.*check permissions.*access or create queue directory.*/tmp/smsenginefilequeuetest/sillynonexistantdirfortest/var/spool/smsfilequeue.*!,
      $tn );


#reproduce permissions problem
# Can't call method "read" on an undefined value at 
#   /usr/lib/perl5/site_perl/5.8.0/Queue/Dir.pm line 304.

registerTest("SmsFileQueue filequeue dir not writable");
my $d = "/tmp/smsenginefilequeuetest/sillydirfortest/var/spool/smsfilequeue";
if ( ! -d $d) {
    my $com = "mkdir -p $d";
    system ($com) == 0 || die "cannot $com $!";
}

chmod 0400, "/tmp/smsenginefilequeuetest/sillydirfortest/var/spool/smsfilequeue"; # r only

$badconfsms = SmsEngine::SmsFileQueue->new( debug => 0,
					    file => "smsfilequeue.conf",
					    queuename => "smsfilequeuesillydirfortest");
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], 
      qr!.*check permissions.*access or create queue directory.*/tmp/smsenginefilequeuetest/sillydirfortest/var/spool/smsfilequeue.*!,
      $tn );

chmod 0200, "/tmp/smsenginefilequeuetest/sillydirfortest/var/spool/smsfilequeue"; # w only
chmod 0000, "/tmp/smsenginefilequeuetest/sillydirfortest/var/spool/smsfilequeue";
chmod 0777, "/tmp/smsenginefilequeuetest/sillydirfortest/var/spool/smsfilequeue";

my $ned = "/tmp/smsenginefilequeuetest/sillydirfortest";
if ( -d $ned) {
    my $rmtestdircom = "rm -rf $ned";
    system ($rmtestdircom) == 0 || die "cannot $rmtestdircom $!";
}

#### okay
### enough error case create tests

# create dir and check is ready for us to write to it

# nuke old test data
if ( -d "/tmp/smsenginefilequeuetest") {
    my $rmtestdircom = "rm -rf /tmp/smsenginefilequeuetest";
    system ($rmtestdircom) == 0 || die "cannot $rmtestdircom $!";
}
::fail "rmdir test setup" if ( -d "/tmp/smsenginefilequeuetest");

# create empty queue dir
my $testdir = "/tmp/smsenginefilequeuetest/var/spool/smsfilequeue";
my $mkdircom = "mkdir -p $testdir";
system ($mkdircom) == 0 || die "cannot $mkdircom $!";
if ( ! -d $testdir) {
    die "cannot create or access $testdir ?$!?";
}

# instantiate okay
registerTest("SmsFileQueue create okay");
my $smsfq = SmsEngine::SmsFileQueue->new( debug => 1, verbose => 1,
					  file => "smsfilequeue.conf",
					  queuename => "smsfilequeue");
::ok( defined($smsfq), $tn );
::is( $smsfq->{error}, undef, $tn);
::is( $smsfq->{errors}->[0], undef, $tn);
::is( $smsfq->getparam("file"), "smsfilequeue.conf", $tn );
::is( $smsfq->getparam("dir"), 
      '/tmp/smsenginefilequeuetest/var/spool/smsfilequeue', $tn );
#::is( $smsfq->getparam("basedir"), 1, $tn );
::is( $smsfq->getparam("queuedir"), 
      '/tmp/smsenginefilequeuetest/var/spool/smsfilequeue', $tn );
::is( $smsfq->getparam("queuename"), 'smsfilequeue', $tn );
::is( $smsfq->getparam("queueconfiggroup"), 'smsfilequeue', $tn );
::is( $smsfq->getparam("debug"), 0, $tn );
::is( $smsfq->getparam("verbose"), 0, $tn );

if ($smsfq->{error}) {
    my $errs;
    foreach my $err(@{$smsfq->{errors}}) {
	print "$err\n";
	$errs .= "$err\n";
    }
    my $err = "cannot instantiate SmsFileQueue :( installation or config problem.\n$errs";
    die $err;
}


{

    my $msg = "test message smsqueue.t";
    my $to = "badtonumber";

    my $req = { text => qq($msg),
		to => [ $to ] };

    my $res = $smsfq->enqueue( $req );

    ::is( $res->{error}, 0, $tn);

    if ($res->{error} != 0) {
	my $err = "could happen if someone removes the Q directory". 
	    " or disk goes away or file system is full.\n".
	    $res->{errors}->[0];
	warn $err;
    }

    # TODO: verify item is enqueued ? 
}

# TODO: dequeue items, readnext

# Final wrap up
