#!/usr/bin/perl -Tw

# the existance of t directory doesn't override test.pl (unless tests in there fail)
#  also tests under t are assumed to use t/lib ? and Test::More
# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl test.pl'

# DEBUGGER make testdb TEST_FILE=t/xiamsms.t TEST_VERBOSE=1
# make TEST_FILES=t/xiamsms-config.t test

use strict;
use bytes;

use lib qw(t/lib);
use Test::More tests => 
    1 +          # use_ok
    8 +          # xiamsms smsengine create
    9 +          # filter config
    12 +         # filter application
    3 ;          # no config file
#   x ;              # TODO: config syntax errors
BEGIN { use_ok('SmsEngine::SmsEngine') }

my $ti = 1;
my $tn = "";
sub registerTest
{
    $ti++;
    $tn = shift;
}

my $goodresult;

registerTest("Does SmsEngine load config?");

#start xiamsms testbed
my $sendsms = SmsEngine->new( file => "xiamexample.conf", 
			      debug => 1,
			      verbose => 0 );
			      #iface => "o2xiam_iface", 
			      #account => "topupclub_o2irl_xiam_account_live" );

registerTest("create SmsEngine XiamSms");
::ok(defined($sendsms), $tn);
die "subsequent tests need successful xiam smsengine create" 
    if (!defined($sendsms));
::like($sendsms->getparam("alarmemail"), qr/jamesc/,$tn);
#$sendsms->showconfig();

registerTest("Can SmsEngine speak xiamsms? test config");
$sendsms->{ObjectSms}->showconfig();
#print "getparam(https_xmlrpc_url) = " . 
#    $sendsms->getparam("https_xmlrpc_url") . "\n";

::like( $sendsms->getparam("url"), qr!http://!,  $tn);
::is( $sendsms->getparam("providerid"), 129,  $tn );
::is( $sendsms->getparam("from"), 53125,  $tn );
::is( $sendsms->getparam("database"), "database",  $tn );
::ok( $sendsms->getparam("smsinaddress") eq "localhost", $tn );
::ok( $sendsms->getparam("smsinport") > 0 , $tn );

registerTest("Can SmsEngine speak xiamsms? test filter config");
my $smsinfiltersettings = $sendsms->getparam("smsinfiltersettings");
::like( $sendsms->getparam("smsinfilter"), qr!free_filter!,  $tn);
::like( $sendsms->getparam("smsinfilter"), qr!non_free_filter!,  $tn );
::like( $sendsms->getparam("smsinfilter"), qr!default_filter!,  $tn );
::ok( defined($smsinfiltersettings), $tn );
::is( $smsinfiltersettings->{free_filter}->{object}, "filter", $tn );
::is( $smsinfiltersettings->{free_filter}->{shortcode}, "353863833342", $tn );
::like( $smsinfiltersettings->{free_filter}->{smsinkeywords}, qr/REGISTER/, $tn );
::ok( $smsinfiltersettings->{free_filter}->{priority}, $tn );
::ok( $smsinfiltersettings->{free_filter}->{swallow}, $tn );

if (0) { foreach my $filterset(keys(%$smsinfiltersettings)) {
    #print STDERR "Dump filterset " .
    #    Data::Dumper->Dumper($filterset) . "\n";
    print "XiamsmsSms arg filter $filterset .oO0Oo.".
	$smsinfiltersettings->{$filterset}."\n";
    foreach my $filterthing(keys(%{$smsinfiltersettings->{$filterset}})) {
	print "  $filterthing = ".$smsinfiltersettings->{$filterset}->{$filterthing}."\n";
	
    }
}}


{
registerTest("xiamsms filter application");

{
# TODO: need filter priority/order  
#   and  if filter catches something stop there or continue
my ($shortcode, $message) = ("no shortcode match so message allowed by default filter", "no match" );
my ($result,$filtername,$failmessage,$address,$port) = 
    $sendsms->{ObjectSms}->applyfilters($shortcode,$message);
::is( $result, 1, $tn);
::is( $filtername, "default_filter", $tn );
::ok( !defined($failmessage), $tn );
}

{
my ($shortcode, $message) = ("353863833342", "shortcode match, no msg match so rejected" );
my ($result,$filtername,$failmessage,$address,$port) = 
    $sendsms->{ObjectSms}->applyfilters($shortcode,$message);
::is( $result, 1, $tn);
::is( $filtername, "free_filter", $tn );
::ok( defined($failmessage), $tn );
}

{
my ($shortcode, $message) = ("353863832032", "non free shortcode match" );
my ($result,$filtername,$failmessage,$address,$port) = 
    $sendsms->{ObjectSms}->applyfilters($shortcode,$message);
::is( $result, 1, $tn);
::is( $filtername, "non_free_filter", $tn );
::ok( !defined($failmessage), $tn );
}

{
my ($shortcode, $message) = ("353863833342", "TUC REGISTER shortcode and REGISTER match" );
my ($result,$filtername,$failmessage,$address,$port) = 
    $sendsms->{ObjectSms}->applyfilters($shortcode,$message);
::is( $result, 1, $tn);
::is( $filtername, "free_filter", $tn );
::ok( !defined($failmessage), $tn );
}
}

### daemon code if (defined($failmessage) then send failmessage back
### if defined($address) && defined($port) not done yet
### otherwise pass it on as usual

# DONE: do not die (SmsEngine) on error, return an error and undef instead,
#       lower layers (SmsEngine itself should print to STDERR the errors?)
# DONE: all scripts which instantiate SmsEngine now must check
#       if error defined (and maybe die)
# DONE: bad config files should not produce cryptic errors
# TODO: test space after port numbers in config file  (code was added to fix that)
# DONE: test line not matching group start "[groupname]" or group var "varname = value"

### whole slew of config parse tests

registerTest("SmsEngine report no config error, do not die");
my $noconfsms = SmsEngine->new( file => "notexistconf.conf" );
::is( $noconfsms->{error}, 1, $tn);
::like( $noconfsms->{errors}->[0], qr/cannot find config file/, $tn );
::like( $noconfsms->{errors}->[0], qr/notexistconf.conf/, $tn );


if (0){
my $badconfsms;
registerTest("SmsEngine report config syntax error");
$badconfsms = SmsEngine->new( file => "badconf.conf", 
			      debug => 0,
			      verbose => 0 );

if (0 && $badconfsms->{error}) {
    #foreach my $err(@Config::IniFiles::errors) {
    #	print "IniFile error $err\n";
    #}
    use Data::Dumper;
    print Data::Dumper->Dumper($badconfsms);
    foreach my $err(@{$badconfsms->{errors}}) {
	print "$err\n";
    }
}

::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!cannot load config file!, $tn );
::like( $badconfsms->{errors}->[0], qr!etc/badconf.conf!, $tn );
::like( $badconfsms->{errors}->[1], qr!Line 130 in file !, $tn );
::like( $badconfsms->{errors}->[1], qr!etc/badconf.conf is mal-formed:!, $tn );
::like( $badconfsms->{errors}->[1], qr!bad line!, $tn );

registerTest("SmsEngine report config error no ifacegroup");
$badconfsms = SmsEngine->new( file => "badconf-noifacegroup.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::is( $badconfsms->{errors}->[0] 
          =~ "config group \\[commonsms\\] var ifacegroup is mandatory", 1, $tn );

registerTest("SmsEngine report config error not an iface");
$badconfsms = SmsEngine->new( file => "badconf-notaniface.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadiface config error with ifacegroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!object not_iface not an interface?!, $tn );

registerTest("SmsEngine report config error missing iface param");
$badconfsms = SmsEngine->new( file => "badconf-ifacemissingaccount.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadiface config error with ifacegroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!object myo2iface!, $tn );
::like( $badconfsms->{errors}->[1], qr!argument accountgroup is mandatory!, $tn );
$badconfsms = SmsEngine->new( file => "badconf-ifacemissinginout.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadiface config error with ifacegroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!object myo2iface!, $tn );
::like( $badconfsms->{errors}->[1], qr!argument inoutgroup is mandatory!, $tn );

registerTest("SmsEngine report config error missing iface or account group");
$badconfsms = SmsEngine->new( file => "badconf-missingaccount.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadaccount config error with accountgroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!not an account?!, $tn );
::like( $badconfsms->{errors}->[1], qr!notheretopupclub_o2_account_live!, $tn );
$badconfsms = SmsEngine->new( file => "badconf-missinginout.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadiface config error with ifacegroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!inout object notheresmsengine_tucd!, $tn );
::like( $badconfsms->{errors}->[1], qr!argument outaddress is mandatory!, $tn );
#    print Data::Dumper->Dumper($badconfsms);


registerTest("SmsEngine report config error missing iface params");
$badconfsms = SmsEngine->new( file => "badconf-ifacemissing-outaddress.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadiface config error with ifacegroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!inout object smsengine_tucd!, $tn );
::like( $badconfsms->{errors}->[1], qr!argument outaddress is mandatory!, $tn );
$badconfsms = SmsEngine->new( file => "badconf-ifacemissing-outport.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadiface config error with ifacegroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!inout object smsengine_tucd!, $tn );
::like( $badconfsms->{errors}->[1], qr!argument outport is mandatory!, $tn );
$badconfsms = SmsEngine->new( file => "badconf-ifacemissing-inaddress.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadiface config error with ifacegroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!inout object smsengine_tucd!, $tn );
::like( $badconfsms->{errors}->[1], qr!argument inaddress is mandatory!, $tn );
$badconfsms = SmsEngine->new( file => "badconf-ifacemissing-inport.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadiface config error with ifacegroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!inout object smsengine_tucd!, $tn );
::like( $badconfsms->{errors}->[1], qr!argument inport is mandatory!, $tn );

$badconfsms = SmsEngine->new( file => "badconf-ifacemissing-multiple.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadiface config error with ifacegroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!object myo2iface!, $tn );
::like( $badconfsms->{errors}->[1], qr!argument accountgroup is mandatory!, $tn );
#::like( $badconfsms->{errors}->[2], qr!object myo2iface!, $tn );
#::like( $badconfsms->{errors}->[2], qr!argument inoutgroup is mandatory!, $tn );
::like( $badconfsms->{errors}->[2], qr!inout object smsengine_tucd!, $tn );
::like( $badconfsms->{errors}->[2], qr!argument outaddress is mandatory!, $tn );
::like( $badconfsms->{errors}->[3], qr!argument outport is mandatory!, $tn );
::like( $badconfsms->{errors}->[4], qr!argument inaddress is mandatory!, $tn );
::like( $badconfsms->{errors}->[5], qr!argument inport is mandatory!, $tn );

registerTest("SmsEngine report config error missing o2 account params");
$badconfsms = SmsEngine->new( file => "badconf-o2-missing-smsinport.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadaccount config error with accountgroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!XiamsmsSms loadaccount!, $tn );
::like( $badconfsms->{errors}->[1], qr!argument smsinport is mandatory!, $tn );
$badconfsms = SmsEngine->new( file => "badconf-o2-missing-smsinaddress.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadaccount config error with accountgroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!XiamsmsSms loadaccount!, $tn );
::like( $badconfsms->{errors}->[1], qr!argument smsinaddress is mandatory!, $tn );

registerTest("SmsEngine report config error missing o2 account file");
$badconfsms = SmsEngine->new( file => "badconf-o2-missing-certfile.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadaccount config error with accountgroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!XiamsmsSms loadaccount!, $tn );
::like( $badconfsms->{errors}->[1], qr!Can\'t read file!, $tn );
$badconfsms = SmsEngine->new( file => "badconf-o2-missing-keyfile.conf"); 
::is( $badconfsms->{error}, 1, $tn);
::like( $badconfsms->{errors}->[0], qr!loadaccount config error with accountgroup!, $tn );
::like( $badconfsms->{errors}->[1], qr!XiamsmsSms loadaccount!, $tn );
::like( $badconfsms->{errors}->[1], qr!Can\'t read file!, $tn );


#    print Data::Dumper->Dumper($badconfsms);
# TODO: (low priority) full test fake and kannel account loading

# bad libs they die die die :(
#exit(19);

# DONE in daemons, not in libs: emergency message if can't forward to tucd 
#  specifically mentioning port and config file
#   <enrico> james, seems to be working now, in port was set to 8151, changed
#        it back to 28222 and it worked. inport of group [smsengine_tucd]
}




