#!/usr/bin/perl -w
#!/usr/bin/perl -Tw

# make TEST_FILES=t/xiamsms-rx.t test
# 

use strict;
use bytes;

use lib qw(t/lib);
use Test::More tests => 
    1 +          # use_ok
    6 +          # xiamsms smsengine create
    3 +          # test log deliver reports (pretend receive sms)
    2 +          # test query message status (not found)
    6 +          # test query message status
    1;           # test query batch message status

BEGIN { use_ok('SmsEngine::SmsEngine') }

my $ti = 1;
my $tn = "";
sub registerTest
{
    $ti++;
    $tn = shift;
}

my $goodresult;

registerTest("Does SmsEngine load config?");

#start xiamsms testbed
my $sendsms = SmsEngine->new( file => "xiamexample.conf", 
			      debug => 1,
			      verbose => 0 );
			      #iface => "o2xiam_iface", 
			      #account => "topupclub_o2irl_xiam_account_live" );

registerTest("create SmsEngine XiamSms");
::ok(defined($sendsms), $tn);
die "subsequent tests need successful xiam smsengine create" 
    if (!defined($sendsms));
::like($sendsms->getparam("alarmemail"), qr/jamesc/,$tn);
#$sendsms->showconfig();

registerTest("Can SmsEngine speak xiamsms? test config");
$sendsms->{ObjectSms}->showconfig();
#print "getparam(https_xmlrpc_url) = " . 
#    $sendsms->getparam("https_xmlrpc_url") . "\n";

::like( $sendsms->getparam("url"), qr!http://!,  $tn);
::is( $sendsms->getparam("providerid"), 129,  $tn );
::ok( $sendsms->getparam("smsinaddress") eq "localhost", $tn );
::ok( $sendsms->getparam("smsinport") > 0 , $tn );

{
    registerTest("pretend to receive sms, log deliver reports in db");

    my $dr;
    $dr->{messageid} = "xiamsms-rxtest001";
    ($dr->{status},
     $dr->{errorcode},
     $dr->{errormessage},
     $dr->{startdate},
     $dr->{totalnumberofmessages},
     $dr->{messagessent},
     $dr->{messagesskipped}) = (0,0,"Success",1111,1,1,0);

    my $xiamifaceid = 0; # ngeh
    my $ldr = $sendsms->{ObjectSms}->logdeliverreportindb($dr,$xiamifaceid);
    ::is($ldr, 0, $tn);

    $dr->{messageid} = "xiamsms-rxtest002";
    ($dr->{status},
     $dr->{errorcode},
     $dr->{errormessage},
     $dr->{startdate},
     $dr->{totalnumberofmessages},
     $dr->{messagessent},
     $dr->{messagesskipped}) = (1,1,"Fail",2222,1,1,0);

    $ldr = $sendsms->{ObjectSms}->logdeliverreportindb($dr,$xiamifaceid);
    ::is($ldr, 0, $tn);

    $dr->{messageid} = "xiamsms-rxtest003";
    ($dr->{status},
     $dr->{errorcode},
     $dr->{errormessage},
     $dr->{startdate},
     $dr->{totalnumberofmessages},
     $dr->{messagessent},
     $dr->{messagesskipped}) = (2,0,"Accepted",3333,1,1,0);

    $ldr = $sendsms->{ObjectSms}->logdeliverreportindb($dr,$xiamifaceid);
    ::is($ldr, 0, $tn);
}

registerTest("SmsEngine xiamsms querysenddeliverystatus requestid not found");
my $qsds = $sendsms->{ObjectSms}->querysenddeliverystatus( 
              { RequestID => "xiamsms-notfound"} );

::ok( $qsds->{ErrorCode} > 0, $tn );
::like( $qsds->{ErrorMessage}, qr/^no matching thing xiamsms-notfound/, $tn );

registerTest("SmsEngine xiamsms querysenddeliverystatus");
$qsds = $sendsms->{ObjectSms}->querysenddeliverystatus( 
	    { RequestID => "xiamsms-rxtest001"} );

::is( $qsds->{STATUS}, 0, $tn );
::is( $qsds->{ErrorCode}, 0, $tn );
::is( $qsds->{ErrorMessage}, "Success", $tn );
::is( $qsds->{TOTALNUMBEROFMESSAGES}, 1, $tn);
::is( $qsds->{MESSAGESSENT}, 1, $tn );
::is( $qsds->{MESSAGESSKIPPED}, 0, $tn );


# possible that there would be batch id in system with that number ... but not that likely
# we expect ERR API053: Request is not a valid BATCH request
registerTest("SmsEngine xiamsms querybatchrequest invalid id");
my $qbr = $sendsms->{ObjectSms}->querybatchrequest( { RequestID => "xiamsms-batchnotfound" } );
print "querybatchrequest result " .
    #" status=" . $qbr->{STATUS} .
    " errorcode=" . $qbr->{ErrorCode} . "\n";
::is($qbr->{ErrorCode}, 1, $tn);

