#!/usr/bin/perl -Tw

# make TEST_FILES=t/xiamsms.t test

use strict;
use bytes;

use lib qw(t/lib);
use Test::More tests => 
    1 +          # use_ok
    6 +          # xiamsms smsengine create, load config
    11 +         # sendsms
#   3 +          # network down
    7 +          # querysenddeliverystatus
    8;           # o2 send batch, query batch status

BEGIN { use_ok('SmsEngine::SmsEngine') }

my $ti = 1;
my $tn = "";
sub registerTest
{
    $ti++;
    $tn = shift;
}

my $testtonum = "35386xxbadnum";

my $goodresult;

registerTest("Does SmsEngine load config?");

#start xiamsms testbed
my $sendsms = SmsEngine->new( file => "xiamexample.conf", 
			      debug => 1,
			      verbose => 0 );
			      #iface => "o2xiam_iface", 
			      #account => "topupclub_o2irl_xiam_account_live" );

registerTest("create SmsEngine XiamSms");
::ok(defined($sendsms), $tn);
die "subsequent tests need successful xiam smsengine create" 
    if (!defined($sendsms));
::like($sendsms->getparam("alarmemail"), qr/jamesc/,$tn);
#$sendsms->showconfig();

registerTest("Can SmsEngine speak xiamsms? test config");
$sendsms->{ObjectSms}->showconfig();
#print "getparam(https_xmlrpc_url) = " . 
#    $sendsms->getparam("https_xmlrpc_url") . "\n";

::like( $sendsms->getparam("url"), qr!http://!,  $tn);
::is( $sendsms->getparam("providerid"), 129,  $tn );
::ok( $sendsms->getparam("smsinaddress") eq "localhost", $tn );
::ok( $sendsms->getparam("smsinport") > 0 , $tn );

### daemon code if (defined($failmessage) then send failmessage back
### if defined($address) && defined($port) not done yet
### otherwise pass it on as usual

{
registerTest("Can SmsEngine speak xiamsms? call sendsms");

my $to = $testtonum;
my $from = undef; # "SmsEngine.pm"; # text from not supported by all smscs/handsets
my $text = "SmsEngine.pm example text message";
my $res = $sendsms->sendsms($to,$from,$text);
#print "sendsms result: . $res\n";

# TODO: keep compatibility with O2 MMS return ?
# use HTTP::Daemon return + xiam return combined
# result is a HTTP::Response=HASH(0x870058c)

# TODO multiple status of to numbers/messages
#    if ($res->is_success) {
#	print $res->content, "\n";
#	print LOG $res->content, "\n";
#    } else {
#	print $res->status_line, "\n";
#	print LOG $res->status_line, "\n";

#print $result->{ErrorCode} . "," .
#    $result->{RequestID} . "," .
#    $result->{ErrorMessage} . "," .
#    $result->{Failed_MSISDN}->[0]->{errorcode} . "," .
#    $result->{Failed_MSISDN}->[0]->{num} . "," .
#    "\n";

::ok( $res->is_success, $tn);
::is( $res->status_line, "200 1 message sent", $tn);
::like( $res->content, qr!\<\?xml version="1.0" encoding="UTF-8"\?\>!, $tn);
::like( $res->content, qr#\<!DOCTYPE xiamSMS SYSTEM "xiamSMSMessage.dtd">#, $tn);
::like( $res->content, qr!<xiamSMS status="OK" statusText="XML contained 1 xir message">!, $tn);
::like( $res->content, qr!<submitResponse id=!, $tn);
::like( $res->content, qr!\<result status="OK" statusText=""\>[+]?$to\</result\>!, $tn);
::like( $res->content, qr!</submitResponse></xiamSMS>!, $tn);

::is( $res->{ErrorCode}, 0, $tn);
::ok( $res->{RequestID}, $tn );
::is( $res->{ErrorMessage}, "Success", $tn );

$goodresult = $res;
}

#if (1) { 
#    warn "remove me before checkin";
#    warn "temporary exit to speed up testing while implementing filtering";
#    exit(0); 
#} 


if (0) { # test annoyingly long timeout
registerTest("xiamsms network down");
#(in config server exists but nothing on port)
#big nasty 5min? 500sec? =~ 8min? timeout  
my $brokensms = SmsEngine->new( file => "brokenexample.conf", 
			      debug => 0,
			      verbose => 0 );
			      #iface => "xiamsms_iface", 
			      #account => "topupclub_o2_account" );

my $to = "35386badnum";
my $from = "SmsEngine.pm"; # text from not supported by all smscs/handsets
my $text = "SmsEngine.pm example text message";
my $brokenresult = $brokensms->sendsms($to,$from,$text);
print "sendsms result: $brokenresult " .
    $brokenresult->{RequestID} . "," .
    $brokenresult->{ErrorCode} . "," .
    $brokenresult->{ErrorMessage} . "," .
    "\n";

::is( $brokenresult->{ErrorCode}, 8152, $tn);
::like( $brokenresult->{ErrorMessage}, qr!SmsEngine connect Error!,, $tn );
::is( $brokenresult->{ErrorMessage}, "Success", $tn );
}


#use Data::Dumper; print STDERR Dumper($goodresult);

die "subsequent tests need sendsms test to pass and retrieve good RequestID" 
    if (!defined($goodresult->{RequestID}) || !$goodresult->{RequestID});

registerTest("SmsEngine xiamsms querysenddeliverystatus");
my $qsds = $sendsms->{ObjectSms}->querysenddeliverystatus( 
              { RequestID => $goodresult->{RequestID} } );

# might be in any state, might not be finished
use Data::Dumper; print STDERR Dumper($qsds);
print "querysenddeliverystatus result " .
    $qsds->{TOTALNUMBEROFMESSAGES} ."<=".
    $qsds->{MESSAGESSENT} ."+". $qsds->{MESSAGESSKIPPED} .
    " status=" . $qsds->{STATUS} .
    " errorcode=" . $qsds->{ErrorCode} . "\n";
::is( $qsds->{smsengine_messageid}, $goodresult->{RequestID}, $tn);
::is( $qsds->{TOTALNUMBEROFMESSAGES}, 0, $tn); # status unknown should be 1
::is( $qsds->{MESSAGESSENT}, 0, $tn );
::is( $qsds->{MESSAGESSKIPPED}, 0, $tn );
::ok( $qsds->{MESSAGESSENT} + $qsds->{MESSAGESSKIPPED} <= 1, $tn );
::ok( $qsds->{STATUS} <= 5, $tn );
::is( $qsds->{ErrorCode}, 0, $tn );




# xiam xml/sms batchid == normal messageid
# we expect ERR API053: Request is not a valid BATCH request
registerTest("SmsEngine xiamsms querybatchrequest invalid id");
my $qbr = $sendsms->{ObjectSms}->querybatchrequest( { RequestID => $goodresult->{RequestID} } );
print "querybatchrequest result " .
    #" status=" . $qbr->{STATUS} .
    " errorcode=" . $qbr->{ErrorCode} . "\n";
::is($qbr->{ErrorCode}, 0, $tn);

my $in10mins = time() + 600;
my $in5mins = time() + 300;

registerTest("SmsEngine xiamsms sendbatchmessage");
my $badbatchnum = "35386badbatchnum";
my $btoparams = { text => "perl module test SmsEngine xiamsms sendbatchmessage", 
		  to => [ $badbatchnum, $testtonum ],
		  StartTime => $in10mins };
my $bresult = $sendsms->{ObjectSms}->sendbatchmessage( $btoparams );
print "sendbatchmessage result " .
    " id=" . $bresult->{RequestID} .
    " errorcode=" . $bresult->{ErrorCode} . "\n";
::is( $bresult->{ErrorCode}, 0, $tn);
::ok( defined($bresult->{RequestID}), $tn );

{
registerTest("SmsEngine xiamsms querybatchrequest valid id");
my $qbr = $sendsms->{ObjectSms}->querybatchrequest( { RequestID => $bresult->{RequestID} } );
print "querybatchrequest result " .
    " status=" . $qbr->{STATUS} .
    " errorcode=" . $qbr->{ErrorCode} . "\n";
::is( $qbr->{ErrorCode}, 0, $tn);
::is( $qbr->{TOTALNUMBEROFMESSAGES}, 0, $tn );   # status unknown should be 2
::is( $qbr->{MESSAGESSENT}, 0, $tn );            # status unknown should be 1
::is( $qbr->{MESSAGESSKIPPED}, 0, $tn );         # status unknown should be 1
::is( $qbr->{STATUS}, 2, $tn );
# status as of yet unknown

}




