# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl test.pl'

######################### We start with some black magic to print on failure.

# Change 1..1 below to 1..last_test_to_print .
# (It may become useful if the test is moved to ./t subdirectory.)

BEGIN { $| = 1; print "1..5\n"; }
#END {print "not ok 1\n" unless $loaded;}
use strict;
use SmsEngine::SmsEngine;
my $loaded = 1;
print "ok 1\n";

######################### End of black magic.

# Insert your test code below (better if it prints "ok 13"
# (correspondingly "not ok 13") depending on the success of chunk 13
# of the test code):

my $grepFileError = "";

sub grepFile
{
    my $file = shift; my $r = 1; my $str; my $res;
    while ($r && ($str = shift)) {
        $res = `grep -c "$str" $file`;
        if ($res == "") { 
            $r = 0; 
            $grepFileError = "could not find $str in $file";
        }
    }
    return $r;
}

my $ti = 1;
my $tn = "";
sub registerTest
{
    $ti++;
    $tn = shift;
}


sub testResult
{
    my $r = shift;
    print "TEST $ti $tn ".(($r)?"PASS":"FAIL")."\n";
    print (($r)?"ok $ti\n":"not ok $ti\n");
    if (!$r) { 
        print "Error: $grepFileError\n";
    }
}


registerTest("smsengine tests moved to t/smsengine.t");
testResult(1);
exit 0;

registerTest("Does SmsEngine load config?");

my $sendsms = SmsEngine->new( file => "example.conf" );
# if we're not dead config was found
testResult(1);

registerTest("Does SmsEngine load fakesms config?");
$sendsms->loadiface("fake_iface");
$sendsms->loadaccount("fake_account");
testResult(1);

registerTest("Does SmsEngine send sms using fakesms config?");
my $to = "353861111111";
my $from = "SmsEngine.pm"; # text from not supported by all smscs/handsets
my $text = "SmsEngine.pm example text message";
my $result = $sendsms->sendsms($to,$from,$text);
undef $sendsms;

my $r1 = grepFile("/tmp/fakeSmsEngineIface.log",
		  "SmsEngine Fake sendsms to $to from SmsEngine.pm text SmsEngine.pm example text message");

testResult($r1);

registerTest("Can SmsEngine speak kannel?");

#TODO: start fake kannel testbed
#TODO: account deprecated
my $sendsms = SmsEngine->new( file => "example.conf", 
			      ifacegroup => "smskannel_iface", 
			      account => "clickatell_kannel_account" );

my $to = "35386badnum";
my $from = "SmsEngine.pm"; # text from not supported by all smscs/handsets
my $text = "SmsEngine.pm example text message";
my $result = $sendsms->sendsms($to,$from,$text);
print "sendsms result: $result\n";

#TODO: validate kannel testbed received data
testResult(1);


#start o2proprietary testbed
my $sendsms = SmsEngine->new( file => "example.conf", 
			      debug => 0,
			      verbose => 0 );
			      #iface => "o2proprietary_iface", 
			      #account => "topupclub_o2_account" );

registerTest("Can SmsEngine speak o2proprietary? test global config alarmemail");
testResult( $sendsms->getparam("alarmemail") =~ "jamesc" );
#$sendsms->showconfig();

registerTest("Can SmsEngine speak o2proprietary? test o2 proprietary config");
$sendsms->{ObjectSms}->showconfig();
#print "getparam(https_xmlrpc_url) = " . 
#    $sendsms->getparam("https_xmlrpc_url") . "\n";

testResult( $sendsms->getparam("https_xmlrpc_url") =~ "https://" &&
	    $sendsms->getparam("https_cert_file") =~ "^/.*\.pem" &&
	    $sendsms->getparam("https_pkcs12_file") =~ "^/.*\.p12" &&
	    $sendsms->getparam("https_pkcs12_password") ne "" &&
	    $sendsms->getparam("proxy") =~ "http://proxy:8080" &&
	    $sendsms->getparam("smsinaddress") eq "localhost" &&
	    $sendsms->getparam("smsinport") > 0
	    );

registerTest("Can SmsEngine speak o2proprietary? test filter config");
my $smsinfiltersettings = $sendsms->getparam("smsinfiltersettings");
testResult( $sendsms->getparam("smsinfilter") =~ "free_filter" &&
	    $sendsms->getparam("smsinfilter") =~ "non_free_filter" &&
	    $sendsms->getparam("smsinfilter") =~ "default_filter" &&
	    defined($smsinfiltersettings) && 
	    $smsinfiltersettings->{free_filter}->{object} eq "filter" &&
	    $smsinfiltersettings->{free_filter}->{shortcode} eq "353863833342" &&
	    $smsinfiltersettings->{free_filter}->{smsinkeywords} =~ "REGISTER" &&
	    $smsinfiltersettings->{free_filter}->{priority} == 1 &&
	    $smsinfiltersettings->{free_filter}->{swallow} == 1
	    );

if (0) { foreach my $filterset(keys(%$smsinfiltersettings)) {
    #print STDERR "Dump filterset " .
    #    Data::Dumper->Dumper($filterset) . "\n";
    print "O2ProprietarySms arg filter $filterset .oO0Oo.".
	$smsinfiltersettings->{$filterset}."\n";
    foreach my $filterthing(keys(%{$smsinfiltersettings->{$filterset}})) {
	print "  $filterthing = ".$smsinfiltersettings->{$filterset}->{$filterthing}."\n";
	
    }
}
}

registerTest("o2proprietary filter application");

# TODO: need filter priority/order  
#   and  if filter catches something stop there or continue
my ($shortcode, $message) = ("no shortcode match so message allowed by default filter", "no match" );
my ($result,$filtername,$failmessage,$address,$port) = 
    $sendsms->{ObjectSms}->applyfilters($shortcode,$message);
testResult ( $result == 1 && 
	     $filtername eq "default_filter" && 
	     !defined($failmessage));

($shortcode, $message) = ("353863833342", "shortcode match, no msg match so rejected" );
my ($result,$filtername,$failmessage,$address,$port) = 
    $sendsms->{ObjectSms}->applyfilters($shortcode,$message);
testResult ( $result == 1 && 
	     $filtername eq "free_filter" && 
	     defined($failmessage));

($shortcode, $message) = ("353863832032", "non free shortcode match" );
my ($result,$filtername,$failmessage,$address,$port) = 
    $sendsms->{ObjectSms}->applyfilters($shortcode,$message);
testResult ( $result == 1 && 
	     $filtername eq "non_free_filter" && 
	     !defined($failmessage));

($shortcode, $message) = ("353863833342", "TUC REGISTER shortcode and REGISTER match" );
my ($result,$filtername,$failmessage,$address,$port) = 
    $sendsms->{ObjectSms}->applyfilters($shortcode,$message);
testResult ( $result == 1 && 
	     $filtername eq "free_filter" && 
	     !defined($failmessage) );

### daemon code if (defined($failmessage) then send failmessage back
### if defined($address) && defined($port) not done yet
### otherwise pass it on as usual

registerTest("Can SmsEngine speak o2proprietary? call sendsms");

my $result = $sendsms->sendsms($to,$from,$text);
print "sendsms result: $result " .
    $result->{ErrorCode} . "," .
    $result->{RequestID} . "," .
    $result->{ErrorMessage} . "," .
    $result->{Failed_MSISDN}->[0]->{errorcode} . "," .
    $result->{Failed_MSISDN}->[0]->{num} . "," .
    "\n";

#validate o2proprietary testbed received data
testResult( $result->{ErrorCode} == 0 && 
	    $result->{RequestID} > 0 && 
	    $result->{ErrorMessage} eq "Success" && 
	    $result->{Failed_MSISDN}->[0]->{errorcode} == 202 &&
	    $result->{Failed_MSISDN}->[0]->{num} eq "35386badnum"
	    );

#if (1) { 
#    warn "remove me before checkin";
#    warn "temporary exit to speed up testing while implementing filtering";
#    exit(0); 
#} 




# TODO: do not die (SmsEngine) on error, return an error and undef instead,
#       lower layers (SmsEngine itself should print to STDERR the errors?)
# TODO: all scripts which instantiate SmsEngine now must check
#       if error defined (and maybe die)
# TODO: bad config files should not produce cryptic errors
# TODO: test space after port numbers in config file  (code was added to fix that)
# TODO: test line not matching group start "[groupname]" or group var "varname = value"

### whole slew of config parse tests

registerTest("SmsEngine report no config error, do not die");
my $noconfsms = SmsEngine->new( file => "notexistconf.conf" );
testResult( $noconfsms->{error} &&
	    $noconfsms->{errors}->[0] =~ "cannot find config file" &&
	    $noconfsms->{errors}->[0] =~ "notexistconf.conf");

registerTest("SmsEngine report config syntax error");
my $badconfsms = SmsEngine->new( file => "badconf.conf", 
			      debug => 0,
			      verbose => 0 );

if (0 && $badconfsms->{error}) {
    #foreach my $err(@Config::IniFiles::errors) {
    #	print "IniFile error $err\n";
    #}
    use Data::Dumper;
    print Data::Dumper->Dumper($badconfsms);
    foreach my $err(@{$badconfsms->{errors}}) {
	print "$err\n";
    }
}

testResult( $badconfsms->{error} &&
	    $badconfsms->{errors}->[0] =~ "cannot load config file" &&
	    $badconfsms->{errors}->[0] =~ "etc/badconf.conf" &&
	    $badconfsms->{errors}->[1] =~ "Line 130 in file " &&
	    $badconfsms->{errors}->[1] =~ "etc/badconf.conf is mal-formed:" &&
	    $badconfsms->{errors}->[1] =~ "bad line" );


registerTest("SmsEngine report config error no ifacegroup");
my $badconfsms = SmsEngine->new( file => "badconf-noifacegroup.conf"); 
testResult( $badconfsms->{error} &&
	    $badconfsms->{errors}->[0] =~ "config group \\[commonsms\\] var ifacegroup is mandatory"
	    );

registerTest("SmsEngine report config error not an iface");
my $badconfsms = SmsEngine->new( file => "badconf-notaniface.conf"); 
testResult( $badconfsms->{error} &&
	    $badconfsms->{errors}->[0] =~ "loadiface config error with ifacegroup" &&
	    $badconfsms->{errors}->[1] =~ "object not_iface not an interface?" );

registerTest("SmsEngine report config error missing iface param");
my $badconfsms = SmsEngine->new( file => "badconf-ifacemissingaccount.conf"); 
testResult( $badconfsms->{error} &&
	    $badconfsms->{errors}->[0] =~ "loadiface config error with ifacegroup" &&
	    $badconfsms->{errors}->[1] =~ "object myo2iface" &&
	    $badconfsms->{errors}->[1] =~ "argument accountgroup is mandatory" );
my $badconfsms = SmsEngine->new( file => "badconf-ifacemissinginout.conf"); 
testResult( $badconfsms->{error} &&
	    $badconfsms->{errors}->[0] =~ "loadiface config error with ifacegroup" &&
	    $badconfsms->{errors}->[1] =~ "object myo2iface" &&
	    $badconfsms->{errors}->[1] =~ "argument inoutgroup is mandatory" );

registerTest("SmsEngine report config error missing iface or account group");
my $badconfsms = SmsEngine->new( file => "badconf-missingaccount.conf"); 
testResult( $badconfsms->{error} &&
	    $badconfsms->{errors}->[0] =~ "loadaccount config error with accountgroup" &&
	    $badconfsms->{errors}->[1] =~ "not an account?" &&
	    $badconfsms->{errors}->[1] =~ "notheretopupclub_o2_account_live" );
my $badconfsms = SmsEngine->new( file => "badconf-missinginout.conf"); 
testResult( $badconfsms->{error} &&
	    $badconfsms->{errors}->[0] =~ "loadiface config error with ifacegroup" &&
	    $badconfsms->{errors}->[1] =~ "inout object notheresmsengine_tucd" &&
	    $badconfsms->{errors}->[1] =~ "argument outaddress is mandatory");
    print Data::Dumper->Dumper($badconfsms);


registerTest("SmsEngine report config error missing iface params");
my $badconfsms = SmsEngine->new( file => "badconf-ifacemissing-outaddress.conf"); 
testResult( $badconfsms->{error} &&
	    $badconfsms->{errors}->[0] =~ "loadiface config error with ifacegroup" &&
	    $badconfsms->{errors}->[1] =~ "inout object smsengine_tucd" &&
	    $badconfsms->{errors}->[1] =~ "argument outaddress is mandatory" );
my $badconfsms = SmsEngine->new( file => "badconf-ifacemissing-outport.conf"); 
testResult( $badconfsms->{error} &&
	    $badconfsms->{errors}->[0] =~ "loadiface config error with ifacegroup" &&
	    $badconfsms->{errors}->[1] =~ "inout object smsengine_tucd" &&
	    $badconfsms->{errors}->[1] =~ "argument outport is mandatory" );
my $badconfsms = SmsEngine->new( file => "badconf-ifacemissing-inaddress.conf"); 
testResult( $badconfsms->{error} &&
	    $badconfsms->{errors}->[0] =~ "loadiface config error with ifacegroup" &&
	    $badconfsms->{errors}->[1] =~ "inout object smsengine_tucd" &&
	    $badconfsms->{errors}->[1] =~ "argument inaddress is mandatory" );
my $badconfsms = SmsEngine->new( file => "badconf-ifacemissing-inport.conf"); 
testResult( $badconfsms->{error} &&
	    $badconfsms->{errors}->[0] =~ "loadiface config error with ifacegroup" &&
	    $badconfsms->{errors}->[1] =~ "inout object smsengine_tucd" &&
	    $badconfsms->{errors}->[1] =~ "argument inport is mandatory" );

my $badconfsms = SmsEngine->new( file => "badconf-ifacemissing-multiple.conf"); 
testResult( $badconfsms->{error} &&
	    $badconfsms->{errors}->[0] =~ "loadiface config error with ifacegroup" &&
 	    $badconfsms->{errors}->[1] =~ "object myo2iface" &&
	    $badconfsms->{errors}->[1] =~ "argument accountgroup is mandatory" &&
 	    #$badconfsms->{errors}->[2] =~ "object myo2iface" &&
	    #$badconfsms->{errors}->[2] =~ "argument inoutgroup is mandatory" &&
	    $badconfsms->{errors}->[2] =~ "inout object smsengine_tucd" &&
	    $badconfsms->{errors}->[2] =~ "argument outaddress is mandatory" &&
	    $badconfsms->{errors}->[3] =~ "argument outport is mandatory" &&
	    $badconfsms->{errors}->[4] =~ "argument inaddress is mandatory" &&
	    $badconfsms->{errors}->[5] =~ "argument inport is mandatory" );

registerTest("SmsEngine report config error missing o2 account params");
my $badconfsms = SmsEngine->new( file => "badconf-o2-missing-smsinport.conf"); 
testResult( $badconfsms->{error} &&
	    $badconfsms->{errors}->[0] =~ "loadaccount config error with accountgroup" &&
 	    $badconfsms->{errors}->[1] =~ "O2ProprietarySms loadaccount" &&
	    $badconfsms->{errors}->[1] =~ "argument smsinport is mandatory" );
my $badconfsms = SmsEngine->new( file => "badconf-o2-missing-smsinaddress.conf"); 
testResult( $badconfsms->{error} &&
	    $badconfsms->{errors}->[0] =~ "loadaccount config error with accountgroup" &&
 	    $badconfsms->{errors}->[1] =~ "O2ProprietarySms loadaccount" &&
	    $badconfsms->{errors}->[1] =~ "argument smsinaddress is mandatory" );

registerTest("SmsEngine report config error missing o2 account file");
my $badconfsms = SmsEngine->new( file => "badconf-o2-missing-certfile.conf"); 
testResult( $badconfsms->{error} &&
	    $badconfsms->{errors}->[0] =~ "loadaccount config error with accountgroup" &&
 	    $badconfsms->{errors}->[1] =~ "O2ProprietarySms loadaccount" &&
	    $badconfsms->{errors}->[1] =~ "Can\'t read file" );
my $badconfsms = SmsEngine->new( file => "badconf-o2-missing-keyfile.conf"); 
testResult( $badconfsms->{error} &&
	    $badconfsms->{errors}->[0] =~ "loadaccount config error with accountgroup" &&
 	    $badconfsms->{errors}->[1] =~ "O2ProprietarySms loadaccount" &&
	    $badconfsms->{errors}->[1] =~ "Can\'t read file" );


#    print Data::Dumper->Dumper($badconfsms);
# TODO: (low priority) full test fake and kannel account loading


# bad libs they die die die :(
#exit(19);

# TODO: emergency message if can't forward to tucd specifically mentioning port and config file
#<enrico> james, seems to be working now, in port was set to 8151, changed it back to 28222 and it worked.
#inport of group [smsengine_tucd]


if (0) { # test annoyingly long timeout
registerTest("o2proprietary network down");
#(in config server exists but nothing on port)
#big nasty 5min? 500sec? =~ 8min? timeout  
my $brokensms = SmsEngine->new( file => "brokenexample.conf", 
			      debug => 0,
			      verbose => 0 );
			      #iface => "o2proprietary_iface", 
			      #account => "topupclub_o2_account" );

my $brokenresult = $brokensms->sendsms($to,$from,$text);
print "sendsms result: $result " .
    $brokenresult->{RequestID} . "," .
    $brokenresult->{ErrorCode} . "," .
    $brokenresult->{ErrorMessage} . "," .
    "\n";

testResult( $brokenresult->{ErrorCode} == 8152 &&
	    $brokenresult->{ErrorMessage} =~ "SmsEngine connect Error" && 
	    $brokenresult->{ErrorMessage} ne "Success"
	    );

}



die "subsequent tests need sendsms test to pass and retrieve good RequestID" 
    if (!defined($result->{RequestID}) || $result->{RequestID} <= 0);

registerTest("SmsEngine o2proprietary querysenddeliverystatus");
my $qsds = $sendsms->{ObjectSms}->querysenddeliverystatus( { RequestID => $result->{RequestID} } );
# might be in any state, might not be finished
print "querysenddeliverystatus result " .
    $qsds->{TOTALNUMBEROFMESSAGES} ."<=".
    $qsds->{MESSAGESSENT} ."+". $qsds->{MESSAGESSKIPPED} .
    " status=" . $qsds->{STATUS} .
    " errorcode=" . $qsds->{ErrorCode} . "\n";
testResult( $qsds->{TOTALNUMBEROFMESSAGES} == 1 &&
	    $qsds->{MESSAGESSENT} + $qsds->{MESSAGESSKIPPED} <= 1 &&
	    $qsds->{STATUS} <= 5 &&
	    $qsds->{ErrorCode} == 0
	    );


# possible that there would be batch id in system with that number ... but not that likely
# we expect ERR API053: Request is not a valid BATCH request
registerTest("SmsEngine o2proprietary querybatchrequest invalid id");
my $qbr = $sendsms->{ObjectSms}->querybatchrequest( { RequestID => $result->{RequestID} } );
print "querybatchrequest result " .
    " status=" . $qbr->{STATUS} .
    " errorcode=" . $qbr->{ErrorCode} . "\n";
testResult( $qbr->{ErrorCode} == 53 );


if (0) {
registerTest("SmsEngine o2proprietary queryaccountrequest");
# API doc has an error
my $qar = $sendsms->{ObjectSms}->queryaccountrequest();
# also currently problem perl libs sending request empty structure not compatible way o2 do it o2 require empty structure of exactly form <struct></struct> perl/XML::Parser SOAP::Lite somewhere does <struct/>
# no wait!  https://sms-open.o2.co.uk:444/apilistener/SMSOA_xmlrpc_listener doesn't understant <struct/>
# but https://216.239.194.94:444/vvtlistener1/SMSOA_xmlrpc_listener does
print "queryaccountrequest result: " .
    $qar->{ErrorCode} . "," .
    $qar->{RequestID} . "," .
    $qar->{REQUEST_ID} . "," .
    $qar->{STATUS} . "," .
    $qar->{PERCENTAGECOMPLETE} . "," .
    $qar->{STARTDATE} . "," .
    $qar->{MESSAGESSKIPPED} . "," .
    $qar->{AccountRequestData}->[0]->{ErrorMessage} .
    "\n";

testResult( $result->{ErrorCode} == 0 &&
	    $qar->{AccountRequestData}->[0]->{ErrorMessage} == "Success"
	    );
}


# possible that there would be batch id in system with that number ... but not that likely
# we expect ERR API053: Request is not a valid BATCH request
my $in10mins = time() + 600;
my $in5mins = time() + 300;

registerTest("SmsEngine o2proprietary sendbatchmessage");
my $badbatchnum = "35386badbatchnum";
my $btoparams = { text => "perl module test SmsEngine o2proprietary sendbatchmessage", 
		  to => [ $badbatchnum ],
		  StartTime => $in10mins };
my $bresult = $sendsms->{ObjectSms}->sendbatchmessage( $btoparams );
print "sendbatchmessage result " .
    " id=" . $bresult->{RequestID} .
    " errorcode=" . $bresult->{ErrorCode} . "\n";
testResult( $bresult->{ErrorCode} == 0 &&
	    $bresult->{RequestID} > 0 );

registerTest("SmsEngine o2proprietary querybatchrequest valid id");
my $qbr = $sendsms->{ObjectSms}->querybatchrequest( { RequestID => $bresult->{RequestID} } );
print "querybatchrequest result " .
    " status=" . $qbr->{STATUS} .
    " errorcode=" . $qbr->{ErrorCode} . "\n";
testResult( $qbr->{ErrorCode} == 0 &&
	    $qbr->{TOTALNUMBEROFMESSAGES} == 1 &&
	    $qbr->{MESSAGESSENT} == 0 && $qbr->{MESSAGESSKIPPED} == 1 &&
	    $qbr->{STATUS} == 5
	    );

registerTest("SmsEngine o2proprietary retrieveskippedrecords");
my $res = $sendsms->{ObjectSms}->retrieveskippedrecords( { RequestID => $bresult->{RequestID} } );
print "retrieveskippedrecords result " .
    " errorcode=" . $res->{ErrorCode} .
    " MESSAGE_ID=" . $res->{MessagesSkipped}->[0]->{MESSAGE_ID} .
    " ERROR_CODE=" . $res->{MessagesSkipped}->[0]->{ERROR_CODE} .
    " MSISDN=" . $res->{MessagesSkipped}->[0]->{MSISDN} .
    " TIMESKIPPED=" . $res->{MessagesSkipped}->[0]->{TIMESKIPPED} .
    " MESSAGE=" . $res->{MessagesSkipped}->[0]->{MESSAGE} . "\n";

testResult( $res->{ErrorCode} == 0 &&
	    $res->{MessagesSkipped}->[0]->{ERROR_CODE} == 202 &&
	    $res->{MessagesSkipped}->[0]->{MSISDN} == $badbatchnum );

# expect ERR API105: Action not supported, submitted request is already FINISHED
registerTest("SmsEngine o2proprietary updatebatchrequest");
my $res = $sendsms->{ObjectSms}->updatebatchrequest( { RequestID => $bresult->{RequestID},
						       StartTime => $in5mins } );
print "updatebatchrequest result " .
    " errorcode=" . $res->{ErrorCode} . "\n";
testResult( $res->{ErrorCode} == 105 );

# expect ERR API105: Action not supported, submitted request is already FINISHED
registerTest("SmsEngine o2proprietary startbatchrequest");
my $res = $sendsms->{ObjectSms}->startbatchrequest( { RequestID => $bresult->{RequestID} } );
print "startbatchrequest result " .
    " errorcode=" . $res->{ErrorCode} . "\n";
testResult( $res->{ErrorCode} == 105 );

# expect ERR API105: Action not supported, submitted request is already FINISHED
registerTest("SmsEngine o2proprietary stopbatchrequest");
my $res = $sendsms->{ObjectSms}->stopbatchrequest( { RequestID => $bresult->{RequestID} } );
print "stopbatchrequest result " .
    " errorcode=" . $res->{ErrorCode} . "\n";
testResult( $res->{ErrorCode} == 105 );

registerTest("SmsEngine o2proprietary removebatchrequest");
my $res = $sendsms->{ObjectSms}->removebatchrequest( { RequestID => $bresult->{RequestID} } );
print "removebatchrequest result " .
    " errorcode=" . $res->{ErrorCode} . "\n";
testResult( $res->{ErrorCode} == 0 );

# purposely not testing stopallbatchrequests as that could cause problems on live systems




registerTest("Can SmsEngine send PCS picture message?");

my $pcsmsg = "3000000002010000481c01".
    "00000000080000000000000000140000000000000000e300000000c60000004118c00000ee000000360d800000ee73dec8490fbf9e3ceedb9c681c0736c360fefb1868dc0736cf78fec318713e0fb6db3cd6db18313f0db6db0cd6731830fb18f6cf7800000023ffc0000000000000e39de0000000000000019040000000".
    "00000003faf00000000000000ffffc0000000000001f77000000000000000063740000000000003f68bc000000000660f71f3e0000000007e0c1f7fb0290001fe381eb63ff8440001027f1ffe39fc2a0000549c80df79b6770001d6aa9cf3df8e770001feea9ecbe771000001fe8880abeb2177000dfe88fffffffff77ff";

my $to;
$to->[0]="badnumfortest";

my $toparams = { text => qq($pcsmsg), 
		 to => $to,
		 Type => "PCS"};

my $result = $sendsms->{ObjectSms}->sendmessage( $toparams );
#$smsengine->{ObjectSms}->printsendresponse($result) if ($verbose);

my $result = $sendsms->sendsms($to,$from,$text);
print "sendsms result: $result " .
    $result->{ErrorCode} . "," .
    $result->{RequestID} . "," .
    $result->{ErrorMessage} . "," .
    $result->{Failed_MSISDN}->[0]->{errorcode} . "," .
    $result->{Failed_MSISDN}->[0]->{num} . "," .
    "\n";

#validate o2proprietary testbed received data
testResult( $result->{ErrorCode} == 0 && 
	    $result->{RequestID} > 0 && 
	    $result->{ErrorMessage} eq "Success" && 
	    $result->{Failed_MSISDN}->[0]->{errorcode} == 202
	    );



#example of test validation
#my $r1 = grepFile($listenerlogfile,
#	     "GET /cgi-bin/sendsms?",
#	     "username=testuser",
#	     "password=password",
#             "to=353861111111",
#             "from=SmsEngine.pm",
#             "text=.*example\%20text\%20message");

