#!/usr/bin/perl -w

# Create a user agent object
use Carp;
use bytes;
use strict;
use SmsEngine::SmsEngine;

#open(LOG, ">>xiamq.log") || croak "can't open log: $!";

my $messageid;
my $ifaceid = 0;
my $configfile ="smsengine.topupclub.xiam.conf";

my $usage = <<END;

usage: $0 [<options>] -i messageid

options:  [-d] [-v] 
          [-c configfile]
          [-f xiamifaceidindb] 
          [-q]

 -d debug
 -v verbose
 -q keep querying until status is finished

e.g. $0 -v -i brbqJzi
e.g. mysql -u root smsengine -e "select messageid from msg_status"
e.g. mysql -u root smsengine -e "select * from msg_status where messageid = '+bfgr'"

END

my ($debug, $verbose, $query);

while (defined($ARGV[0])) {

    if ($ARGV[0] =~ "^-d") {
	$debug = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-v") {
	$verbose = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-q") {
	$query = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-c") {
	shift(@ARGV);
	$configfile = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-f") {
	shift(@ARGV);
	$ifaceid = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-i") {
	shift(@ARGV);
	$messageid = shift(@ARGV);
    } else {
	croak "unknown param " . shift(@ARGV) . "\n".$usage;
    }

}

print STDERR "-i $messageid -c $configfile -f $ifaceid $debug $verbose\n"
    if (defined($debug));

croak "no message id to query?\n$usage" if (!defined $messageid);

#start xiamsms
my $smsengine = SmsEngine->new( file => $configfile,
			      debug => $debug,
			      verbose => 0 );
			      #iface => "o2xiam_iface", 
			      #account => "topupclub_o2irl_xiam_account_live" );

$smsengine->{ObjectSms}->showconfig() if (defined($debug));

my $qres = $smsengine->{ObjectSms}->querysenddeliverystatus( 
	       { RequestID => $messageid } );

$smsengine->{ObjectSms}->printqueryresponse($qres) if ($verbose);

if ($query) {

    while ($qres->{TOTALNUMBEROFMESSAGES} > 
	   $qres->{MESSAGESSENT} + $qres->{MESSAGESSKIPPED} &&
	   $qres->{ErrorCode} == 0) {
	print "$qres->{TOTALNUMBEROFMESSAGES} < $qres->{MESSAGESSENT} + $qres->{MESSAGESSKIPPED}?\n";
	print "Status:".$qres->{STATUS}." ".$qres->{STATUS}."\n";

	# retry
	print "Waiting for message to clear queues\n";
	sleep(5);
	$qres = $smsengine->{ObjectSms}->querysenddeliverystatus( 
		     { RequestID => $messageid } );
	$smsengine->{ObjectSms}->printqueryresponse($qres) if ($verbose);
    }

    #$qres->{smsengine_status},
    #$qres->{smsengine_status_info},

    print "Status:".$qres->{STATUS}." ".$qres->{STATUS}."\n";
    print "Beautiful. All messages finished with.\n";	
    
}


=head1 NAME

o2xiamquery.pl - test script query smsengine msg status stored in db for o2 xiam sms/xml iface

=head1 SYNOPSIS

$usage

=head1 NOTES

=cut
