#!/usr/bin/perl

use SmsEngine::SmsEngine;
use Carp;
use bytes;
use strict;

open(LOG, ">>xiamout.log") || croak "can't open log: $!";

my $configfile ="smsengine.topupclub.xiam.conf";

# defaults for all TODO: message or to should not be default after test
# defaults now set depending on message interface being used $smsengine or xml direct
my $messageid; # = make_msg_id();
my $from; # = 51101; # from should be 51101, 50308 or 53125 as appropriate for TUCDEV, TUCTEST or TUC
my $to;
my $msg;
#my $msg = "TUCDEV Test Doolin SmsEngine XIAM SMS";
my $xiamProviderID; # = "129";

my $usage = <<END;

send message to xiam OR send xml directly to send message or simulate receipt of messages

send message: $0 [<options>] "message" tonum
send xml: $0 [<options>] [-rxml <xml>]

usage: $0 [<options>] <command>

options:  [-d] [-v] 
    [-c <configfile>]
    [-n]
    [-f <from>]

 -d debug
 -v verbose
 -n do nothing

command:  ( <msg> or -m <filewithmessage>
            to [to] [to] ... or -t <filewithlistofmobilenumbers>
END
;

my $devusage = <<END;
development options:
    [-i <messageid>] [-x <XiamProviderID>] 
    [-u <url>]
    [-rxml <reqxml>] or [-r <id>]


 -q query message (not implemented (not possible xiam))

 -r <id> send report request for <id> (for simulation/testing only)
 -rxml <reqxml> send xiam sms/xml directly for testing
    xiam headers and <xiamSMS></xiamSMS> are inserted by this script around the xml
  -rdeldoc -rdelact -rxxx

[msg | -m msgfile]   send sms message out

e.g. $0 -d -v "Hello from TUC" +353867456789 
e.g. $0 -d -v -i brbqJzi "Hello from TUC" -f +353867456789 +353867456789 badnum1 badnum2
e.g. $0 -d -v -x 128 "Hello from TUC" +353867456789
e.g. $0 -d -v -x 129 "Hello from TUCDEV" +353867456789
e.g. $0 -d -v -x 130 "Hello from TUCTEST" +353867456789
e.g. $0 -d -v "Hello from TUC" -t filewithlistofmobilenumbers (up to 100)
e.g. $0 -d -v "Hello from TUCTEST" +353867456789 -u 'http://193.95.160.218:8080/smsxml/collector'

e.g. $0 -rdelact 1234 "TUC balance " -f +353867456789 50308 -u http://localhost:8101/sms/xiam/in   
   # simulates delivery request id 1234
e.g. $0 -d -r 1234 -u http://localhost:8101/sms/xiam/in   # simulates report request id 1234
e.g. $0 -d -rxml "<xiamSMS></xiamSMS>" -u http://localhost:8101/sms/xiam/in   # bad
e.g. $0 -d -rxml "<deliverRequest id=xir-moo><to>to</to><from>from</from></deliverRequest>" # incomplete
END
;

my $url; # = 'http://193.95.160.218:8080/smsxml/collector';
my ($debug, $verbose);
my ($msgfile, $tofile, $type, $req, $reqxml, $query, $nothing);

my $i=0;
while (defined($ARGV[0])) {

    if ($ARGV[0] =~ "^-d") {
	$debug = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-v") {
	$verbose = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-n") {
	$nothing = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-c") {
	shift(@ARGV);
	$configfile = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-r") {
	$req = shift(@ARGV);
	$reqxml = shift(@ARGV);
        #croak "cannot do this with xiam TODO: smsengine must store deliver reports and send on when tuc requests them Hmmmm.";        
        print "cannot send this to xiam;";
        print "use for simulating incoming requests\n";

    } elsif ($ARGV[0] =~ "^-m") {
	shift(@ARGV);
	$msgfile = shift(@ARGV);
        croak "not done yet";
    } elsif ($ARGV[0] =~ "^-q") {
	$query = shift(@ARGV);
        croak "not done yet";
    } elsif ($ARGV[0] =~ "^-i") {
	shift(@ARGV);
	$messageid = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-x") {
	shift(@ARGV);
	$xiamProviderID = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-t") {
	shift(@ARGV);
	$tofile = shift(@ARGV);
        croak "not done yet";
    } elsif ($ARGV[0] =~ "^-u") {
	shift(@ARGV);
	$url = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-f") {
	shift(@ARGV);
	$from = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-\\\w") {
	croak "unknown param: " . shift(@ARGV) . "\n" . $usage;
    } elsif (!defined($msg) && !defined($msgfile)) {
	$msg = shift(@ARGV);
    } else { #if (!defined($to)) {
	$to->[$i++] = shift(@ARGV);
    }# else {
    #die "unknown param " . shift(@ARGV) . "\n".$usage;
    #}

}

if ($msgfile) {
    $msg = "";
    open(MSGFILE, "<$msgfile") || die("ERROR: Couldn't open $msgfile $!");
    while (<MSGFILE>) {
	$msg.=$_;
    }   
    close(MSGFILE);	
}

if (defined($tofile)) {
    open(TOFILE, "<$tofile") || die("ERROR: Couldn't open $tofile $!");
    #$i=0;
    while (<TOFILE>) {
	next if (/^\#/);  
	chomp($_);
	$to->[$i++]=qq($_);
	die "ERROR: o2 SMS push api specifies maximum of 100 messages for sendmessage\nUse batch sms scripts\n" if ($i>100);
    }   
    close(TOFILE);	
}


print STDERR "-m $msgfile -q $query -i $messageid -x $xiamProviderID -t $tofile" .
    " -d $debug -v $verbose -r $req $reqxml" .
    " -u $url -f $from text $msg to $to->[0] i $i\n" if (defined($debug));

$usage .= $devusage if (defined $verbose);
croak "no text to send?\n$usage" if (!defined($msg) &&
				     !defined($req) || ($req =~ /xml/ && !defined($reqxml)));

croak "no number to send to?\n$usage" if (!defined($to->[0]) &&
				     !defined($req) || ($req =~ /xml/ && !defined($reqxml)));

warn "from should be mandatory (though *mutter* *mutter*)\n".
    "from should be 51101, 50308 or 53125 as appropriate for TUCDEV, TUCTEST or TUC ... I think.\n".
    "don't worry. will be set in smsengine (see configfile $configfile).\n"
    if (!defined($from) || ($req =~ /xml/ && !defined($reqxml)));

use Data::Dumper; print Dumper($to) if (defined($debug));

#start xiamsms
my $smsengine = SmsEngine->new( file => $configfile,
				debug => $debug,
				verbose => 0 );
                              #iface => "o2xiam_iface", 
			      #account => "topupclub_o2irl_xiam_account_live" );

die ("wauh. failed to instantiate xiam smsengine :(\n$!") if (!defined($smsengine));
$smsengine->{ObjectSms}->showconfig() if (defined($debug));

print Dumper($url);
print Dumper($reqxml);
print Dumper($xiamProviderID);

# if we are not sending XML direct (sensible option)
if (!defined($url) && !defined($xiamProviderID) && !defined($reqxml)) {


    #TODO:  check sendbatchmessage or sendsimple message ?
    #my $res = $smsengine->{ObjectSms}->_xiamsmsxmlsend($xiamSMS);

    my $l = length $msg;
    if ($l > 143) {
	print STDERR "Warning: message length is $l, don't know what xiam does with that\n";
    }

    my $toparams = { text => qq($msg), 
		     to => $to };
    $toparams->{from} = $from if (defined $from);
    $toparams->{messageid} = $messageid if (defined $messageid);
    # smsengine sets messageid and from if they are not set
    #$messageid = make_msg_id() if (!defined($messageid));
    #$from = 53125 if (!defined($from)); 

    print LOG "o2xiamsend.pl call sendmessage $to->[0] $msg\n"; 
    print "o2xiamsend.pl call sendmessage $to->[0] $msg\n"; 

    if (!defined($nothing)) {
	my $res = $smsengine->{ObjectSms}->sendmessage( $toparams );
	use Data::Dumper; print Dumper($res) if ($debug);
	$smsengine->{ObjectSms}->printsendresponse($res) if ($verbose);
    } else {
 	warn "doing nothing. -n flag set.";
    }

} else {

    warn "setting url or providerid or reqxml bypasses smsengine XiamSms.pm a bit. Use for testing only.\n";
    # $url and providerid so we must build xml directly (this is us in test mode)
    #$messageid = make_msg_id() if (!defined($messageid));
    $from = 53125 if (!defined($from)); 
    # from should be 51101, 50308 or 53125 as appropriate for TUCDEV, TUCTEST or TUC

    #$xiamProviderID = 129 if (!defined($xiamProviderID));
    #$url = 'http://193.95.160.218:8080/smsxml/collector' if (!defined($url));

    my $xiamSMS;
    if (defined($reqxml)) {

	if ($req =~ /xml/) {
	    $xiamSMS = $reqxml;
	} else {
	    # simulate requests from xiam
	    $xiamSMS = makeTestXml();
	}

    } else {

	# build normal requests. Hmmmm. $smsengine->thing should do this for us.
	print "messageid: $messageid\n" if (defined($debug));

	$xiamSMS = "<submitRequest id=\"$messageid\"><from>$from</from>\n";
	    
	foreach my $j (@$to) { 
	    $xiamSMS .= "      <to>".$j."</to>\n";
	}

	$xiamSMS .= "<content type=\"text\">$msg</content>" .
	    "<requestDeliveryReport value=\"yes\" />" .
	    "</submitRequest>";
    }

    print LOG "o2xiamsend.pl call _xiamsmsxmlsend\n"; 
    print "o2xiamsend.pl call _xiamsmsxmlsend\n"; 

    print LOG $xiamSMS . "\n"; 
    print $xiamSMS . "\n"
	if (defined($debug) || defined($verbose));

    print "URL: ", $url, "\n"
	if (defined($debug) || defined($verbose));
    print LOG $url, "\n";

    print "X-XIAM-Provider-ID: ", $xiamProviderID, "\n"
	if (defined($debug) || defined($verbose));
    print LOG "X-XIAM-Provider-ID: ", $xiamProviderID, "\n";

    if (!defined($nothing)) {
	my $res = $smsengine->{ObjectSms}->_xiamsmsxmlsend ( $xiamSMS, $messageid, $xiamProviderID, $url );
	use Data::Dumper; print Dumper($res) if ($debug);
	$smsengine->{ObjectSms}->printsendresponse($res) if ($verbose);
    } else {
 	warn "doing nothing. -n flag set.";
    }

}


sub make_msg_id {
    #my $self = shift;
    # use MD5 as the id maybe?
    my $ts = time();
    my $data = $ts;
    use Digest::MD5  qw(md5 md5_hex md5_base64);
    my $digest = md5_base64($data);
    #my $digest = "doolin".$ts;
    print "id md5 $digest\n" if ($debug);

    return $digest;
}

sub makeTestXml {
    my $xiamSMS;
    # simulate requests from xiam

    #croak "cannot do this with xiam TODO: ".
    #    "smsengine must store deliver reports and send on when tuc requests them Hmmmm.";        

    if ($req =~ /xml/) {
	$xiamSMS = $reqxml;

    } elsif ($req =~ /deldoc/) { # as in xiam sms/xml dev guide jun 2004 (not in reality)

	$xiamSMS = "  <deliverRequest id=\"xir-$reqxml\">\n";
	$xiamSMS .= <<EOT
      <from>$from</from>
      <to>$to->[0]</to>
      <content type="text">$msg</content>
      <smscTimestamp type="Absolute">
        <year>2003</year>
        <month>03</month>
      </smscTimestamp>
      <carrierType value="SMPP" />
      <receivedOnPath value="42" />
      <xirMessageID value="$reqxml" />
EOT
;
	$xiamSMS .= "  </deliverRequest>";

    } elsif ($req =~ /delact/) { # as actually received dec 02 2004

	$xiamSMS = <<EOT
<deliverRequest id="$reqxml">
<receivedOnGroup value="SMPP-O2"/>
<to npi="Unknown">$to->[0]</to>
<udh value=""/>
<carrierType value="SMPP"/>
<sendOnPath value="103"/>
<from>$from</from>
<content type="text">$msg</content>
<xirMessageID value="xir-o2xiamsend-dr"/>
</deliverRequest>
EOT
;

    } else {

	$xiamSMS = <<EOT
<from>$from</from>
<to npi="Unknown">$to->[0]</to>
<extraRef value="moo"/>
<reportRequest id="$reqxml">
<carrierType value="SMPP"/>
<smscStatusTime value="0"/>
<receivedOnGroup value="SMPP-O2"/>
<receivedOnPath value="103"/>
<smscReference value="urp"/>
<smscStatus value="0"/>
<smscStatusText value="DELIVERED"/>
<smscTimeStamp value="0"/>
<smscSpecificStatus value="5"/>
<xirMessageID value="xir-o2xiamsend-rr"/>
</reportRequest>
EOT
;
	$xiamSMS = "  <reportRequest id=\"$reqxml\"></reportRequest>";
    }

    return $xiamSMS
}

=head1 NAME

o2xiamsend.pl - test script send sms o2 xiam sms/xml iface

=head1 SYNOPSIS

$usage

=head1 NOTES

  sms out to xiam
  IPs: 193.95.160.218, 193.95.160.219, 193.95.160.223
  Port: 8080
  URL: /smsxml/* nope /smsxml/collector
  
  1. All are production. 223 is a virtual IP that resolves to either 218 or
     219. For the moment, please go straight to 218. We will advise in future to
     move to 223.
  3. As these are live systems, we just have to pick a shortcode and set up
     your keywords on that code. Shay: I'll use 53125, unless you have any
     objections. keywords TUC, TUCTEST, TUCDEV
  7. An ID identifies an account, which has one URL for MO traffic, so if you
     want MOs to go to 3 different URLs, depending on keywords, I will have to
     set up 3 accounts for you. You can then choose to send from all 3 or could
     do all MT through one account only.

  IDs are: 
    TUC: 128
    TUCDEV: 129 (routed to TUCTEST URL for the moment)
    TUCTEST: 130

  figuring out the url:

   #my $req = HTTP::Request->new(POST => 'http://localhost:8101/smsxml');
   #my $req = HTTP::Request->new(POST => 'http://localhost:8101/sms/xiam/in');

   #my $req = HTTP::Request->new(POST => 'http://193.95.160.218:8080/smsxml/');
   #my $req = HTTP::Request->new(POST => 'http://193.95.160.219:8080/smsxml');
   #my $req = HTTP::Request->new(POST => 'http://193.95.160.223:8080/smsxml');
   # nope my $req = HTTP::Request->new(POST => 'http://193.95.160.218:8080/smsxml/*');
   #my $req = HTTP::Request->new(POST => 'http://193.95.160.218:8080/smsxml/smshttp/collector');
   #my $req = HTTP::Request->new(POST => 'http://193.95.160.218:8080/smshttp/collector');
   #my $req = HTTP::Request->new(POST => 'http://193.95.160.218:8080/smsxml/collector');
   #404 /smsxml/smshttp/collector
   #my $req = HTTP::Request->new(POST => 'http://193.95.160.218:8080/smsxml/test.jsp');
   #my $req = HTTP::Request->new(POST => 'http://193.95.160.218:8080/smsxml/index.jsp');
   #my $req = HTTP::Request->new(POST => 'http://193.95.160.218:8080/smsxml/smshttp-test.jar');
 
=head1 use to simulate incoming msgs

 test/o2xiamsend.pl -v -u http://localhost:8101/sms/xiam/in -rxml "<deliverRequest id=\"1907500\">
  <receivedOnGroup value=\"SMPP-O2\"/>
  <to npi=\"Unknown\">53125</to>
  <udh value=\"\"/>
  <carrierType value=\"SMPP\"/>
  <sendOnPath value=\"103\"/>
  <from>353861953134</from>
  <content type=\"text\">TUCTEST</content>
  <xirMessageID value=\"1907500\"/>
  </deliverRequest>"

=head1 ngeh?

  test/o2xiamsend.pl -d -v -i jamesto53125TUCTEST "TUCTEST" -f 53125 53125 -x 129
  test/o2xiamsend.pl -d -v -i jamesto53125TUCTEST "TUCTEST" -f 53125 +353863833599 -x 129
  test/o2xiamsend.pl -d -v -i jamesto53125TUCTEST "TUCTEST" -f 50308 +353863833342 -x 129
  test/o2xiamsend.pl -d -v -i jamesto53125TUCTEST "TUCTEST" -f 51101 +353863832032 -x 129
  test/o2xiamsend.pl -d -v -i jamesto53125TUCDEV "TUCDEV" -f 53125 +353863833599 -x 129

=cut
