#!/usr/bin/perl -w
#
#  o2xmlrpcbatchout.pl
#    - xml-rpc https interface to o2 sms for sending sms
#
#  Copyright (C) 2004 Doolin Technologies
#
#  $Id: o2xmlrpcbatchout.pl,v 1.11 2004/12/17 14:29:18 jamesc Exp $
#

use strict;
use bytes;

use SmsEngine::SmsEngine;

my $configfile = "smsengine.topupclub.conf";

my $usage = <<END;
usage: $0 [-d] [-v] [-q] [-c configfile] [-C mtcode] [-m msg] (to | -t filewithlistofmobilenumbers )
usage: $0 [-d] [-v] [-r reqid] [requestidmethod]

 -m msg    message text to send out
[-q]       send query on the message id to get message status
[-r reqid] send query on other specified request id
requestidmethod  default is querybatchrequest, (o2 xmlrpc only, not xiam)
           requestidmethod also can be:
           startbatchrequest stopbatchrequest stopallbatchrequests
           updatebatchrequest removebatchrequest retrieveskippedrecords

e.g. $0 -d -v -q -m "Hello from TUC" -t filewithlistofmobilenumbers (up to 25000)
e.g. $0 -d -v -r 165234
e.g. $0 -d -v -r  ( a cheat that does queryaccountrequest )

END
;

my ($debug, $verbose);
my ($msg, $query, $reqid, $tofile, $to, $mtcode, $method);

    my $i=0;
while (defined($ARGV[0])) {

    if ($ARGV[0] =~ "^-d") {
	$debug = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-v") {
	$verbose = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-m") {
	shift(@ARGV);
	$msg = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-q") {
	$query = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-r") {
	shift(@ARGV);
	$reqid = shift(@ARGV);
        print "reqid $reqid argv0 $ARGV[0]\n" if ($debug);
	$method = shift(@ARGV)
	    if ($ARGV[0] !~ "^-");
    } elsif ($ARGV[0] =~ "^-t") {
	shift(@ARGV);
	$tofile = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-C") {
	shift(@ARGV);
	$mtcode = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-c") {
	shift(@ARGV);
	$configfile = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-") {
	die "unknown param " . shift(@ARGV) . "\n".$usage;
    } else {
	$to->[$i++] = shift(@ARGV);
    }

}

# new SmsEngine container (config)
# load config will die if can't find config file
my $smsengine = SmsEngine->new( file => $configfile);
if ($smsengine->{error}) {
    foreach my $err(@{$smsengine->{errors}}) {
	print "$err\n";
    }
    die "cannot instantiate SmsEngine :( installation or config problem.";
}

#my $result = $smsengine->sendsms($to,$from,$text);
#print "sendsms result: $result\n";

#my $verbose = $smsengine->getparam("verbose");
#my $debug = $smsengine->getparam("debug");
my $basedir = $smsengine->getparam("basedir");

# a sanity check, should never happen
# valid o2 xmlrpc only, not xiam
#for my $c (qw(https_xmlrpc_url https_cert_file 
#	      https_pkcs12_file https_pkcs12_password
#	      )) {
#    die "SmsEngine module failed to load param $c from config?" if (!$smsengine->getparam("$c"));
#}



if ($debug) {
    #$xmlrpctrace = "transport dispatch result parameters headers objects method fault freeform trace debug";
    SOAP::Trace->import(trace => 'debug');
}

if($msg) {

    if (!$tofile) {
	#$to->[$i++] = shift(@ARGV);
	die "ERROR: no destination to which to send sms?\n$usage" if (!$to->[0]);
	my $nexttoarg;
	while ($nexttoarg = shift(@ARGV)){
	    $to->[$i++] = $nexttoarg; 
	}
    } else {
	open(TOFILE, "<$tofile") || die("ERROR: Couldn't open $tofile $!");
	while (<TOFILE>) {
	    next if (/^\#/);  
	    chomp($_);
	    $to->[$i++]=qq($_);
	    die "ERROR: o2 SMS push api specifies maximum of 25000 messages for sendmessage\nUse batch sms scripts\n" if ($i>25000);
	}   
	close(TOFILE);
    }

    my $toparams = { text => qq($msg), 
		     to => $to };

    if (defined $mtcode) {
	$toparams->{MTCode} = $mtcode;
    }

    my $res = $smsengine->{ObjectSms}->sendbatchmessage( $toparams );
    $smsengine->{ObjectSms}->printsendresponse($res) if ($verbose);

    # TODO: configurable logdir/filename
    my $logfile = $basedir."logs/smsengine.batchout.log";
    $smsengine->{ObjectSms}->logsendresponse($logfile,$res);

    if ($query) {

	my @reqStatus = ( "?", "Queued", "Processing", "?", "Failed", "Finished" );

        my $qres = $smsengine->{ObjectSms}->querybatchrequest( 
            { RequestID => $res->{RequestID} } );
        $smsengine->{ObjectSms}->printqueryresponse($qres) if ($verbose);

	##### TODO: all this response parsing should be encapsulated in O2ProprietarySMS
	
	##### TODO: the fault/wait/done queueing also could be encapsulated in O2ProprietarySMS

	# TODO: timeout and query retry wait until messages sent+skipped=total

	while ($qres->{TOTALNUMBEROFMESSAGES} > 
	       $qres->{MESSAGESSENT} + $qres->{MESSAGESSKIPPED}) {
	    print "$qres->{TOTALNUMBEROFMESSAGES} < $qres->{MESSAGESSENT} + $qres->{MESSAGESSKIPPED}?\n";
	    print "Status:".$qres->{STATUS}." ".$reqStatus[$qres->{STATUS}]."\n";

	    # retry
	    print "Waiting for message to clear queues\n";
	    sleep(5);
	    $qres = $smsengine->{ObjectSms}->querybatchrequest( 
		     { RequestID => $res->{RequestID} } );
	    $smsengine->{ObjectSms}->printqueryresponse($qres) if ($verbose);
	}

	print "Status:".$qres->{STATUS}." ".$reqStatus[$qres->{STATUS}]."\n";
	print "Beautiful. All messages finished with.\n";	

    }
    
    # TODO: message receipts

} elsif ($reqid) {

    my $method = shift(@ARGV);
    if (defined($method)) { 
	if ($method =~ "start") {
	    my $res = $smsengine->{ObjectSms}->startbatchrequest( { RequestID => $reqid } );
	    print "result code: " . $res->{ErrorCode} . "\n"if ($verbose);
	} elsif ($method =~ "stop") {
	    my $res = $smsengine->{ObjectSms}->stopbatchrequest( { RequestID => $reqid } );
	    print "result code: " . $res->{ErrorCode} . "\n"if ($verbose);
	} elsif ($method eq "stopallbatchrequests") {
	    #woah!
	    warn "WARNING: This will Stop all active requests on LIVE system as well as test syhstems!!!!\n";
	    print "Confirm with yes:";
	    my $confirm;
	    read STDIN,$confirm,3;
	    if ($confirm =~ /yes/i){
		my $res = $smsengine->{ObjectSms}->stopallbatchrequests();
		print "result code: " . $res->{ErrorCode} . "\n";
	    } else { 
		print "okay.\n ... wuss!\n";
	    }
	} elsif ($method =~ "update") {
	    my $starttime = shift(@ARGV);
	    if (!defined($starttime)){
		$starttime = time(); 
		print "setting batch $reqid start time to $starttime (now!)\n";
	    }
	    my $res = $smsengine->{ObjectSms}->updatebatchrequest( { RequestID => $reqid, StartTime => $starttime } );
	    print "result code: " . $res->{ErrorCode} . "\n" if ($verbose);
	} elsif ($method =~ "remove") {
	    my $res = $smsengine->{ObjectSms}->removebatchrequest( { RequestID => $reqid } );
	    print "result code: " . $res->{ErrorCode} . "\n" if ($verbose);
	} elsif ($method =~ "retrieve") {
	    my $res = $smsengine->{ObjectSms}->retrieveskippedrecords( { RequestID => $reqid } );
	    print "result code: " . $res->{ErrorCode} . "\n" if ($verbose);
	} else {
	    print "I don\'t do $method really ...\n";
	    my $qres = $smsengine->{ObjectSms}->querybatchrequest( { RequestID => $reqid } );
	    $smsengine->{ObjectSms}->printqueryresponse($qres) if ($verbose);
	}
    } else {
	my $qres = $smsengine->{ObjectSms}->querybatchrequest( { RequestID => $reqid } );
	$smsengine->{ObjectSms}->printqueryresponse($qres); # if ($verbose);
    }

} else {
    warn "forgot to specify -m <message>?";
    my $params = { };
    my $qres = $smsengine->{ObjectSms}->queryaccountrequest( $params );
    $smsengine->{ObjectSms}->printqueryresponse($qres); # if ($verbose);

}


exit(0);

# test stuff below

#################################################################


#sendbatchmessage
#querybatchrequest
#queryaccountrequest


=head1 NAME

o2xmlrpcbatchout.pl - xml-rpc https interface to o2 sms for sending sms in batches

=head1 SYNOPSIS

perl o2xmlrpcbatchout.pl

=head1 DESCRIPTION

 uses SmsEngine.pm and O2ProprietarySms.pm
 sends xmlrpc call $smsengine->{ObjectSms}->sendbatchmessage to o2 test server
 link https with certs & keys
 receives reply and parses out data

 use -q query option when sending to immediately follow up status of messages until all are sent

 use -r <reqid> option to query status of batches after they have been initiated
 Note the <reqid> when initiating the batch.

 use -r <reqid> <command> to control the batch 
  (command: start/stop/update/remove/retrieve/query/stopallbatchrequests)

 O2 SMS push api specifies maximum of 25000 messages for sendbatchmessage.
 In practice O2 put a limit of 500 messages max on accounts.
 This quota can be increased by talking to O2.
 Split destinations and manage transmission in batches of batches if 
  batches larger than max accepted are to be sent.

 TODO: allow ValidateOnly option, other options

 TODO: allow message customisation

 TODO: Originator

 TODO: logs of faults and messages in queues

=head1 COPYRIGHT

Copyright (C) 2004 Doolin Technologies

=cut
