#!/usr/bin/perl -w
#
#  o2xmlrpcout.pl
#    - xml-rpc https interface to o2 sms for sending sms
#
#  Copyright (C) 2004 Doolin Technologies
#
#  $Id: o2xmlrpcout.pl,v 1.16 2004/12/17 14:29:18 jamesc Exp $
#

use strict;
use bytes;

use SmsEngine::SmsEngine;

my $configfile = "smsengine.topupclub.conf";

#usage: $0 [-d] [-v] [-q] [-c configfile] [-C mtcode] 
#          [-Type NOR|FLA|RIN|LOG|CLO|PCS|MPP|CON|MPN|RTS|MPR|BIN|MPB] 
#          -m msg (to | -t filewithlistofmobilenumbers )

my $usage = <<END;
usage: $0 [-d] [-v] [-q] 
          [-c configfile] 
          [-C mtcode] (o2 xmlrpc only) 
          [-Type NOR|FLA|RIN|LOG|CLO|PCS|BIN] (o2 xmlrpc only)
          (msg | -m filewithmessage)
          (to [to] [to] ... | -t filewithlistofmobilenumbers )

usage: $0 [-d] [-v] [-r reqid]

[msg | -m msgfile]   send sms message out
[-q]       send query on the message id to get message status
[-r reqid] send query on other specified request id

e.g. $0 -d -v -q "Hello from TUC" 353867456789 
e.g. $0 -d -v -q "Hello from TUC" 353867456789 badnum1 badnum2
e.g. $0 -d -v -q "Hello from TUC" -t filewithlistofmobilenumbers (up to 100 o2 xmlrpc)
e.g. $0 -m data/o2smsapiexample.pcs -d -q -Type PCS badnum (o2 xmlrpc only)

e.g. $0 -d -v -r 165234

END

my ($verbose, $debug);
my ($msgfile, $msg, $query, $reqid, $tofile, $to, $mtcode, $type);

my $i=0;
while (defined($ARGV[0])) {

    if ($ARGV[0] =~ "^-d") {
	$debug = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-v") {
	$verbose = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-m") {
	shift(@ARGV);
	$msgfile = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-q") {
	$query = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-r") {
	shift(@ARGV);
	$reqid = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-t") {
	shift(@ARGV);
	$tofile = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-C") {
	shift(@ARGV);
	$mtcode = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-c") {
	shift(@ARGV);
	$configfile = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-Type") {
	shift(@ARGV);
	$type = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-") {
	die "unknown param " . shift(@ARGV) . "\n".$usage;
    } elsif (!defined($msg) && !defined($msgfile)) {
	$msg = shift(@ARGV);
    } else {
	$to->[$i++] = shift(@ARGV);
    }
}


# new SmsEngine container (config)
# load config will die if can't find config file
my $smsengine = SmsEngine->new( file => $configfile);
if ($smsengine->{error}) {
    foreach my $err(@{$smsengine->{errors}}) {
	print "$err\n";
    }
    die "cannot instantiate SmsEngine :( installation or config problem.";
}

#my $result = $smsengine->sendsms($to,$from,$text);
#print "sendsms result: $result\n";

#$verbose = $smsengine->getparam("verbose");
#$debug = $smsengine->getparam("debug");
my $basedir = $smsengine->getparam("basedir");

# a sanity check, should never happen
# only valid for o2 uk proprietary, not o2 xiam
#for my $c (qw(https_xmlrpc_url https_cert_file 
#	      https_pkcs12_file https_pkcs12_password
#	      )) {
#    die "SmsEngine module failed to load param $c from config?" 
#	if (!$smsengine->getparam("$c"));
#}


if ($msgfile) {
    $msg = "";
    open(MSGFILE, "<$msgfile") || die("ERROR: Couldn't open $msgfile $!");
    while (<MSGFILE>) {
	$msg.=$_;
    }   
    close(MSGFILE);	
}

if($msg) {

    if (defined($tofile)) {
	open(TOFILE, "<$tofile") || die("ERROR: Couldn't open $tofile $!");
        #$i=0;
	while (<TOFILE>) {
	    next if (/^\#/);  
	    chomp($_);
	    $to->[$i++]=qq($_);
	    die "ERROR: o2 SMS push api specifies maximum of 100 messages for sendmessage\nUse batch sms scripts\n" if ($i>100);
	}   
	close(TOFILE);	
    }

    $tofile = "" if (!defined($tofile));
    die "ERROR: no destination to which to send sms?\n$usage" if (!defined($to->[0]));
    print "to $i numbers $tofile first num is " . $to->[0]."\n" if ($verbose);

    # multi-part message control is inside O2ProprieatarySMS
    # but extra info here would be nice
    # TODO: message prepare (make data structure ready for message send)
    #       also allow check info like message parts
    # TODO if $type multi-part ctrl direct
    my $l = length $msg;
    if ((!defined($type) || $type eq "NOR") && $l > 143) {
	my $parts = $l / 100;
	print STDERR "Warning: message length is $l, message will be split into $parts parts\n";
	print STDERR "Warning: I do not know how o2 bill these messages.";
	my $input;
	read STDIN,$input,1;

    }

    my $toparams = { text => qq($msg), 
		     #to => [ qq($to) ] } );
		     to => $to };

    if (defined $mtcode) {
	$toparams->{MTCode} = $mtcode;
    }
    if (defined $type) {
	$toparams->{Type} = $type;
    }

    my $res = $smsengine->{ObjectSms}->sendmessage( $toparams );

    $smsengine->{ObjectSms}->printsendresponse($res) if ($verbose);

    # TODO: configurable logdir/filename
    my $logfile = $basedir."logs/smsengine.cmdout.log";
    $smsengine->{ObjectSms}->logsendresponse($logfile,$res);

    if ($query) {

	my @reqStatus = ( "?", "Queued", "Processing", "?", "Failed", "Finished" );

        my $qres = $smsengine->{ObjectSms}->querysenddeliverystatus( 
            { RequestID => $res->{RequestID} } );
        $smsengine->{ObjectSms}->printqueryresponse($qres) if ($verbose);

	##### TODO: all this response parsing should be encapsulated in O2ProprietarySMS
	
	##### TODO: the fault/wait/done queueing also could be encapsulated in O2ProprietarySMS

	# TODO: timeout and query retry wait until messages sent+skipped=total

	while ($qres->{TOTALNUMBEROFMESSAGES} > 
	       $qres->{MESSAGESSENT} + $qres->{MESSAGESSKIPPED}) {
	    print "$qres->{TOTALNUMBEROFMESSAGES} < $qres->{MESSAGESSENT} + $qres->{MESSAGESSKIPPED}?\n";
	    print "Status:".$qres->{STATUS}." ".$reqStatus[$qres->{STATUS}]."\n";

	    # retry
	    print "Waiting for message to clear queues\n";
	    sleep(5);
	    $qres = $smsengine->{ObjectSms}->querysenddeliverystatus( 
		     { RequestID => $res->{RequestID} } );
	    $smsengine->{ObjectSms}->printqueryresponse($qres) if ($verbose);
	}

	print "Status:".$qres->{STATUS}." ".$reqStatus[$qres->{STATUS}]."\n";
	print "Beautiful. All messages finished with.\n";	

    }
    
    # TODO: message receipts

} elsif ($reqid) {
    print "querysenddeliverystatus $reqid\n" if ($debug);
    my $qres = $smsengine->{ObjectSms}->querysenddeliverystatus( 
        { RequestID => $reqid } );
    use Data::Dumper; print Dumper($qres) if ($debug);
    if (!defined($qres) || defined($qres->{ErrorCode}) &&
        $qres->{ErrorCode} != 0) {
        print "ErrorCode: " . $qres->{ErrorCode} . "\n";
        print "ErrorMessage: " . $qres->{ErrorMessage} . "\n";
    } else {
        $smsengine->{ObjectSms}->printqueryresponse($qres) if ($verbose);
    }

} else {

    die "ERROR: no message to send?\n$usage" if (!$msg);
    die "msg is $msg to is $to tofile is $tofile reqid is $reqid\n";

}


exit(0);

# test stuff below

#################################################################

# unit tests
if(0) {
    my $res = $smsengine->{ObjectSms}->sendsimplemessage( { text => "test text sendsimplemessage", 
				   to => "353861919191" } );
    $smsengine->{ObjectSms}->printsendresponse($res);
}

if(0) {
    my $res = $smsengine->{ObjectSms}->sendmessage( { text => "TUC test sendmessage 1 msg", 
			     to => [ "353861919191" ] } );
    $smsengine->{ObjectSms}->printsendresponse($res);

    my $qres = $smsengine->{ObjectSms}->querysenddeliverystatus( { RequestID => $res->{RequestID} } );
    $smsengine->{ObjectSms}->printqueryresponse($qres);

    exit(0);
}

if(0) {
# valid type not NOR
    my $res = $smsengine->{ObjectSms}->sendsimplemessage( { text => "test text sendsimplemessage", 
				   to => "353861919191",
				   Type => "FLA" } );
    $smsengine->{ObjectSms}->printsendresponse($res);
}

if(0) {
# invalid type
    my $res = $smsengine->{ObjectSms}->sendsimplemessage( { text => "test text sendsimplemessage", 
				   to => "353861919191",
				   Type => "etihs" } );
    $smsengine->{ObjectSms}->printsendresponse($res);
}

if(0) {
    my $res = $smsengine->{ObjectSms}->querysenddeliverystatus( { RequestID => 168618 } );
    $smsengine->{ObjectSms}->printqueryresponse($res);
}

if(0) {
    #<string>test nasty chars &lt;angles> `'"quotes"'` ({[brackets]}) &amp;#163; &amp;lt; ^</string>
    #my $res = $smsengine->{ObjectSms}->sendmessage( { text => "TUC test $smsengine->{ObjectSms}->sendmessage 0 msgs", 
#			     to => [ ] } );
    #$smsengine->{ObjectSms}->printsendresponse($res);

    my $res = $smsengine->{ObjectSms}->sendmessage( { text => "TUC test $smsengine->{ObjectSms}->sendmessage 1 msg", 
			     to => [ "353861919191" ] } );
    $smsengine->{ObjectSms}->printsendresponse($res);

    my $res2 = $smsengine->{ObjectSms}->sendmessage( { text => "TUC test $smsengine->{ObjectSms}->sendmessage 2 msgs", 
			     to => [ "353861919191", "123456789" ] } );
    $smsengine->{ObjectSms}->printsendresponse($res2);

}

if(0) {
    my $res = $smsengine->{ObjectSms}->sendsimplemessage( { text => "test text sendsimplemessage", 
				   to => "invalid number 353861919191 this won't work I hope." } );
    $smsengine->{ObjectSms}->printsendresponse($res);

# interesting, "Success" errorcode 0 but Failed_MSISDN array
#<methodResponse>
#<params>
#<param><value><struct>
#<member><name>RequestID</name><value><i4>168143</i4></value></member>
#<member><name>ErrorCode</name><value><i4>0</i4></value></member>
#<member><name>ErrorMessage</name><value><string>Success</string></value></member>
#<member><name>Failed_MSISDN</name><value><array><data>
#<value><struct>
#<member><name>errorcode</name><value><string>202</string></value>
#</member>
#<member><name>num</name><value><string>invalid number 353861919191 this won't work I hope.</string></value>
#</member>
#</struct></value>
#</data></array></value>
#</member>
#</struct></value>
#</param>
#</params>
#</methodResponse>

}

# how transmit a pound sign ?
# a pound sign (or other chars >127 cause message encoded base64 which prolly not good ?
# XMLRPC::Lite encodes & and < as &amp; and &lt;
#   so &#163; becomes &amp;#163; GAH!  TODO figure this out as could be relevant later
if(0) {
    my $res = $smsengine->{ObjectSms}->sendsimplemessage( { text => "test nasty chars <angles> `'\"quotes\"'` ({[brackets]}) £ ^", 
				   to => "353861919191" } );
    $smsengine->{ObjectSms}->printsendresponse($res);
}
if(0) {
    #<string>test nasty chars &lt;angles> `'"quotes"'` ({[brackets]}) &amp;#163; &amp;lt; ^</string>
    my $res = $smsengine->{ObjectSms}->sendsimplemessage( { text => "test nasty chars <angles> `'\"quotes\"'` ({[brackets]}) &#163; &lt; ^", 
				   to => "353861919191" } );
    $smsengine->{ObjectSms}->printsendresponse($res);
}


#51101 (Standard rate)
#50308 (Free text)
# test texts to us - careful of infinate loops!
# TODO: disallow texting our own shortcode

if(0) {
    #<string>test nasty chars &lt;angles> `'"quotes"'` ({[brackets]}) &amp;#163; &amp;lt; ^</string>
    my $res = $smsengine->{ObjectSms}->sendsimplemessage( { text => "TUC test text to own shortcode 51101", 
				   to => "51101" } );
    $smsengine->{ObjectSms}->printsendresponse($res);
}
if(0) {
    #<string>test nasty chars &lt;angles> `'"quotes"'` ({[brackets]}) &amp;#163; &amp;lt; ^</string>
    my $res = $smsengine->{ObjectSms}->sendsimplemessage( { text => "TUC test text to own shortcode 50308", 
				   to => "50308" } );
    $smsengine->{ObjectSms}->printsendresponse($res);
}


if(0) {
    my $res = $smsengine->{ObjectSms}->querysenddeliverystatus( { RequestID => 168618 } );
    $smsengine->{ObjectSms}->printqueryresponse($res);
}

if(0) {
    #<string>test nasty chars &lt;angles> `'"quotes"'` ({[brackets]}) &amp;#163; &amp;lt; ^</string>
    #my $res = $smsengine->{ObjectSms}->sendmessage( { text => "TUC test sendmessage 0 msgs", 
#			     to => [ ] } );
    #$smsengine->{ObjectSms}->printsendresponse($res);

    my $res = $smsengine->{ObjectSms}->sendmessage( { text => "TUC test sendmessage 1 msg", 
			     to => [ "353861919191" ] } );
    $smsengine->{ObjectSms}->printsendresponse($res);

    my $res2 = $smsengine->{ObjectSms}->sendmessage( { text => "TUC test sendmessage 2 msgs", 
			     to => [ "353861919191", "123456789" ] } );
    $smsengine->{ObjectSms}->printsendresponse($res2);

}

#$smsengine->{ObjectSms}->sendmessage
#$smsengine->{ObjectSms}->sendsimplemessage
#sendsimplemtbill
#sendbatchmessage
#$smsengine->{ObjectSms}->querysenddeliverystatus
#querybatchrequest
#queryaccountrequest


=head1 NAME

o2xmlrpcout.pl - xml-rpc https interface to o2 sms for sending sms

=head1 SYNOPSIS

perl o2xmlrpcout.pl

=head1 DESCRIPTION

 uses SmsEngine.pm and O2ProprietarySms.pm
 sends xmlrpc call $smsengine->{ObjectSms}->sendmessage to o2 test server
 link https with certs & keys
 receives reply and parses out data

 use -q query option to follow up status of messages until all are sent

 O2 SMS push api specifies maximum of 100 messages for sendmessage.
 In practice messages with more than 100 numbers are replied to with error.
 Use batch sms scripts if more than 100 messages are required

=head1 COPYRIGHT

Copyright (C) 2004 Doolin Technologies

=cut
