#!/usr/bin/perl -w

#use Time::HiRes qw(time);

use strict;
use bytes;
use Queue::Dir;
use Storable;
use Storable qw(store_fd fd_retrieve);

my $usage = "usage: $0 [-d] [-s storetestnum] [-r itemremovenum [-r otheritemremovenum] ] [queuedir]\n";
my ($debug, $rootdir, $store, $remove, $removeend);

while (defined($ARGV[0])) {
    if ($ARGV[0] =~ "-d") {
	$debug = shift(@ARGV);
    } elsif ($ARGV[0] =~ "-s") {
	shift(@ARGV);
	$store = shift(@ARGV);
    } elsif ($ARGV[0] =~ "-r") {
	shift(@ARGV);
	if (!defined($remove)) {
	    $removeend = $remove = shift(@ARGV);
	} else {
	    $removeend = shift(@ARGV);
	}
    } elsif (!defined($rootdir)) {
	$rootdir= shift(@ARGV);
    } else {
	die "unknown param " . shift(@ARGV) . "\n".$usage;
    }

}

if (!defined($rootdir)) {
    $rootdir = "/tmp/smsoutQ";
}

mkdir $rootdir if (!-d $rootdir);
die "can't create queue directory $rootdir" if (!-d $rootdir);

my $q = new Queue::Dir  
    id => "smsoutQ",
    paths => [ $rootdir ],
    promiscuous => 1,
    #sort => 'sortsub',
    #filter => sub { ... },
    lockdir => 'lock',
    #lockmax => 300, # default 300 sec anyway
    ;

die "fail to create Q" if (!defined $q);

$q->unlock;
$q = undef;

if (defined($store)) {

#????The 'paths' parameter ("ARRAY(0x804c8d4)") to Queue::Dir::new did not pass the 'directory' callback

    $q = new Queue::Dir  
	id => "smsoutQ",
	paths => [ $rootdir ],
	promiscuous => 1,
	lockdir => 'lock',
	;

    die "fail to create Q" if (!defined $q);

    my %oid = ( 'to' => "353867777777", 
		'text' => "hello foo",
		'time' => 0,
		'id' => 0 );

    my $id = 0;

    while ($id < $store) {
	$oid{time} = time();
	$oid{id} = $id++;

	print Dumper(%oid);

	my ($fh, $qid) = $q->store();
	die "fail to write to Q" if (!defined($qid) || !defined($fh));
	#(ref $fh eq 'IO::File', "Proper fh");
	#print $fh "Hello World $id\n";
	store_fd \%oid, $fh;

	print "id $id, qid $qid, name:" . $q->name($qid) . "\n";
	$fh->close;
	$q->unlock;

	$qid = $fh = undef; # delete things

    }

    $q = undef; # delete things

} else {

    my $q = new Queue::Dir id => 'smsoutQ', paths => [ $rootdir ], lockdir => 'lock';
    die "fail to open Q" if (!defined $q);

    my $text;

    use Data::Dumper;

    #### realising things
    ### we retrieve all qids in order enev from locked things
    if (0) {
	my $count=0;
	my $qid = $q->next();
	while (defined($qid)) {
	    print "qid $qid\n  name:" . $q->name($qid) ."\n";
	    $qid = $q->next();
	}
	print "\n\n\n";
    }

    #### realising things
    ### THE LAST $qid we get does not necessarily match what we get when we do a visit 
    ### Dir.pm sub visit does 
    ###     my $qid	= shift || $self->{_current}->[0] || $self->next;

    ### we retrieve all qids in order enev from locked things
    my $count=0;
    my $qid = $q->next();
    my $fh = $q->visit(undef,$qid); # the last $qid get doesn't match the visit 

    # TODO: test $fh, is it a valid file descriptor?

    while (defined($qid) && defined($fh)) {
	$count++;
	my $ref = fd_retrieve $fh;
	print "\nitem $count QID $qid\n  name:" . $q->name($qid) ."\n";
        #print "  file name: " . $q->name ."\n";
        #print "  id: " . $q->{_current}->[0] ."\n";
	print "  fh is " . (defined($fh)?"defined":"not defined");
	print Dumper($ref);

	$q->unlock;
	$qid = $q->next();
	$fh = $q->visit(undef,$qid) if (defined($qid)); 
        # do not call visit if not defined qid so as not to skip over last entry in q
    }
    $q->unlock;

    print "qid $qid, file handle failed.\n" 
	if (defined($qid) && !defined($fh));

    print "$count things in Q\n";

    #### realising things
    ### we retrieve all qids in order enev from locked things
    ## but 
    if (0) {
	print "\n\n\n";
	my $count=0;
	$qid = $q->next;   # next read => restart of q
	my $fh = $q->visit(); # MUST pass $qid? (NO)
	while (defined($qid) && defined($fh)) {
	    $count++;
	    my $ref = fd_retrieve $fh;
	    print "qid $qid\n  name:" . $q->name($qid) . "\n  text: ".$ref->{text}."\n";
	    #print Dumper($ref);
	    $q->unlock;
	    $qid = $q->next();
	    $fh = $q->visit() if (defined($qid));
	    # do not call visit if not defined qid so as not to skip over last entry in q
	}
	$q->unlock;
	print "\n$count things in Q\n\n";
	print "\n\n\n";
    }

    #### realising things
    ### we retrieve all qids in order enev from locked things
    if (0) {
	my $count=0;
	my $qid = $q->next();
	while (defined($qid)) {
	    my $fh = $q->visit("r",$qid);
	    #$text = $fh->getline;
	    my $ref = fd_retrieve $fh;
	    
	    #print "qid $qid, name:" . $q->name($qid) . ", text: $text\n";
	    print "qid $qid\n  name:" . $q->name($qid) . "\n  text: ".$ref->{text}."\n";
	    #print Dumper($ref);
	    
	    if (defined($remove) && $count >= $remove && $count <= $removeend) { # if we handle q entry
		# dispose of qid entry in q file
		my $status = $q->done($qid);
		print "item rm from queue $status\n";
	    }

	    $q->unlock;
	    $qid = $q->next();
	    $count++;
	}

	print "\n$count things in Q\n\n";

	$qid = $q->next();
	print "back to start of Q? qid $qid, name:" . $q->name($qid) . "\n";
	$qid = $q->next();
	print "next qid $qid, name:" . $q->name($qid) . "\n";

	# no easy way to reset Q ?
	#$q->_refresh();
	# yeah, unless current is undef refresh doesn't reload q
	$q->_refresh();
	$qid = $q->next();
	print "back to start of Q via _refresh doesn't work? qid $qid, name:" . $q->name($qid) . "\n";
    }

    # in practice we will process Q entries one by one
    # we will never in middle of q wish to reset it I think
    # in practice in testing we reset when we wish to search whole queue
}

# dispose of qid entry in q file
#my $status = $q->done($qid);

#$q->unlock;








