#!/usr/bin/perl -w

use bytes;
use strict;
use SmsEngine::SmsFileQueue;

my $configfile = "smsengine.topupclub.conf";

my $usage = <<END;
usage: $0 [-d] [-v]
          [-c <configfile>]
          -m <msgfile] | <msg> 
          <to>

e.g. $0 -d -v "Hello from TUC" 353867456789 
e.g. $0 -d -v -m msgfile.txt -t tonumbers.txt [353867456789, ..] 

END
;

my ($verbose, $debug);
my ($msgfile, $msg, $tofile, $to);

my $i=0;
while (defined($ARGV[0])) {

    if ($ARGV[0] =~ "^-d") {
	$debug = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-v") {
	$verbose = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-m") {
	shift(@ARGV);
	$msgfile = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-t") {
	shift(@ARGV);
	$tofile = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-c") {
	shift(@ARGV);
	$configfile = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-") {
	die "unknown param " . shift(@ARGV) . "\n".$usage;
    } elsif (!defined($msg) && !defined($msgfile)) {
	$msg = shift(@ARGV);
    } else {
	$to->[$i++] = shift(@ARGV);
	warn "sms out to file queue multiple messages not tested\n" if ($i==2);
    }

}

my $smsfq = SmsEngine::SmsFileQueue->new( debug => 0,
					  file => $configfile,
					  queuename => "smsfilequeue");

if ($smsfq->{error}) {
    my $errs;
    foreach my $err(@{$smsfq->{errors}}) {
	print "$err\n";
	$errs .= "$err\n";
    }
    my $err = "cannot instantiate SmsFileQueue :( installation or config problem.\n$errs";
    die $err;
}

#$verbose = $smsfq->getparam("verbose") if (!defined($verbose));
#$debug = $smsfq->getparam("debug")if (!defined($debug));
my $basedir = $smsfq->getparam("basedir");


if (defined($tofile)) {
    open(TOFILE, "<$tofile") || die("ERROR: Couldn't open $tofile $!");
    #$i=0;
    while (<TOFILE>) {
	next if (/^\#/);  
	chomp($_);
	$to->[$i++]=qq($_);
	warn "sms out to file queue multiple messages not tested\n" if ($i==2);
	warn "sms o2 sms push api max 100 numbers limit exceeded\n" if ($i==100);
	#die "ERROR: o2 SMS push api specifies maximum of 100 messages for sendmessage\nUse batch sms scripts\n" if ($i>100);
    }   
    close(TOFILE);	
}

$tofile = "" if (!defined($tofile));
die "ERROR: no destination to which to send sms?\n$usage" if (!defined($to->[0]));
print "to $i numbers $tofile first num is " . $to->[0]."\n" if ($verbose);

if ($msgfile) {
    $msg = "";
    open(MSGFILE, "<$msgfile") || die("ERROR: Couldn't open $msgfile $!");
    while (<MSGFILE>) {
	$msg.=$_;
    }   
    close(MSGFILE);	
}

print "message $msg\n" if ($verbose);

if ($msg) {
    my $req = { text => qq($msg),
		#to => [ $to->[0] ],
		to => $to,
		OtherParam => "smsouttofilequeue" };

    # TODO batch or call other methods

    my $res = $smsfq->enqueue( $req );

    if ($res->{error} != 0) {
	my $err = "could happen if someone removes the Q directory". 
	    " or disk goes away or file system is full.\n".
	    $res->{errors}->[0] .
	    "This is TeRRible! Atrocious even! :(";
	warn $err;
    } else {
	print "sms queued in file.\n";
    }
}

