#!/usr/bin/perl -w

use bytes;
use strict;
use SmsEngine::SmsOutToSmsEngineDaemon;

my $configfile = "smsengine.topupclub.conf";

my $usage = <<END;
usage: $0 [-d] [-v]
          [-c <configfile>]
          -m <msgfile] | <msg> 
          <to>

e.g. $0 -d -v "Hello from TUC" 353867456789 
e.g. $0 -d -v -m msgfile.txt -t tonumbers.txt [353867456789, ..] 

usage: $0 [-d] [-v]
          -x <xml>

e.g. $0 -d -v -x "<SMS><request><mobile>1234</mobile><msg>TUC smsengine test-o2xmlrpcout.daemon invalid mobile number</msg></request></SMS>"
e.g. $0 -d -v -x "<SMS><batchrequest><mobile>3538614xxxxx</mobile><mobile>1234</mobile><msg>TUC smsengine batch test-o2xmlrpcout.daemon invalid mobile number</msg></batchrequest></SMS>"

END
;

my ($verbose, $debug);
my ($msgfile, $msg, $xmlmsg, $tofile, $to);

my $i=0;
while (defined($ARGV[0])) {

    if ($ARGV[0] =~ "^-d") {
	$debug = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-v") {
	$verbose = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-m") {
	shift(@ARGV);
	$msgfile = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-x") {
	shift(@ARGV);
	$xmlmsg = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-t") {
	shift(@ARGV);
	$tofile = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-c") {
	shift(@ARGV);
	$configfile = shift(@ARGV);
    } elsif ($ARGV[0] =~ "^-") {
	die "unknown param " . shift(@ARGV) . "\n".$usage;
    } elsif (!defined($msg) && !defined($msgfile)) {
	$msg = shift(@ARGV);
    } else {
	$to->[$i++] = shift(@ARGV);
    }
}


my $smsout = SmsEngine::SmsOutToSmsEngineDaemon->new( debug => 0,
						      file => $configfile);

if ($smsout->{error}) {
    my $errs;
    foreach my $err(@{$smsout->{errors}}) {
	print "$err\n";
	$errs .= "$err\n";
    }
    my $err = "cannot instantiate SmsOutToEngineDaemon :( installation or config problem.\n$errs";
    die $err;
}

#$verbose = $smsout->getparam("verbose") if (!defined($verbose));
#$debug = $smsout->getparam("debug")if (!defined($debug));
my $basedir = $smsout->getparam("basedir");

if (defined($tofile)) {
    open(TOFILE, "<$tofile") || die("ERROR: Couldn't open $tofile $!");
    #$i=0;
    while (<TOFILE>) {
	next if (/^\#/);  
	chomp($_);
	$to->[$i++]=qq($_);
	warn "sms out multiple messages not tested\n" if ($i==2);
	warn "sms o2 sms push api max 100 numbers limit exceeded\n" if ($i==100);
	#die "ERROR: o2 SMS push api specifies maximum of 100 messages for sendmessage\nUse batch sms scripts\n" if ($i>100);
    }   
    close(TOFILE);	
}

print "to $tofile " . $to->[0] ."\n";
die "ERROR: no destination to which to send sms?\n$usage" 
    if (!defined($to->[0]) && !defined($xmlmsg));

if ($msgfile) {
    $msg = "";
    open(MSGFILE, "<$msgfile") || die("ERROR: Couldn't open $msgfile $!");
    while (<MSGFILE>) {
	$msg.=$_;
    }   
    close(MSGFILE);	
}

{
    my $message;
    if ($msgfile) {
	$message = $msg;
    } elsif ($xmlmsg) {
	$message = $xmlmsg;
    } else {
	$message = "<SMS><request>";
	#$message .= "<mobile>" . $to->[0] . "</mobile>";
	foreach my $j (@$to) { 
	    $message .= "<mobile>" . $j . "</mobile>";
	}
	$message .= "<msg>" . $msg . "</msg></request></SMS>";

    }

    my $res = $smsout->send($message);

    print STDERR "\nres is -$res-\n";
}

