#!/usr/bin/perl

use strict;
use bytes;

my $grepFileError = "";

sub grepFile
{
    my $file = shift; my $r = 1; my $str;
    while ($r && ($str = shift)) {
        my $res = `grep -c "$str" $file`;
        if ($res == "") { 
            $r = 0; 
            $grepFileError = "could not find $str in $file";
        }
    }
    return $r;
}

my $ti = 0;
my $tn = "";
sub registerTest
{
    $ti++;
    $tn = shift;
}

sub testResult
{
    my $r = shift;
    #print "TEST $ti $tn ".(($r)?"PASS":"FAIL")."\n";
    print (($r)?"ok $ti $tn\n":"not ok $ti $tn\n");
    if (!$r) { 
        print "Error: $grepFileError\n";
    }
}

my $usage = <<END;
usage: $0 [-d[ebug]] [-m <mobilenumber>] [<host> [<port>]]
e.g. $0 -d localhost 8987
sends incoming sms xml to o2xmlrpcin.daemon socket
verify response
don\'t use a random phone number. verify text replies may be sent out by specified number.
configure daemon to reply to a socket listening and logging tool for seperate test

END

my ($debug, $additional, $mtcode);
my ($host, $port, $mobilenumber, $message, $invalidmobilenumber);
$invalidmobilenumber = "35386xxxxxxx";

while (defined($ARGV[0])) {
    if ($ARGV[0] =~ "-h") {
	die $usage;
    } elsif ($ARGV[0] =~ "-d") {
	$debug = shift(@ARGV);
    } elsif ($ARGV[0] =~ "-m") {
	shift(@ARGV);
	$mobilenumber = shift(@ARGV);
    } elsif (!defined($host)) {
	$host = shift(@ARGV);
    } elsif (!defined($port)) {
	$port = shift(@ARGV);
    } else {
	die "unknown param? " . shift(@ARGV) . "\n".$usage;
    }
}

$mobilenumber = "invalid number" if !(defined($mobilenumber));

my $testpath = $0;
$testpath =~ s/\/[^\/]*$/\//; #filename whack
my $smsenginepath = $testpath;
#top dir whack (doesn't work if on cmdline called indirect dir)
$smsenginepath =~ s/\/[^\/]*\/$/\//; 
$smsenginepath = "${testpath}..\/";
my $o2xmlrpcinutil = $testpath . "o2xmlrpcin.pl";
die "need executable $o2xmlrpcinutil" if (! -x $o2xmlrpcinutil);
die "need logs dir in $smsenginepath" if (! -d "${smsenginepath}logs");

# Yo. This test requires a mobile number for YOU to manually verify
# that you receive sms.
#die "need to specify mobilenumber\n$usage"     
if (!defined($mobilenumber)){
    print "No mobile number specified on command-line.\n" . 
	"Only error tests will be performed.\n$usage";
}
$host = "localhost" if (!defined($host));
$port = 8987 if (!defined($port));

sub o2xmlrpcin {
    my $host = shift;
    my $port = shift;
    my $network = shift;
    my $shortcode = shift;
    my $number = shift;
    my $message = shift;
    #my $timeout = shift;
    #my $timeoutparam = "-t $timeout" if (defined($timeout));
    my $cmd = `$o2xmlrpcinutil -s $host -p $port "$number" "$message" $shortcode $network`;
    print "command: $cmd\n" if (defined($debug));
    my $res = `$cmd`;
    $_=$res;
    my ($id) =  m/\<id\>([0-9]*)\<\/id\>/;
    print "result: $res id: $id\n" if (defined($debug));
    return ($res, $id);
}

registerTest("receive nobody listening (test test)");
#500 Can't connect to localhost:8987 (connect: Connection refused)
my $network="O2I";
#my $shortcode = "353863833342"; # free I think
my $shortcode = "353863832032";
my $message = "receive nobody listening (test test)";
my ($res,$id) = o2xmlrpcin($host,45634,$network,$shortcode,$mobilenumber,$message);

#testResult($res =~ "500 Can't connect to localhost:" &&
#	   $res =~ "(connect: Connection refused)");
testResult($res eq "");


my $netstat = `netstat -al |grep 8987`;
die "daemon must be started up before test is running" 
    if ($netstat !~ "8987");
#my $cmd = "${smsenginepath}bin/o2xmlrpcin.daemon";
#system ("xterm -n o2xmlrpcin.daemon -e \"$cmd\" &");
#print "system: $listenservercmd" if (defined($debug));

`killall "listener.pl"`;
my $netstat = `netstat -al |grep 8151`;
die "something must not be running (on localhost:8151 maybe?) before test is running" 
    if ($netstat =~ "8151");
#die "socket listener/logger must be running (on localhost:8151 maybe?) before test is running" 
#    if ($netstat !~ "8151");

my $listenerresponse = "<SMS><response><mobile>12121212</mobile><msg>no. I don't feel like it.</msg></response></SMS>";
my $listenerlog = "${smsenginepath}logs/test-o2xmlrpcin.daemon.log";
my $listenservercmd = "${testpath}listener.pl -r \"</SMS>\" \"$listenerresponse\" -l $listenerlog 8151 -w 1";
sub runlistener {
    `killall "listener.pl"`;
    system "mv $listenerlog ${listenerlog}.old";
    print "system: $listenservercmd" if (defined($debug));
    system "$listenservercmd &";
}

runlistener();
registerTest("simple message in");
$message = "receive somebody listening (test test)";
my ($res,$id) = o2xmlrpcin($host,$port,$network,$shortcode,$mobilenumber,$message);
print "res: $res\nid: $id\n";

my $r = grepFile($listenerlog,
		 "<SMS><request>",
		 "<mobile>$mobilenumber</mobile>",
		 "<msg>$message</msg>",
		 "</request></SMS>");

testResult($r);


runlistener();
registerTest("simple message in, caught by filter, not forwarded");
$message = "receive no keyword match";
my $shortcode = "353863833342"; # free I think
my ($res,$id) = o2xmlrpcin($host,$port,$network,$shortcode,$mobilenumber,$message);
print "res: $res\nid: $id\n";

my $r = grepFile($listenerlog,
		 "<SMS><request>");
testResult(!$r);


runlistener();
registerTest("simple message in, caught by filter but okay, forwarded");
$message = "receive REGISTER and keyword match";
my $shortcode = "353863833342"; # free I think
my ($res,$id) = o2xmlrpcin($host,$port,$network,$shortcode,$mobilenumber,$message);
print "res: $res\nid: $id\n";

my $r = grepFile($listenerlog,
		 "<SMS><request>",
		 "<mobile>$mobilenumber</mobile>",
		 "<msg>$message</msg>",
		 "</request></SMS>");
testResult($r);


runlistener();
registerTest("NO INTERNET message in to free, caught by filter, forwarded");
$message = "TUC NO INTERNET free forward ";
my $shortcode = "353863833342"; # free I think
my ($res,$id) = o2xmlrpcin($host,$port,$network,$shortcode,$mobilenumber,$message);
print "res: $res\nid: $id\n";

my $r = grepFile($listenerlog,
		 "<SMS><request>",
		 "<mobile>$mobilenumber</mobile>",
		 "<msg>$message</msg>",
		 "</request></SMS>");
testResult($r);

runlistener();
registerTest("NO INTERNET message in to non-free, caught by filter, forwarded");
$message = "TUC NO INTERNET non-free forward ";
my $shortcode = "353863832032"; # non-free I think.
my ($res,$id) = o2xmlrpcin($host,$port,$network,$shortcode,$mobilenumber,$message);
print "res: $res\nid: $id\n";

my $r = grepFile($listenerlog,
		 "<SMS><request>",
		 "<mobile>$mobilenumber</mobile>",
		 "<msg>$message</msg>",
		 "</request></SMS>");
testResult($r);

runlistener();
registerTest("NO DOGS message in to free, caught by filter, blocked");
$message = "TUC NO DOGS free blocked";
my $shortcode = "353863833342"; # free I think
my ($res,$id) = o2xmlrpcin($host,$port,$network,$shortcode,$mobilenumber,$message);
print "res: $res\nid: $id\n";

my $r = grepFile($listenerlog,
		 "<SMS><request>");
testResult(!$r);

runlistener();
registerTest("NO DOGS message in to non-free, caught by filter, forwarded");
$message = "TUC NO DOGS non-free forward ";
my $shortcode = "353863832032"; # non-free I think.
my ($res,$id) = o2xmlrpcin($host,$port,$network,$shortcode,$mobilenumber,$message);
print "res: $res\nid: $id\n";

my $r = grepFile($listenerlog,
		 "<SMS><request>",
		 "<mobile>$mobilenumber</mobile>",
		 "<msg>$message</msg>",
		 "</request></SMS>");
testResult($r);



`killall "listener.pl"`;
