#!/usr/bin/perl

use strict;
use bytes;

my $ti = 0;
my $tn = "";
sub registerTest
{
    $ti++;
    $tn = shift;
}


sub testResult
{
    my $r = shift;
    #print "TEST $ti $tn ".(($r)?"PASS":"FAIL")."\n";
    print (($r)?"ok $ti $tn\n":"not ok $ti $tn\n");
    if (!$r) { 
        #print "Error: $grepFileError\n";
    }
}

my $usage = <<END;
usage: $0 [-d[ebug]] [-m <mobilenumber> [-a[dditional]] [-b[ig]] ] [<host> [<port>]]
e.g. $0 -m 353861111111 localhost 8152
e.g. $0 -m 353861111111 -a # all tests including "additional" == long text, large messages
e.g. $0 -m 353861111111 -b # big batch tests only
sends xml to o2xmlrpcout.daemon socket
verify response
don\'t use a random number. verify texts are received by specified number.
If you give a -m <mobilenumber> 2 texts are sent (one normal, one part of batch)
+some error case tests
If you give a -m <mobilenumber> and -a[dditional] option extra 3 texts are sent
  long text 200 chars
  one part of 100 number normal message
  one part of 500 number batch
  (one message with 19999 bad numbers, 1 good num (over quota, 500000 bytes + 
    over network to o2))
+some error case tests
If you give a -c an mtcode (premium rate) message is sent

-m nnn -a => total 2 + 3 texts

END

my ($debug, $additional, $mtcode, $bigonly);
my ($host, $port, $mobilenumber, $message, $invalidmobilenumber);
$invalidmobilenumber = "35386xxxxxxx";

while (defined($ARGV[0])) {
    if ($ARGV[0] =~ "-h") {
	die $usage;
    } elsif ($ARGV[0] =~ "-d") {
	$debug = shift(@ARGV);
    } elsif ($ARGV[0] =~ "-c") {
	$mtcode = shift(@ARGV);
    } elsif ($ARGV[0] =~ "-a") {
	$additional = shift(@ARGV);
	die "-a option needs mobile number -m option before\n".$usage 
	    if (!defined($mobilenumber));
    } elsif ($ARGV[0] =~ "-b") {
	$bigonly = shift(@ARGV);
	die "-b option needs mobile number -m option before\n".$usage 
	    if (!defined($mobilenumber));
    } elsif ($ARGV[0] =~ "-m") {
	shift(@ARGV);
	$mobilenumber = shift(@ARGV);
    } elsif (!defined($host)) {
	$host = shift(@ARGV);
    } elsif (!defined($port)) {
	$port = shift(@ARGV);
    } else {
	die "unknown param? " . shift(@ARGV) . "\n".$usage;
    }
}

my $testpath = $0;
$testpath =~ s/\/[^\/]*$/\//; #filename whack
my $testsockutil = $testpath . "testsock.pl";
die "need executable $testsockutil" if (! -x $testsockutil);
my $testsockmsgfile = $testpath . "testsockmsgfile.xml";

# Yo. This test requires a mobile number for YOU to manually verify
# that you receive sms.
#die "need to specify mobilenumber\n$usage"     
if (!defined($mobilenumber)){
    print "No mobile number specified on command-line.\n" . 
	"Only error tests will be performed.\n$usage";
}
$host = "localhost" if (!defined($host));
$port = 8152 if (!defined($port));


sub testsock {
   my $message = shift;
   my $timeout = shift;
   my $timeoutparam = "-t $timeout" if (defined($timeout));

   # must write very larges to file, otherwise get corrupted on cmd line
   #my $cmd = "$testsockutil $timeoutparam $host $port \"$message\"";
   my $cmd = "$testsockutil $timeoutparam $host $port -f $testsockmsgfile";
   open(OUT,">$testsockmsgfile") || die "can't open $testsockmsgfile for write $!";
   print OUT $message;
   close (OUT);

   print "command: $cmd\n" if (defined($debug));
   my $res = `$cmd`;
   $_=$res;
   my ($id) =  m/\<id\>([^<>]*)\<\/id\>/;
   print "result: $res id: $id\n" if (defined($debug));
   return ($res, $id);
}

goto bigonlytests if (defined($bigonly));

#<SMS><request><mobile>35386badnumber</mobile><msg>TUC smsengine tuc interface test bad num</msg></request></SMS>
#<SMS><response><errorcode>0</errorcode><msg>Success</msg><id>49969740</id><mobileerror><mobile>35386badnumber</mobile><faultcode>202</faultcode><msg></msg></mobileerror></response></SMS>

registerTest("Can we send (invalid mobile number)?");
$message = "<SMS><request><mobile>$invalidmobilenumber</mobile><msg>TUC smsengine test-o2xmlrpcout.daemon invalid mobile number</msg></request></SMS>";
my ($res,$id) = testsock($message);
testResult($res =~ "<SMS><response><errorcode>0</errorcode><msg>Success</msg>" &&
	   $res =~ "<mobileerror><mobile>$invalidmobilenumber</mobile><faultcode>202</faultcode>" && 
	   $res =~ "</mobileerror></response></SMS>" && 
	   $id > 0);


if (defined($mobilenumber)) {

#<SMS><request><mobile>35386xxxxxxx</mobile><msg>TUC smsengine tuc interface test</msg></request></SMS>
#<SMS><response><errorcode>0</errorcode><msg>Success</msg><id>49964446</id></response></SMS>

    registerTest("Can we send (good mobile number)?");
    $message = "<SMS><request><mobile>$mobilenumber</mobile><msg>TUC smsengine test-o2xmlrpcout.daemon good mobile number</msg></request></SMS>";
    my ($res,$id) = testsock($message);
    testResult($res =~ "<SMS><response><errorcode>0</errorcode><msg>Success</msg>" &&
	       $res =~ "</response></SMS>" && 
	       $id > 0);

    registerTest("query status");
    $message = "<SMS><query><id>$id</id></query></SMS>";
    my ($res,$notid) = testsock($message);
    testResult($res =~ "<SMS><response>" &&
	       $res =~ "<TotalNumberOfMessages>1</TotalNumberOfMessages>" && 
	       $res =~ "<MessagesSent>[01]</MessagesSent>" &&
	       $res =~ "<MessagesSkipped>0</MessagesSkipped>" &&
	       $res =~ "<StartDate>[0-9]+</StartDate>" &&
	       $res =~ "<Message>TUC smsengine test-o2xmlrpcout.daemon good mobile number</Message>" &&
	       $res =~ "<Status>[0-5]</Status>" &&
	       $res =~ "<StatusText>.*</StatusText>" &&
	       $res =~ "<ErrorCode>0</ErrorCode><ErrorMessage>Success</ErrorMessage>" && 
	       $res =~ "</response></SMS>"
	       # id for query message is not set
	       );


#<SMS><request><mobile>35386xxxxxxx</mobile><msg>TUC smsengine tuc interface test bad mtcode and good num</msg><mtcode>bad mtcode</mtcode></request></SMS>
#<SMS><response><errorcode>69</errorcode><msg>ERR API069: MTCode specified is not permitted for this Account</msg><id></id></response></SMS>

    registerTest("send with bad mtcode");
    $message = "<SMS><request><mobile>$mobilenumber</mobile><msg>TUC smsengine test-o2xmlrpcout.daemon good mobile number, bad mtcode</msg><mtcode>bad mtcode</mtcode></request></SMS>";
    my ($res,$id) = testsock($message);
    testResult($res =~ "<SMS><response><errorcode>69</errorcode>" && 
	       $res =~ "<msg>ERR API069: MTCode specified is not permitted for this Account</msg>" &&
	       $res =~ "<id></id>" && 
	       $res =~ "</response></SMS>");


#<SMS><request><mobile>35386xxxxxxx</mobile><msg>TUC good mtcode and good num</msg><mtcode>alphyra.ie/pull_51101_0c</mtcode></request></SMS>
#<SMS><response><errorcode>0</errorcode><msg>Success</msg><id>49966804</id></response></SMS>

# I think mtcodes are more expensive and are not required now so I do not test
    if (defined($mtcode)) { 
	registerTest("send with good mtcode");
	$message = "<SMS><request><mobile>$mobilenumber</mobile><msg>TUC smsengine test-o2xmlrpcout.daemon good mobile number, good mtcode</msg><mtcode>alphyra.ie/pull_51101_0c</mtcode></request></SMS>";
	my ($res,$id) = testsock($message);
	testResult($res =~ "<SMS><response><errorcode>0</errorcode>" && 
		   $res =~ "<msg>Success</msg>" &&
		   $id > 0 && 
		   $res =~ "</response></SMS>");
    }

    if (defined($additional)) {
	registerTest("200 char message (split - long text)");
	$message = "<SMS><request><mobile>$mobilenumber</mobile><msg>00000000001111111111222222222233333333334444444444555555555566666666667777777777888888888899999999990000000000111111111122222222223333333333444444444455555555556666666666777777777788888888889999999999</msg></request></SMS>";
	my ($res,$id) = testsock($message);
	testResult($res =~ "<SMS><response><errorcode>0</errorcode><msg>Success</msg>" &&
		   $res =~ "</response></SMS>" && 
		   $id > 0);
    }

    if (defined($additional)) {
	registerTest("100 numbers non-batch text");
	$message = "<SMS><request>";
	for (my $i=0;$i<10;$i++){
	    $message .="<mobile>bad$i</mobile>";
	}
	$message .= "<mobile>$mobilenumber</mobile>";
	for (my $i=11;$i<100;$i++){
	    $message .="<mobile>bad$i</mobile>";
	}
	$message .= "<msg>100 numbers non-batch test 99 bad</msg></request></SMS>";

	my ($res,$id) = testsock($message);
	testResult($res =~ "<SMS><response><errorcode>0</errorcode><msg>Success</msg>" &&
		   $res =~ "<mobileerror><mobile>bad0</mobile><faultcode>202</faultcode><msg>Invalid format specified for num in MSISDN structure</msg></mobileerror>" && 
		   $res =~ "<mobileerror><mobile>bad99</mobile><faultcode>202</faultcode><msg>Invalid format specified for num in MSISDN structure</msg></mobileerror>" && 
		   $res =~ "</response></SMS>" && 
		   $id > 0);

	registerTest("query status 100 num test 99 bad");
	$message = "<SMS><query><id>$id</id></query></SMS>";
	my ($res,$notid) = testsock($message);
	testResult($res =~ "<SMS><response>" &&
		   $res =~ "<TotalNumberOfMessages>100</TotalNumberOfMessages>" && 
		   $res =~ "<MessagesSent>[01]</MessagesSent>" &&
		   $res =~ "<MessagesSkipped>[0-9]+</MessagesSkipped>" &&
		   $res =~ "<StartDate>[0-9]+</StartDate>" &&
		   $res =~ "<Status>[0-5]</Status>" &&
		   $res =~ "<StatusText>.*</StatusText>" &&
		   $res =~ "<ErrorCode>0</ErrorCode><ErrorMessage>Success</ErrorMessage>" && 
		   $res =~ "</response></SMS>"
		   # id for query message is not set
		   );

#101 numbers test 100 bad</msg></request></SMS>
#<SMS><response><errorcode>8</errorcode><msg>ERR API008: Invalid RequestSize</msg><id></id><batch>0</batch></response></SMS>
	registerTest("101 numbers non-batch text 100 bad");
	$message = "<SMS><request>";
	for (my $i=0;$i<90;$i++){
	    $message .="<mobile>bad$i</mobile>";
	}
	$message .= "<mobile>$mobilenumber</mobile>";
	for (my $i=91;$i<101;$i++){
	    $message .="<mobile>bad$i</mobile>";
	}
	$message .= "<msg>101 numbers non-batch test 100 bad</msg></request></SMS>";

	my ($res,$id) = testsock($message);
	testResult($res =~ "<SMS><response><errorcode>8</errorcode>" && 
		   $res =~ "<msg>ERR API008: Invalid RequestSize</msg>" &&
		   $res =~ "<id></id>" && 
		   $res =~ "</response></SMS>");

    }
}


registerTest("test socket timeout");
$message = "";
my ($res,$id) = testsock($message);
testResult($res eq "");

registerTest("test socket timeout need more");
$message = "<SMS>junk";
my ($res,$id) = testsock($message);
testResult($res eq "");

registerTest("test not enough info in xml requests 1");
$message = "<SMS></SMS>";
my ($res,$id) = testsock($message);
testResult($res =~ "<SMS><error><root></root><type></type><msg>no response. xml parse error\\\?</msg></error></SMS>");

registerTest("test not enough info in xml requests 2");
$message = "<SMS><notquitenothing></notquitenothing></SMS>";
my ($res,$id) = testsock($message);
testResult($res =~ "<SMS><error><root></root><type></type><msg>no response. xml parse error\\\?</msg></error></SMS>");

registerTest("test not enough info in xml requests 3");
$message = "<SMS><request></request></SMS>";
my ($res,$id) = testsock($message);
testResult($res =~ "Not enough parameters in request");

registerTest("test not enough info in xml requests 4");
$message = "<SMS><request><msg>no mobile</msg></request></SMS>";
my ($res,$id) = testsock($message);
testResult($res =~ "Not enough parameters in request");


# batch request & status query

if (defined($mobilenumber)) {

#<SMS><batchrequest><mobile>353861953134</mobile><mobile>bad num 1</mobile><mobile>bad num 2</mobile><msg>TUC 3 mobiles</msg></batchrequest></SMS>
#<SMS><response><errorcode>0</errorcode><msg>Success</msg><id>52838895</id><batch>1</batch><mobileerror><mobile>bad num 1</mobile><faultcode>202</faultcode><msg></msg></mobileerror><mobileerror><mobile>bad num 2</mobile><faultcode>202</faultcode><msg></msg></mobileerror></response></SMS>

    registerTest("batch messages (1 good mobile number, 2 bad)");
    $message = "<SMS><batchrequest><mobile>$mobilenumber</mobile><mobile>bad num 1</mobile><mobile>bad num 2</mobile><msg>TUC smsengine test-o2xmlrpcout.daemon batch 1 good number, 2 bad</msg></batchrequest></SMS>";
    my ($res,$id) = testsock($message);
    testResult($res =~ "<SMS><response><errorcode>0</errorcode><msg>Success</msg>" &&
	       $res =~ "<mobileerror><mobile>bad num 1</mobile><faultcode>202</faultcode>" && 
	       $res =~ "<mobileerror><mobile>bad num 2</mobile><faultcode>202</faultcode>" && 
	       $res =~ "</response></SMS>" && 
	       $id > 0);

    registerTest("batch query status, valid id");
    $message = "<SMS><batchquery><id>$id</id></batchquery></SMS>";
    my ($res,$notid) = testsock($message);
    testResult($res =~ "<SMS><response>" &&
	       $res =~ "<TotalNumberOfMessages>3</TotalNumberOfMessages>" && 
	       $res =~ "<MessagesSent>[01]</MessagesSent>" &&
	       $res =~ "<MessagesSkipped>[012]</MessagesSkipped>" &&
	       $res =~ "<StartDate>[0-9]+</StartDate>" &&
	       $res =~ "<Message>TUC smsengine test-o2xmlrpcout.daemon batch 1 good number, 2 bad</Message>" &&
	       $res =~ "<Status>[0-5]</Status>" &&
	       $res =~ "<StatusText>.*</StatusText>" &&
	       $res =~ "<ErrorCode>0</ErrorCode>" && 
	       $res =~ "<ErrorMessage>Success</ErrorMessage>" && 
	       $res =~ "</response></SMS>");

    if (defined($additional)) {

	registerTest("500 numbers batch");
	$message = "<SMS><batchrequest>";
	for (my $i=0;$i<200;$i++){
	    $message .="<mobile>bad$i</mobile>";
	}
	$message .= "<mobile>$mobilenumber</mobile>";
	for (my $i=201;$i<500;$i++){
	    $message .="<mobile>bad$i</mobile>";
	}
	$message .= "<msg>500 numbers batch test 499 bad</msg></batchrequest></SMS>";

	my ($res,$id) = testsock($message);
	testResult($res =~ "<SMS><response><errorcode>0</errorcode><msg>Success</msg>" &&
		   $res =~ "<mobileerror><mobile>bad0</mobile><faultcode>202</faultcode><msg>Invalid format specified for num in MSISDN structure</msg></mobileerror>" && 
		   $res =~ "<mobileerror><mobile>bad499</mobile><faultcode>202</faultcode><msg>Invalid format specified for num in MSISDN structure</msg></mobileerror>" && 
		   $res =~ "</response></SMS>" && 
		   $id > 0);

	registerTest("query status 500 num test 499 bad");
	$message = "<SMS><batchquery><id>$id</id></batchquery></SMS>";
	my ($res,$notid) = testsock($message);
	testResult($res =~ "<SMS><response>" &&
		   $res =~ "<TotalNumberOfMessages>500</TotalNumberOfMessages>" && 
		   $res =~ "<MessagesSent>[01]</MessagesSent>" &&
		   $res =~ "<MessagesSkipped>[0-9]+</MessagesSkipped>" &&
		   $res =~ "<StartDate>[0-9]+</StartDate>" &&
		   $res =~ "<Status>[0-5]</Status>" &&
		   $res =~ "<StatusText>.*</StatusText>" &&
		   $res =~ "<ErrorCode>0</ErrorCode><ErrorMessage>Success</ErrorMessage>" && 
		   $res =~ "</response></SMS>");

	if (0) { # don't know quota anymore? it has changed
	registerTest("501 numbers batch (over quota)");
	$message = "<SMS><batchrequest>";
	for (my $i=0;$i<400;$i++){
	    $message .="<mobile>bad$i</mobile>";
	}
	$message .= "<mobile>$mobilenumber</mobile>";
	for (my $i=401;$i<501;$i++){
	    $message .="<mobile>bad$i</mobile>";
	}
	$message .= "<msg>501 numbers batch test (over quota)</msg></batchrequest></SMS>";

	my ($res,$id) = testsock($message);
	testResult($res =~ "<SMS><response><errorcode>2</errorcode>" && 
		   $res =~ "<msg>ERR API002: Quota Exceeded</msg>" &&
		   $res =~ "<id>.*</id>" &&
		   $res =~ "</response></SMS>");
        }

bigonlytests:

	# also at message size >16xxx bytes on socket we had problem before
	# <mobile>badxxx</mobile> 23 chars, 20000 * 23 => 460000 bytes! whoo!
        # + a bit extra
	my $batchcount=20000;
	my $batchcount=7000;
	my $batchmid=int( $batchcount*5/6 );
	registerTest("$batchcount numbers batch (over msisdn max fail limit)");
	$message = "<SMS><batchrequest>";
	for (my $i=0;$i<$batchmid;$i++){
	    $message .="<mobile>bad$i</mobile>";
	}
	$message .= "<mobile>$mobilenumber</mobile>";
	for (my $i=$batchmid+1;$i<$batchcount;$i++){
	    $message .="<mobile>bad$i</mobile>";
	}
	$message .= "<msg>$batchcount numbers HUGE batch test exceed msisdn max fail limit</msg></batchrequest></SMS>";

	my ($res,$id) = testsock($message,my $timeout=60);
	print "response to $batchcount: $res\n";
	testResult($res =~ "<SMS><response><errorcode>72</errorcode>" && 
		   $res =~ "<msg>ERR API072: Max MSISDN fail limit exceeded</msg>" &&
		   $res =~ "<id>.*</id>" &&
		   $res =~ "</response></SMS>");

goto endfini if (defined($bigonly));

    }
}

registerTest("query status, invalid id");
$message = "<SMS><query><id>01010101</id></query></SMS>";
my ($res,$notid) = testsock($message);
testResult($res =~ "<SMS><response>" &&
	   $res =~ "<TotalNumberOfMessages></TotalNumberOfMessages>" && 
	   $res =~ "<MessagesSent></MessagesSent>" &&
	   $res =~ "<MessagesSkipped></MessagesSkipped>" &&
	   $res =~ "<StartDate></StartDate>" &&
	   $res =~ "<Message></Message>" &&
	   $res =~ "<Status></Status>" &&
	   $res =~ "<StatusText>\\\?</StatusText>" &&
	   $res =~ "<ErrorCode>50</ErrorCode>" && 
	   $res =~ "<ErrorMessage>ERR API050: Invalid Request ID</ErrorMessage>" && 
	   $res =~ "</response></SMS>"
	   );

registerTest("batch query status, invalid id");
$message = "<SMS><batchquery><id>01010101</id></batchquery></SMS>";
my ($res,$notid) = testsock($message);
testResult($res =~ "<SMS><response>" &&
	   $res =~ "<TotalNumberOfMessages></TotalNumberOfMessages>" && 
	   $res =~ "<MessagesSent></MessagesSent>" &&
	   $res =~ "<MessagesSkipped></MessagesSkipped>" &&
	   $res =~ "<StartDate></StartDate>" &&
	   $res =~ "<Message></Message>" &&
	   $res =~ "<Status></Status>" &&
	   $res =~ "<StatusText>\\\?</StatusText>" &&
	   $res =~ "<ErrorCode>50</ErrorCode>" && 
	   $res =~ "<ErrorMessage>ERR API050: Invalid Request ID</ErrorMessage>" && 
	   $res =~ "</response></SMS>"
	   );

endfini:
