#!/usr/bin/perl

#http://iis1.cps.unizar.es/Oreilly/perl/advprog/ch12_02.htm

use bytes;

use IO::Socket;

my $usage = <<END;
usage: $0 [-d] [-v] [-t timeout] host port ( messagetosend | -f filewithmessagetosend)

e.g. $0 -t 10 localhost 81152 "<SMS><request><mobile>bad number test</mobile><msg>TUC bad num</msg></request></SMS>"
e.g. $0 localhost 81152 -f ~/src/doolin/smsengine/test/msg1000bad1good.batchrequest

END

my ($debug, $verbose);
my ($host, $port, $msg, $msgfile, $timeout);

while (defined($ARGV[0])) {

    if ($ARGV[0] =~ "-d") {
	$debug = shift(@ARGV);
    } elsif ($ARGV[0] =~ "-v") {
	$verbose = shift(@ARGV);
    } elsif ($ARGV[0] =~ "-f") {
	shift(@ARGV);
	$msgfile = shift(@ARGV);
    } elsif ($ARGV[0] =~ "-t") {
	shift(@ARGV);
	$timeout = shift(@ARGV);
    } else {
	if (!defined($host)) {
	    $host = shift(@ARGV);
	} elsif (!defined($port)) {
	    $port = shift(@ARGV);
	} elsif (!defined($msg)) {
	    $msg = shift(@ARGV);
	} else {
	    die "unknown param? " . shift(@ARGV) . "\n".$usage;
	}
    }
}

die "need to specify host and port\n$usage" 
    if (!defined($host) && !defined($port));
die "need to specify message to send or message file\n$usage" 
    if (!defined($msgfile) && !defined($msg));
die "can't specify both message to send and message file\n$usage" 
    if (defined($msgfile) && defined($msg));

if (defined($msgfile)) {
    $msg = "";
    open(IN,"$msgfile") || die "can't open $msgfile $!";
    while(<IN>){
	$msg .= $_;
    }
    close (IN);
} 

$sock = new IO::Socket::INET (PeerAddr => $host,
                              PeerPort => $port,
                              Proto    => 'tcp'
                             );
die "Socket could not be created. Reason: $!\n" unless $sock;

# should always be defined
if(defined($msg)) {
    print "writing $msg to socket" if ($debug);
    print $sock $msg;
}

$timeout = 6 if (!defined($timeout));

my $start = time();
my $line;
my $now = $start;

while($now-$start<$timeout){
    $line = <$sock>;
    print "read: ".$line if ($line);
    $now=time();
}

close ($sock);

#code for the server that receive the data:
#!/usr/bin/perl
#use IO::Socket;
#
#$sock = new IO::Socket::INET (LocalHost => '172.16.40.11',
#                       LocalPort => 50010,
#                              Proto     => 'tcp',
#                              Listen    => 10,
#                              );
#die "Socket could not be created. Reason: $!" unless $sock;
#open(SAVAN,">savanback");
#while ($new_sock = $sock->accept()) {
#    while (defined ($buf = <$new_sock>)) {
#      print SAVAN $buf || die("$!");
#      print  $buf || die();      
#    }
#}
#close ($sock);
#close(SAVAN); 
